@@ -1846,13 +1846,6 @@ static HRESULT Date_setYear(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsi
     return S_OK;
 }
 
-static HRESULT Date_get_value(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
-{
-    TRACE("\n");
-
-    return dateobj_to_string(date_from_jsdisp(jsthis), r);
-}
-
 static const builtin_prop_t Date_props[] = {
     {L"getDate",             Date_getDate,               PROPF_METHOD},
     {L"getDay",              Date_getDay,                PROPF_METHOD},
@@ -1903,7 +1896,7 @@ static const builtin_prop_t Date_props[] = {
 
 static const builtin_info_t Date_info = {
     JSCLASS_DATE,
-    {NULL, NULL,0, Date_get_value},
+    NULL,
     ARRAY_SIZE(Date_props),
     Date_props,
     NULL,
@@ -1912,13 +1905,13 @@ static const builtin_info_t Date_info = {
 
 static const builtin_info_t DateInst_info = {
     JSCLASS_DATE,
-    {NULL, NULL,0, Date_get_value},
+    NULL,
     0, NULL,
     NULL,
     NULL
 };
 
-static HRESULT create_date(script_ctx_t *ctx, jsdisp_t *object_prototype, DOUBLE time, jsdisp_t **ret)
+static HRESULT create_date(script_ctx_t *ctx, jsdisp_t *object_prototype, DOUBLE time, DateInstance **ret)
 {
     DateInstance *date;
     HRESULT hres;
@@ -1946,7 +1939,7 @@ static HRESULT create_date(script_ctx_t *ctx, jsdisp_t *object_prototype, DOUBLE
     date->daylightDate = tzi.DaylightDate;
     date->daylightBias = tzi.DaylightBias;
 
-    *ret = &date->dispex;
+    *ret = date;
     return S_OK;
 }
 
@@ -2354,7 +2347,7 @@ static HRESULT DateConstr_now(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
 static HRESULT DateConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
-    jsdisp_t *date;
+    DateInstance *date;
     HRESULT hres;
 
     TRACE("\n");
@@ -2396,7 +2389,6 @@ static HRESULT DateConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
         /* ECMA-262 3rd Edition    15.9.3.1 */
         default: {
             double ret_date;
-            DateInstance *di;
 
             hres = date_utc(ctx, argc, argv, &ret_date);
             if(FAILED(hres))
@@ -2406,12 +2398,11 @@ static HRESULT DateConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
             if(FAILED(hres))
                 return hres;
 
-            di = date_from_jsdisp(date);
-            di->time = utc(di->time, di);
+            date->time = utc(date->time, date);
         }
         }
 
-        *r = jsval_obj(date);
+        *r = jsval_obj(&date->dispex);
         return S_OK;
 
     case INVOKE_FUNC: {
@@ -2442,7 +2433,7 @@ static const builtin_prop_t DateConstr_props[] = {
 
 static const builtin_info_t DateConstr_info = {
     JSCLASS_FUNCTION,
-    DEFAULT_FUNCTION_VALUE,
+    Function_value,
     ARRAY_SIZE(DateConstr_props),
     DateConstr_props,
     NULL,
@@ -2451,7 +2442,7 @@ static const builtin_info_t DateConstr_info = {
 
 HRESULT create_date_constr(script_ctx_t *ctx, jsdisp_t *object_prototype, jsdisp_t **ret)
 {
-    jsdisp_t *date;
+    DateInstance *date;
     HRESULT hres;
 
     hres = create_date(ctx, object_prototype, 0.0, &date);
@@ -2459,8 +2450,54 @@ HRESULT create_date_constr(script_ctx_t *ctx, jsdisp_t *object_prototype, jsdisp
         return hres;
 
     hres = create_builtin_constructor(ctx, DateConstr_value, L"Date", &DateConstr_info,
-            PROPF_CONSTR|7, date, ret);
+            PROPF_CONSTR|7, &date->dispex, ret);
 
-    jsdisp_release(date);
+    jsdisp_release(&date->dispex);
+    return hres;
+}
+
+HRESULT variant_date_to_number(double date, double *ret)
+{
+    SYSTEMTIME st;
+    UDATE udate;
+    HRESULT hres;
+
+    hres = VarUdateFromDate(date, 0, &udate);
+    if(FAILED(hres))
+        return hres;
+
+    if(!TzSpecificLocalTimeToSystemTime(NULL, &udate.st, &st))
+        return E_FAIL;
+
+    TRACE("%uy %um %u %ud %uh %um %u.%us\n", st.wYear, st.wMonth, st.wDayOfWeek, st.wDay, st.wHour, st.wMinute,
+        st.wSecond, st.wMilliseconds);
+
+    *ret = make_date(make_day(st.wYear, st.wMonth - 1, st.wDay),
+                     make_time(st.wHour, st.wMinute, st.wSecond, st.wMilliseconds));
+    return S_OK;
+}
+
+HRESULT variant_date_to_string(script_ctx_t *ctx, double date, jsstr_t **r)
+{
+    DateInstance *date_obj;
+    jsval_t val;
+    double time;
+    HRESULT hres;
+
+    hres = variant_date_to_number(date, &time);
+    if(FAILED(hres))
+        return hres;
+
+    hres = create_date(ctx, NULL, time, &date_obj);
+    if(FAILED(hres))
+        return hres;
+
+    hres = dateobj_to_string(date_obj, &val);
+    jsdisp_release(&date_obj->dispex);
+    if(FAILED(hres))
+        return hres;
+
+    assert(is_string(val));
+    *r = get_string(val);
     return hres;
 }
