@@ -1346,6 +1346,15 @@ static void CALLBACK signaled_function(PVOID p, BOOLEAN TimerOrWaitFired)
     ok(!TimerOrWaitFired, "wait shouldn't have timed out\n");
 }
 
+static void CALLBACK wait_complete_function(PVOID p, BOOLEAN TimerOrWaitFired)
+{
+    HANDLE event = p;
+    DWORD res;
+    ok(!TimerOrWaitFired, "wait shouldn't have timed out\n");
+    res = WaitForSingleObject(event, INFINITE);
+    ok(res == WAIT_OBJECT_0, "WaitForSingleObject returned %x\n", res);
+}
+
 static void CALLBACK timeout_function(PVOID p, BOOLEAN TimerOrWaitFired)
 {
     HANDLE event = p;
@@ -1367,11 +1376,27 @@ static void CALLBACK waitthread_test_function(PVOID p, BOOLEAN TimerOrWaitFired)
 
     SetEvent(param->trigger_event);
     ret = WaitForSingleObject(param->wait_event, 100);
-    todo_wine
     ok(ret == WAIT_TIMEOUT, "wait should have timed out\n");
     SetEvent(param->complete_event);
 }
 
+struct unregister_params
+{
+    HANDLE wait_handle;
+    HANDLE complete_event;
+};
+
+static void CALLBACK unregister_function(PVOID p, BOOLEAN TimerOrWaitFired)
+{
+    struct unregister_params *param = p;
+    HANDLE wait_handle = param->wait_handle;
+    BOOL ret;
+    ok(wait_handle != INVALID_HANDLE_VALUE, "invalid wait handle\n");
+    ret = pUnregisterWait(param->wait_handle);
+    todo_wine ok(ret, "UnregisterWait failed with error %d\n", GetLastError());
+    SetEvent(param->complete_event);
+}
+
 static void test_RegisterWaitForSingleObject(void)
 {
     BOOL ret;
@@ -1380,6 +1405,8 @@ static void test_RegisterWaitForSingleObject(void)
     HANDLE complete_event;
     HANDLE waitthread_trigger_event, waitthread_wait_event;
     struct waitthread_test_param param;
+    struct unregister_params unregister_param;
+    DWORD i;
 
     if (!pRegisterWaitForSingleObject || !pUnregisterWait)
     {
@@ -1412,8 +1439,26 @@ static void test_RegisterWaitForSingleObject(void)
     ret = pUnregisterWait(wait_handle);
     ok(ret, "UnregisterWait failed with error %d\n", GetLastError());
 
+    /* test unregister while running */
+
+    SetEvent(handle);
+    ret = pRegisterWaitForSingleObject(&wait_handle, handle, wait_complete_function, complete_event, INFINITE, WT_EXECUTEONLYONCE);
+    ok(ret, "RegisterWaitForSingleObject failed with error %d\n", GetLastError());
+
+    /* give worker thread chance to start */
+    Sleep(50);
+    ret = pUnregisterWait(wait_handle);
+    ok(!ret, "UnregisterWait succeeded\n");
+    ok(GetLastError() == ERROR_IO_PENDING, "UnregisterWait failed with error %d\n", GetLastError());
+
+    /* give worker thread chance to complete */
+    SetEvent(complete_event);
+    Sleep(50);
+
     /* test timeout case */
 
+    ResetEvent(handle);
+
     ret = pRegisterWaitForSingleObject(&wait_handle, handle, timeout_function, complete_event, 0, WT_EXECUTEONLYONCE);
     ok(ret, "RegisterWaitForSingleObject failed with error %d\n", GetLastError());
 
@@ -1443,6 +1488,21 @@ static void test_RegisterWaitForSingleObject(void)
     ret = pUnregisterWait(wait_handle);
     ok(ret, "UnregisterWait failed with error %d\n", GetLastError());
 
+    /* the callback execution should be sequentially consistent with the wait handle return,
+       even if the event is already set */
+
+    for (i = 0; i < 100; ++i)
+    {
+        SetEvent(handle);
+        unregister_param.complete_event = complete_event;
+        unregister_param.wait_handle = INVALID_HANDLE_VALUE;
+
+        ret = pRegisterWaitForSingleObject(&unregister_param.wait_handle, handle, unregister_function, &unregister_param, INFINITE, WT_EXECUTEONLYONCE | WT_EXECUTEINWAITTHREAD);
+        ok(ret, "RegisterWaitForSingleObject failed with error %d\n", GetLastError());
+
+        WaitForSingleObject(complete_event, INFINITE);
+    }
+
     /* test multiple waits with WT_EXECUTEINWAITTHREAD.
      * Windows puts multiple waits on the same wait thread, and using WT_EXECUTEINWAITTHREAD causes the callbacks to run serially.
      */
@@ -2293,7 +2353,7 @@ todo_wine
 
 static void test_thread_description(void)
 {
-    THREAD_DESCRIPTION_INFORMATION *thread_desc;
+    THREAD_NAME_INFORMATION *thread_desc;
     static const WCHAR *desc = L"thread_desc";
     ULONG len, len2, desc_len;
     NTSTATUS status;
@@ -2308,7 +2368,7 @@ static void test_thread_description(void)
     }
 
     desc_len = lstrlenW(desc) * sizeof(*desc);
-    thread_desc = (THREAD_DESCRIPTION_INFORMATION *)buff;
+    thread_desc = (THREAD_NAME_INFORMATION *)buff;
 
     /* Initial description. */
     ptr = NULL;
@@ -2318,20 +2378,20 @@ static void test_thread_description(void)
     LocalFree(ptr);
 
     len = 0;
-    status = pNtQueryInformationThread(GetCurrentThread(), ThreadDescription, NULL, 0, &len);
+    status = pNtQueryInformationThread(GetCurrentThread(), ThreadNameInformation, NULL, 0, &len);
     ok(status == STATUS_BUFFER_TOO_SMALL, "Unexpected status %#x.\n", status);
     ok(len == sizeof(*thread_desc), "Unexpected structure length %u.\n", len);
 
     len2 = 0;
-    thread_desc->Description.Length = 1;
-    thread_desc->Description.MaximumLength = 0;
-    thread_desc->Description.Buffer = (WCHAR *)thread_desc;
-    status = pNtQueryInformationThread(GetCurrentThread(), ThreadDescription, thread_desc, len, &len2);
+    thread_desc->ThreadName.Length = 1;
+    thread_desc->ThreadName.MaximumLength = 0;
+    thread_desc->ThreadName.Buffer = (WCHAR *)thread_desc;
+    status = pNtQueryInformationThread(GetCurrentThread(), ThreadNameInformation, thread_desc, len, &len2);
     ok(!status, "Failed to get thread info, status %#x.\n", status);
     ok(len2 == sizeof(*thread_desc), "Unexpected structure length %u.\n", len);
-    ok(!thread_desc->Description.Length, "Unexpected description length %#x.\n", thread_desc->Description.Length);
-    ok(thread_desc->Description.Buffer == (WCHAR *)(thread_desc + 1),
-            "Unexpected description string pointer %p, %p.\n", thread_desc->Description.Buffer, thread_desc);
+    ok(!thread_desc->ThreadName.Length, "Unexpected description length %#x.\n", thread_desc->ThreadName.Length);
+    ok(thread_desc->ThreadName.Buffer == (WCHAR *)(thread_desc + 1),
+            "Unexpected description string pointer %p, %p.\n", thread_desc->ThreadName.Buffer, thread_desc);
 
     hr = pSetThreadDescription(GetCurrentThread(), NULL);
     ok(hr == HRESULT_FROM_NT(STATUS_SUCCESS), "Failed to set thread description, hr %#x.\n", hr);
@@ -2346,36 +2406,36 @@ static void test_thread_description(void)
     LocalFree(ptr);
 
     len = 0;
-    status = pNtQueryInformationThread(GetCurrentThread(), ThreadDescription, NULL, 0, &len);
+    status = pNtQueryInformationThread(GetCurrentThread(), ThreadNameInformation, NULL, 0, &len);
     ok(status == STATUS_BUFFER_TOO_SMALL, "Failed to get thread info, status %#x.\n", status);
     ok(len == sizeof(*thread_desc) + desc_len, "Unexpected structure length %u.\n", len);
 
     len = 0;
-    status = pNtQueryInformationThread(GetCurrentThread(), ThreadDescription, buff, sizeof(buff), &len);
+    status = pNtQueryInformationThread(GetCurrentThread(), ThreadNameInformation, buff, sizeof(buff), &len);
     ok(!status, "Failed to get thread info.\n");
     ok(len == sizeof(*thread_desc) + desc_len, "Unexpected structure length %u.\n", len);
 
-    ok(thread_desc->Description.Length == desc_len && thread_desc->Description.MaximumLength == desc_len,
-            "Unexpected description length %u.\n", thread_desc->Description.Length);
-    ok(thread_desc->Description.Buffer == (WCHAR *)(thread_desc + 1),
-            "Unexpected description string pointer %p, %p.\n", thread_desc->Description.Buffer, thread_desc);
-    ok(!memcmp(thread_desc->Description.Buffer, desc, desc_len), "Unexpected description string.\n");
+    ok(thread_desc->ThreadName.Length == desc_len && thread_desc->ThreadName.MaximumLength == desc_len,
+            "Unexpected description length %u.\n", thread_desc->ThreadName.Length);
+    ok(thread_desc->ThreadName.Buffer == (WCHAR *)(thread_desc + 1),
+            "Unexpected description string pointer %p, %p.\n", thread_desc->ThreadName.Buffer, thread_desc);
+    ok(!memcmp(thread_desc->ThreadName.Buffer, desc, desc_len), "Unexpected description string.\n");
 
     /* Partial results. */
     len = 0;
-    status = pNtQueryInformationThread(GetCurrentThread(), ThreadDescription, NULL, 0, &len);
+    status = pNtQueryInformationThread(GetCurrentThread(), ThreadNameInformation, NULL, 0, &len);
     ok(status == STATUS_BUFFER_TOO_SMALL, "Unexpected status %#x.\n", status);
     ok(len == sizeof(*thread_desc) + desc_len, "Unexpected structure length %u.\n", len);
 
-    status = pNtQueryInformationThread(GetCurrentThread(), ThreadDescription, buff, len - sizeof(WCHAR), &len);
+    status = pNtQueryInformationThread(GetCurrentThread(), ThreadNameInformation, buff, len - sizeof(WCHAR), &len);
     ok(status == STATUS_BUFFER_TOO_SMALL, "Unexpected status %#x.\n", status);
     ok(len == sizeof(*thread_desc) + desc_len, "Unexpected structure length %u.\n", len);
 
     /* Change description. */
-    thread_desc->Description.Length = thread_desc->Description.MaximumLength = 8;
+    thread_desc->ThreadName.Length = thread_desc->ThreadName.MaximumLength = 8;
     lstrcpyW((WCHAR *)(thread_desc + 1), L"desc");
 
-    status = pNtSetInformationThread(GetCurrentThread(), ThreadDescription, thread_desc, sizeof(*thread_desc));
+    status = pNtSetInformationThread(GetCurrentThread(), ThreadNameInformation, thread_desc, sizeof(*thread_desc));
     ok(status == STATUS_SUCCESS, "Failed to set thread description, status %#x.\n", status);
 
     ptr = NULL;
@@ -2384,14 +2444,14 @@ static void test_thread_description(void)
     ok(!lstrcmpW(ptr, L"desc"), "Unexpected description %s.\n", wine_dbgstr_w(ptr));
     LocalFree(ptr);
 
-    status = pNtSetInformationThread(GetCurrentThread(), ThreadDescription, thread_desc, sizeof(*thread_desc) - 1);
+    status = pNtSetInformationThread(GetCurrentThread(), ThreadNameInformation, thread_desc, sizeof(*thread_desc) - 1);
     ok(status == STATUS_INFO_LENGTH_MISMATCH, "Unexpected status %#x.\n", status);
 
-    status = NtSetInformationThread(GetCurrentThread(), ThreadDescription, NULL, sizeof(*thread_desc));
+    status = NtSetInformationThread(GetCurrentThread(), ThreadNameInformation, NULL, sizeof(*thread_desc));
     ok(status == STATUS_ACCESS_VIOLATION, "Unexpected status %#x.\n", status);
 
-    thread_desc->Description.Buffer = NULL;
-    status = pNtSetInformationThread(GetCurrentThread(), ThreadDescription, thread_desc, sizeof(*thread_desc));
+    thread_desc->ThreadName.Buffer = NULL;
+    status = pNtSetInformationThread(GetCurrentThread(), ThreadNameInformation, thread_desc, sizeof(*thread_desc));
     ok(status == STATUS_ACCESS_VIOLATION, "Unexpected status %#x.\n", status);
 
     hr = pSetThreadDescription(GetCurrentThread(), NULL);
@@ -2408,7 +2468,7 @@ static void test_thread_description(void)
     ok(hr == HRESULT_FROM_NT(STATUS_SUCCESS), "Failed to set thread description, hr %#x.\n", hr);
 
     memset(thread_desc, 0, sizeof(*thread_desc));
-    status = pNtSetInformationThread(GetCurrentThread(), ThreadDescription, thread_desc, sizeof(*thread_desc));
+    status = pNtSetInformationThread(GetCurrentThread(), ThreadNameInformation, thread_desc, sizeof(*thread_desc));
     ok(!status, "Failed to set thread description, status %#x.\n", status);
 
     ptr = NULL;
