@@ -33,6 +33,7 @@
 
 #include "kernelbase.h"
 #include "wine/debug.h"
+#include "wine/condrv.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(process);
 
@@ -144,9 +145,9 @@ static RTL_USER_PROCESS_PARAMETERS *create_process_params( const WCHAR *filename
                                                            const STARTUPINFOW *startup )
 {
     RTL_USER_PROCESS_PARAMETERS *params;
-    UNICODE_STRING imageW, dllpathW, curdirW, cmdlineW, titleW, desktopW, runtimeW, newdirW;
+    UNICODE_STRING imageW, curdirW, cmdlineW, titleW, desktopW, runtimeW, newdirW;
     WCHAR imagepath[MAX_PATH];
-    WCHAR *load_path, *dummy, *envW = env;
+    WCHAR *envW = env;
 
     if (!GetLongPathNameW( filename, imagepath, MAX_PATH )) lstrcpynW( imagepath, filename, MAX_PATH );
     if (!GetFullPathNameW( imagepath, MAX_PATH, imagepath, NULL )) lstrcpynW( imagepath, filename, MAX_PATH );
@@ -171,29 +172,25 @@ static RTL_USER_PROCESS_PARAMETERS *create_process_params( const WCHAR *filename
         else
             cur_dir = NULL;
     }
-    LdrGetDllPath( imagepath, LOAD_WITH_ALTERED_SEARCH_PATH, &load_path, &dummy );
     RtlInitUnicodeString( &imageW, imagepath );
-    RtlInitUnicodeString( &dllpathW, load_path );
     RtlInitUnicodeString( &curdirW, cur_dir );
     RtlInitUnicodeString( &cmdlineW, cmdline );
     RtlInitUnicodeString( &titleW, startup->lpTitle ? startup->lpTitle : imagepath );
     RtlInitUnicodeString( &desktopW, startup->lpDesktop );
     runtimeW.Buffer = (WCHAR *)startup->lpReserved2;
     runtimeW.Length = runtimeW.MaximumLength = startup->cbReserved2;
-    if (RtlCreateProcessParametersEx( &params, &imageW, &dllpathW, cur_dir ? &curdirW : NULL,
+    if (RtlCreateProcessParametersEx( &params, &imageW, NULL, cur_dir ? &curdirW : NULL,
                                       &cmdlineW, envW, &titleW, &desktopW,
                                       NULL, &runtimeW, PROCESS_PARAMS_FLAG_NORMALIZED ))
     {
         RtlFreeUnicodeString( &newdirW );
-        RtlReleasePath( load_path );
         if (envW != env) RtlFreeHeap( GetProcessHeap(), 0, envW );
         return NULL;
     }
     RtlFreeUnicodeString( &newdirW );
-    RtlReleasePath( load_path );
 
     if (flags & CREATE_NEW_PROCESS_GROUP) params->ConsoleFlags = 1;
-    if (flags & CREATE_NEW_CONSOLE) params->ConsoleHandle = (HANDLE)1; /* KERNEL32_CONSOLE_ALLOC */
+    if (flags & CREATE_NEW_CONSOLE) params->ConsoleHandle = CONSOLE_HANDLE_ALLOC;
     else if (!(flags & DETACHED_PROCESS)) params->ConsoleHandle = NtCurrentTeb()->Peb->ProcessParameters->ConsoleHandle;
 
     if (startup->dwFlags & STARTF_USESTDHANDLES)
@@ -249,14 +246,16 @@ struct _PROC_THREAD_ATTRIBUTE_LIST
 /***********************************************************************
  *           create_nt_process
  */
-static NTSTATUS create_nt_process( HANDLE token, SECURITY_ATTRIBUTES *psa, SECURITY_ATTRIBUTES *tsa,
-                                   BOOL inherit, DWORD flags, RTL_USER_PROCESS_PARAMETERS *params,
+static NTSTATUS create_nt_process( HANDLE token, HANDLE debug, SECURITY_ATTRIBUTES *psa,
+                                   SECURITY_ATTRIBUTES *tsa, DWORD process_flags,
+                                   RTL_USER_PROCESS_PARAMETERS *params,
                                    RTL_USER_PROCESS_INFORMATION *info, HANDLE parent,
-                                   const struct proc_thread_attr *handle_list )
+                                   const struct proc_thread_attr *handle_list,
+                                   const struct proc_thread_attr *job_list)
 {
     OBJECT_ATTRIBUTES process_attr, thread_attr;
     PS_CREATE_INFO create_info;
-    ULONG_PTR buffer[offsetof( PS_ATTRIBUTE_LIST, Attributes[6] ) / sizeof(ULONG_PTR)];
+    ULONG_PTR buffer[offsetof( PS_ATTRIBUTE_LIST, Attributes[8] ) / sizeof(ULONG_PTR)];
     PS_ATTRIBUTE_LIST *attr = (PS_ATTRIBUTE_LIST *)buffer;
     UNICODE_STRING nameW;
     NTSTATUS status;
@@ -266,8 +265,6 @@ static NTSTATUS create_nt_process( HANDLE token, SECURITY_ATTRIBUTES *psa, SECUR
     status = RtlDosPathNameToNtPathName_U_WithStatus( params->ImagePathName.Buffer, &nameW, NULL, NULL );
     if (!status)
     {
-        params->DebugFlags = flags;  /* hack, cf. RtlCreateUserProcess implementation */
-
         RtlNormalizeProcessParams( params );
 
         attr->Attributes[pos].Attribute    = PS_ATTRIBUTE_IMAGE_NAME;
@@ -293,7 +290,7 @@ static NTSTATUS create_nt_process( HANDLE token, SECURITY_ATTRIBUTES *psa, SECUR
             attr->Attributes[pos].ReturnLength = NULL;
             pos++;
         }
-        if (inherit && handle_list)
+        if ((process_flags & PROCESS_CREATE_FLAGS_INHERIT_HANDLES) && handle_list)
         {
             attr->Attributes[pos].Attribute    = PS_ATTRIBUTE_HANDLE_LIST;
             attr->Attributes[pos].Size         = handle_list->size;
@@ -309,14 +306,29 @@ static NTSTATUS create_nt_process( HANDLE token, SECURITY_ATTRIBUTES *psa, SECUR
             attr->Attributes[pos].ReturnLength = NULL;
             pos++;
         }
+        if (debug)
+        {
+            attr->Attributes[pos].Attribute    = PS_ATTRIBUTE_DEBUG_PORT;
+            attr->Attributes[pos].Size         = sizeof(debug);
+            attr->Attributes[pos].ValuePtr     = debug;
+            attr->Attributes[pos].ReturnLength = NULL;
+            pos++;
+        }
+        if (job_list)
+        {
+            attr->Attributes[pos].Attribute    = PS_ATTRIBUTE_JOB_LIST;
+            attr->Attributes[pos].Size         = job_list->size;
+            attr->Attributes[pos].ValuePtr     = job_list->value;
+            attr->Attributes[pos].ReturnLength = NULL;
+            pos++;
+        }
         attr->TotalLength = offsetof( PS_ATTRIBUTE_LIST, Attributes[pos] );
 
         InitializeObjectAttributes( &process_attr, NULL, 0, NULL, psa ? psa->lpSecurityDescriptor : NULL );
         InitializeObjectAttributes( &thread_attr, NULL, 0, NULL, tsa ? tsa->lpSecurityDescriptor : NULL );
 
         status = NtCreateUserProcess( &info->Process, &info->Thread, PROCESS_ALL_ACCESS, THREAD_ALL_ACCESS,
-                                      &process_attr, &thread_attr,
-                                      inherit ? PROCESS_CREATE_FLAGS_INHERIT_HANDLES : 0,
+                                      &process_attr, &thread_attr, process_flags,
                                       THREAD_CREATE_FLAGS_CREATE_SUSPENDED, params,
                                       &create_info, attr );
 
@@ -329,8 +341,9 @@ static NTSTATUS create_nt_process( HANDLE token, SECURITY_ATTRIBUTES *psa, SECUR
 /***********************************************************************
  *           create_vdm_process
  */
-static NTSTATUS create_vdm_process( HANDLE token, SECURITY_ATTRIBUTES *psa, SECURITY_ATTRIBUTES *tsa,
-                                    BOOL inherit, DWORD flags, RTL_USER_PROCESS_PARAMETERS *params,
+static NTSTATUS create_vdm_process( HANDLE token, HANDLE debug, SECURITY_ATTRIBUTES *psa,
+                                    SECURITY_ATTRIBUTES *tsa, DWORD flags,
+                                    RTL_USER_PROCESS_PARAMETERS *params,
                                     RTL_USER_PROCESS_INFORMATION *info )
 {
     const WCHAR *winevdm = (is_win64 || is_wow64 ?
@@ -350,7 +363,7 @@ static NTSTATUS create_vdm_process( HANDLE token, SECURITY_ATTRIBUTES *psa, SECU
               winevdm, params->ImagePathName.Buffer, params->CommandLine.Buffer );
     RtlInitUnicodeString( &params->ImagePathName, winevdm );
     RtlInitUnicodeString( &params->CommandLine, newcmdline );
-    status = create_nt_process( token, psa, tsa, inherit, flags, params, info, NULL, NULL );
+    status = create_nt_process( token, debug, psa, tsa, flags, params, info, NULL, NULL, NULL );
     HeapFree( GetProcessHeap(), 0, newcmdline );
     return status;
 }
@@ -359,8 +372,9 @@ static NTSTATUS create_vdm_process( HANDLE token, SECURITY_ATTRIBUTES *psa, SECU
 /***********************************************************************
  *           create_cmd_process
  */
-static NTSTATUS create_cmd_process( HANDLE token, SECURITY_ATTRIBUTES *psa, SECURITY_ATTRIBUTES *tsa,
-                                    BOOL inherit, DWORD flags, RTL_USER_PROCESS_PARAMETERS *params,
+static NTSTATUS create_cmd_process( HANDLE token, HANDLE debug, SECURITY_ATTRIBUTES *psa,
+                                    SECURITY_ATTRIBUTES *tsa, DWORD flags,
+                                    RTL_USER_PROCESS_PARAMETERS *params,
                                     RTL_USER_PROCESS_INFORMATION *info )
 {
     WCHAR comspec[MAX_PATH];
@@ -378,7 +392,7 @@ static NTSTATUS create_cmd_process( HANDLE token, SECURITY_ATTRIBUTES *psa, SECU
     swprintf( newcmdline, len, L"%s /s/c \"%s\"", comspec, params->CommandLine.Buffer );
     RtlInitUnicodeString( &params->ImagePathName, comspec );
     RtlInitUnicodeString( &params->CommandLine, newcmdline );
-    status = create_nt_process( token, psa, tsa, inherit, flags, params, info, NULL, NULL );
+    status = create_nt_process( token, debug, psa, tsa, flags, params, info, NULL, NULL, NULL );
     RtlFreeHeap( GetProcessHeap(), 0, newcmdline );
     return status;
 }
@@ -481,12 +495,13 @@ BOOL WINAPI DECLSPEC_HOTPATCH CreateProcessInternalW( HANDLE token, const WCHAR
                                                       const WCHAR *cur_dir, STARTUPINFOW *startup_info,
                                                       PROCESS_INFORMATION *info, HANDLE *new_token )
 {
-    const struct proc_thread_attr *handle_list = NULL;
+    const struct proc_thread_attr *handle_list = NULL, *job_list = NULL;
     WCHAR name[MAX_PATH];
     WCHAR *p, *tidy_cmdline = cmd_line;
     RTL_USER_PROCESS_PARAMETERS *params = NULL;
     RTL_USER_PROCESS_INFORMATION rtl_info;
-    HANDLE parent = NULL;
+    HANDLE parent = 0, debug = 0;
+    ULONG nt_flags = 0;
     NTSTATUS status;
 
     /* Process the AppName and/or CmdLine to get module name and path */
@@ -536,6 +551,12 @@ BOOL WINAPI DECLSPEC_HOTPATCH CreateProcessInternalW( HANDLE token, const WCHAR
         goto done;
     }
 
+    if (flags & (DEBUG_PROCESS | DEBUG_ONLY_THIS_PROCESS))
+    {
+        if ((status = DbgUiConnectToDbg())) goto done;
+        debug = DbgUiGetThreadDebugObject();
+    }
+
     if (flags & EXTENDED_STARTUPINFO_PRESENT)
     {
         struct _PROC_THREAD_ATTRIBUTE_LIST *attrs =
@@ -569,6 +590,11 @@ BOOL WINAPI DECLSPEC_HOTPATCH CreateProcessInternalW( HANDLE token, const WCHAR
                             params->ConsoleHandle = console->reference;
                             break;
                         }
+                    case PROC_THREAD_ATTRIBUTE_JOB_LIST:
+                        job_list = &attrs->attrs[i];
+                        TRACE( "PROC_THREAD_ATTRIBUTE_JOB_LIST handle count %Iu.\n",
+                               attrs->attrs[i].size / sizeof(HANDLE) );
+                        break;
                     default:
                         FIXME("Unsupported attribute %#Ix.\n", attrs->attrs[i].attr);
                         break;
@@ -577,8 +603,13 @@ BOOL WINAPI DECLSPEC_HOTPATCH CreateProcessInternalW( HANDLE token, const WCHAR
         }
     }
 
-    status = create_nt_process( token, process_attr, thread_attr, inherit,
-                                flags, params, &rtl_info, parent, handle_list );
+    if (inherit) nt_flags |= PROCESS_CREATE_FLAGS_INHERIT_HANDLES;
+    if (flags & DEBUG_ONLY_THIS_PROCESS) nt_flags |= PROCESS_CREATE_FLAGS_NO_DEBUG_INHERIT;
+    if (flags & CREATE_BREAKAWAY_FROM_JOB) nt_flags |= PROCESS_CREATE_FLAGS_BREAKAWAY;
+    if (flags & CREATE_SUSPENDED) nt_flags |= PROCESS_CREATE_FLAGS_SUSPENDED;
+
+    status = create_nt_process( token, debug, process_attr, thread_attr,
+                                nt_flags, params, &rtl_info, parent, handle_list, job_list );
     switch (status)
     {
     case STATUS_SUCCESS:
@@ -587,7 +618,8 @@ BOOL WINAPI DECLSPEC_HOTPATCH CreateProcessInternalW( HANDLE token, const WCHAR
     case STATUS_INVALID_IMAGE_NE_FORMAT:
     case STATUS_INVALID_IMAGE_PROTECT:
         TRACE( "starting %s as Win16/DOS binary\n", debugstr_w(app_name) );
-        status = create_vdm_process( token, process_attr, thread_attr, inherit, flags, params, &rtl_info );
+        status = create_vdm_process( token, debug, process_attr, thread_attr,
+                                     nt_flags, params, &rtl_info );
         break;
     case STATUS_INVALID_IMAGE_NOT_MZ:
         /* check for .com or .bat extension */
@@ -595,12 +627,14 @@ BOOL WINAPI DECLSPEC_HOTPATCH CreateProcessInternalW( HANDLE token, const WCHAR
         if (!wcsicmp( p, L".com" ) || !wcsicmp( p, L".pif" ))
         {
             TRACE( "starting %s as DOS binary\n", debugstr_w(app_name) );
-            status = create_vdm_process( token, process_attr, thread_attr, inherit, flags, params, &rtl_info );
+            status = create_vdm_process( token, debug, process_attr, thread_attr,
+                                         nt_flags, params, &rtl_info );
         }
         else if (!wcsicmp( p, L".bat" ) || !wcsicmp( p, L".cmd" ))
         {
             TRACE( "starting %s as batch binary\n", debugstr_w(app_name) );
-            status = create_cmd_process( token, process_attr, thread_attr, inherit, flags, params, &rtl_info );
+            status = create_cmd_process( token, debug, process_attr, thread_attr,
+                                         nt_flags, params, &rtl_info );
         }
         break;
     }
@@ -771,6 +805,17 @@ DWORD WINAPI DECLSPEC_HOTPATCH GetPriorityClass( HANDLE process )
 }
 
 
+/***********************************************************************
+ *           GetProcessGroupAffinity   (kernelbase.@)
+ */
+BOOL WINAPI DECLSPEC_HOTPATCH GetProcessGroupAffinity( HANDLE process, USHORT *count, USHORT *array )
+{
+    FIXME( "(%p,%p,%p): stub\n", process, count, array );
+    SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
+    return FALSE;
+}
+
+
 /******************************************************************
  *           GetProcessHandleCount   (kernelbase.@)
  */
@@ -879,7 +924,7 @@ DWORD WINAPI DECLSPEC_HOTPATCH GetProcessVersion( DWORD pid )
                                              &info, sizeof(info), NULL );
 
     if (!set_ntstatus( status )) return 0;
-    return MAKELONG( info.SubsystemVersionLow, info.SubsystemVersionHigh );
+    return MAKELONG( info.MinorSubsystemVersion, info.MajorSubsystemVersion );
 }
 
 
@@ -933,70 +978,7 @@ BOOL WINAPI DECLSPEC_HOTPATCH IsProcessorFeaturePresent ( DWORD feature )
  */
 BOOL WINAPI DECLSPEC_HOTPATCH IsWow64Process2( HANDLE process, USHORT *machine, USHORT *native_machine )
 {
-    BOOL wow64;
-    SYSTEM_INFO si;
-
-    TRACE( "(%p,%p,%p)\n", process, machine, native_machine );
-
-    if (!IsWow64Process( process, &wow64 ))
-        return FALSE;
-
-    if (wow64)
-    {
-        if (process != GetCurrentProcess())
-        {
-#if defined(__i386__) || defined(__x86_64__)
-            *machine = IMAGE_FILE_MACHINE_I386;
-#else
-            FIXME("not implemented for other process\n");
-            *machine = IMAGE_FILE_MACHINE_UNKNOWN;
-#endif
-        }
-        else
-        {
-            IMAGE_NT_HEADERS *nt;
-            nt = RtlImageNtHeader( NtCurrentTeb()->Peb->ImageBaseAddress );
-            *machine = nt->FileHeader.Machine;
-        }
-
-        if (!native_machine) return TRUE;
-
-        GetNativeSystemInfo( &si );
-    }
-    else
-    {
-        *machine = IMAGE_FILE_MACHINE_UNKNOWN;
-
-        if (!native_machine) return TRUE;
-
-#ifdef _WIN64
-        GetSystemInfo( &si );
-#else
-        GetNativeSystemInfo( &si );
-#endif
-    }
-
-    switch (si.u.s.wProcessorArchitecture)
-    {
-    case PROCESSOR_ARCHITECTURE_INTEL:
-        *native_machine = IMAGE_FILE_MACHINE_I386;
-        break;
-    case PROCESSOR_ARCHITECTURE_ARM:
-        *native_machine = IMAGE_FILE_MACHINE_ARM;
-        break;
-    case PROCESSOR_ARCHITECTURE_AMD64:
-        *native_machine = IMAGE_FILE_MACHINE_AMD64;
-        break;
-    case PROCESSOR_ARCHITECTURE_ARM64:
-        *native_machine = IMAGE_FILE_MACHINE_ARM64;
-        break;
-    default:
-        FIXME("unknown architecture %u\n", si.u.s.wProcessorArchitecture);
-        *native_machine = IMAGE_FILE_MACHINE_UNKNOWN;
-        break;
-    }
-
-    return TRUE;
+    return set_ntstatus( RtlWow64GetProcessMachines( process, machine, native_machine ));
 }
 
 
@@ -1043,11 +1025,20 @@ HANDLE WINAPI DECLSPEC_HOTPATCH OpenProcess( DWORD access, BOOL inherit, DWORD i
 /***********************************************************************
  *           ProcessIdToSessionId   (kernelbase.@)
  */
-BOOL WINAPI DECLSPEC_HOTPATCH ProcessIdToSessionId( DWORD procid, DWORD *sessionid )
+BOOL WINAPI DECLSPEC_HOTPATCH ProcessIdToSessionId( DWORD pid, DWORD *id )
 {
-    if (procid != GetCurrentProcessId()) FIXME( "Unsupported for other process %x\n", procid );
-    *sessionid = NtCurrentTeb()->Peb->SessionId;
-    return TRUE;
+    HANDLE process;
+    NTSTATUS status;
+
+    if (pid == GetCurrentProcessId())
+    {
+        *id = NtCurrentTeb()->Peb->SessionId;
+        return TRUE;
+    }
+    if (!(process = OpenProcess( PROCESS_QUERY_LIMITED_INFORMATION, FALSE, pid ))) return FALSE;
+    status = NtQueryInformationProcess( process, ProcessSessionInformation, id, sizeof(*id), NULL );
+    CloseHandle( process );
+    return set_ntstatus( status );
 }
 
 
@@ -1143,6 +1134,18 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetProcessAffinityUpdateMode( HANDLE process, DWOR
 }
 
 
+/***********************************************************************
+ *           SetProcessGroupAffinity   (kernelbase.@)
+ */
+BOOL WINAPI DECLSPEC_HOTPATCH SetProcessGroupAffinity( HANDLE process, const GROUP_AFFINITY *new,
+                                                       GROUP_AFFINITY *old )
+{
+    FIXME( "(%p,%p,%p): stub\n", process, new, old );
+    SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
+    return FALSE;
+}
+
+
 /**********************************************************************
  *           SetProcessMitigationPolicy   (kernelbase.@)
  */
@@ -1731,6 +1734,14 @@ BOOL WINAPI DECLSPEC_HOTPATCH UpdateProcThreadAttribute( struct _PROC_THREAD_ATT
        }
        break;
 
+    case PROC_THREAD_ATTRIBUTE_JOB_LIST:
+        if ((size / sizeof(HANDLE)) * sizeof(HANDLE) != size)
+        {
+            SetLastError( ERROR_BAD_LENGTH );
+            return FALSE;
+        }
+        break;
+
     default:
         SetLastError( ERROR_NOT_SUPPORTED );
         FIXME( "Unhandled attribute %lu\n", attr & PROC_THREAD_ATTRIBUTE_NUMBER );
@@ -1761,3 +1772,12 @@ void WINAPI DECLSPEC_HOTPATCH DeleteProcThreadAttributeList( struct _PROC_THREAD
 {
     return;
 }
+
+
+/***********************************************************************
+ *              CompareObjectHandles   (kernelbase.@)
+ */
+BOOL WINAPI DECLSPEC_HOTPATCH CompareObjectHandles( HANDLE first, HANDLE second )
+{
+    return set_ntstatus( NtCompareObjects( first, second ));
+}
