@@ -473,12 +473,21 @@ static const struct combine_test combine_tests[] =
     /* Second path is fully qualified */
     {"X:\\", "C:", "C:\\"},
     {"X:\\", "C:\\", "C:\\"},
+    {"X:\\", "C:\\a", "C:\\a"},
     {"X:\\", "\\\\", "\\\\"},
+    {"X:\\", "\\\\a", "\\\\a"},
     {"X:\\", "\\\\?\\C:", "C:\\"},
     {"X:\\", "\\\\?\\C:\\", "C:\\"},
+    {"X:\\", "\\\\?\\C:\\a", "C:\\a"},
+    {"X:\\", "\\\\?\\UNC", "\\\\?\\UNC"},
     {"X:\\", "\\\\?\\UNC\\", "\\\\"},
+    {"X:\\", "\\\\?\\UNC\\a", "\\\\a"},
+    {"X:\\", "\\\\?\\Volume{e51a1864-6f2d-4019-b73d-f4e60e600c26}",
+     "\\\\?\\Volume{e51a1864-6f2d-4019-b73d-f4e60e600c26}"},
     {"X:\\", "\\\\?\\Volume{e51a1864-6f2d-4019-b73d-f4e60e600c26}\\",
      "\\\\?\\Volume{e51a1864-6f2d-4019-b73d-f4e60e600c26}\\"},
+    {"X:\\", "\\\\?\\Volume{e51a1864-6f2d-4019-b73d-f4e60e600c26}\\a",
+     "\\\\?\\Volume{e51a1864-6f2d-4019-b73d-f4e60e600c26}\\a"},
 
     /* Canonicalization */
     {"C:\\a", ".\\b", "C:\\a\\b"},
