@@ -26,7 +26,6 @@
 #include "audioclient.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 #include "wine/list.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mfplat);
@@ -102,6 +101,8 @@ struct audio_renderer
     HANDLE buffer_ready_event;
     MFWORKITEM_KEY buffer_ready_key;
     unsigned int frame_size;
+    unsigned int queued_frames;
+    unsigned int max_frames;
     struct list queue;
     enum stream_state state;
     unsigned int flags;
@@ -121,7 +122,7 @@ static void release_pending_object(struct queued_object *object)
             PropVariantClear(&object->u.marker.context);
             break;
     }
-    heap_free(object);
+    free(object);
 }
 
 static struct audio_renderer *impl_from_IMFMediaSink(IMFMediaSink *iface)
@@ -228,7 +229,14 @@ static ULONG WINAPI audio_renderer_sink_AddRef(IMFMediaSink *iface)
 
 static void audio_renderer_release_audio_client(struct audio_renderer *renderer)
 {
+    struct queued_object *obj, *obj2;
+
     MFCancelWorkItem(renderer->buffer_ready_key);
+    LIST_FOR_EACH_ENTRY_SAFE(obj, obj2, &renderer->queue, struct queued_object, entry)
+    {
+        release_pending_object(obj);
+    }
+    renderer->queued_frames = 0;
     renderer->buffer_ready_key = 0;
     if (renderer->audio_client)
     {
@@ -253,7 +261,6 @@ static ULONG WINAPI audio_renderer_sink_Release(IMFMediaSink *iface)
 {
     struct audio_renderer *renderer = impl_from_IMFMediaSink(iface);
     ULONG refcount = InterlockedDecrement(&renderer->refcount);
-    struct queued_object *obj, *obj2;
 
     TRACE("%p, refcount %u.\n", iface, refcount);
 
@@ -273,12 +280,8 @@ static ULONG WINAPI audio_renderer_sink_Release(IMFMediaSink *iface)
             IMFMediaType_Release(renderer->current_media_type);
         audio_renderer_release_audio_client(renderer);
         CloseHandle(renderer->buffer_ready_event);
-        LIST_FOR_EACH_ENTRY_SAFE(obj, obj2, &renderer->queue, struct queued_object, entry)
-        {
-            release_pending_object(obj);
-        }
         DeleteCriticalSection(&renderer->cs);
-        heap_free(renderer);
+        free(renderer);
     }
 
     return refcount;
@@ -1330,8 +1333,15 @@ static HRESULT WINAPI audio_renderer_stream_GetMediaTypeHandler(IMFStreamSink *i
 static HRESULT stream_queue_sample(struct audio_renderer *renderer, IMFSample *sample)
 {
     struct queued_object *object;
+    DWORD sample_len, sample_frames;
+    HRESULT hr;
 
-    if (!(object = heap_alloc_zero(sizeof(*object))))
+    if (FAILED(hr = IMFSample_GetTotalLength(sample, &sample_len)))
+        return hr;
+
+    sample_frames = sample_len / renderer->frame_size;
+
+    if (!(object = calloc(1, sizeof(*object))))
         return E_OUTOFMEMORY;
 
     object->type = OBJECT_TYPE_SAMPLE;
@@ -1339,6 +1349,7 @@ static HRESULT stream_queue_sample(struct audio_renderer *renderer, IMFSample *s
     IMFSample_AddRef(object->u.sample.sample);
 
     list_add_tail(&renderer->queue, &object->entry);
+    renderer->queued_frames += sample_frames;
 
     return S_OK;
 }
@@ -1357,9 +1368,17 @@ static HRESULT WINAPI audio_renderer_stream_ProcessSample(IMFStreamSink *iface,
         return MF_E_STREAMSINK_REMOVED;
 
     EnterCriticalSection(&renderer->cs);
+
     if (renderer->state == STREAM_STATE_RUNNING)
         hr = stream_queue_sample(renderer, sample);
     renderer->flags &= ~SAR_SAMPLE_REQUESTED;
+
+    if (renderer->queued_frames < renderer->max_frames && renderer->state == STREAM_STATE_RUNNING)
+    {
+        IMFMediaEventQueue_QueueEventParamVar(renderer->stream_event_queue, MEStreamSinkRequestSample, &GUID_NULL, S_OK, NULL);
+        renderer->flags |= SAR_SAMPLE_REQUESTED;
+    }
+
     LeaveCriticalSection(&renderer->cs);
 
     return hr;
@@ -1371,7 +1390,7 @@ static HRESULT stream_place_marker(struct audio_renderer *renderer, MFSTREAMSINK
     struct queued_object *marker;
     HRESULT hr = S_OK;
 
-    if (!(marker = heap_alloc_zero(sizeof(*marker))))
+    if (!(marker = calloc(1, sizeof(*marker))))
         return E_OUTOFMEMORY;
 
     marker->type = OBJECT_TYPE_MARKER;
@@ -1428,6 +1447,7 @@ static HRESULT WINAPI audio_renderer_stream_Flush(IMFStreamSink *iface)
             release_pending_object(obj);
         }
     }
+    renderer->queued_frames = 0;
     LeaveCriticalSection(&renderer->cs);
 
     return hr;
@@ -1576,6 +1596,12 @@ static HRESULT audio_renderer_create_audio_client(struct audio_renderer *rendere
         return hr;
     }
 
+    if (FAILED(hr = IAudioClient_GetBufferSize(renderer->audio_client, &renderer->max_frames)))
+    {
+        WARN("Failed to get buffer size, hr %#x.\n", hr);
+        return hr;
+    }
+
     if (SUCCEEDED(hr = MFCreateAsyncResult(NULL, &renderer->render_callback, NULL, &result)))
     {
         if (FAILED(hr = MFPutWaitingWorkItem(renderer->buffer_ready_event, 0, result, &renderer->buffer_ready_key)))
@@ -1749,18 +1775,15 @@ static HRESULT WINAPI audio_renderer_render_callback_GetParameters(IMFAsyncCallb
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI audio_renderer_render_callback_Invoke(IMFAsyncCallback *iface, IMFAsyncResult *result)
+static void audio_renderer_render(struct audio_renderer *renderer, IMFAsyncResult *result)
 {
-    struct audio_renderer *renderer = impl_from_render_callback_IMFAsyncCallback(iface);
-    unsigned int src_frames, dst_frames, max_frames, src_len;
+    unsigned int src_frames, dst_frames, max_frames, pad_frames, src_len;
     struct queued_object *obj, *obj2;
     BOOL keep_sample = FALSE;
     IMFMediaBuffer *buffer;
     BYTE *dst, *src;
     HRESULT hr;
 
-    EnterCriticalSection(&renderer->cs);
-
     LIST_FOR_EACH_ENTRY_SAFE(obj, obj2, &renderer->queue, struct queued_object, entry)
     {
         if (obj->type == OBJECT_TYPE_MARKER)
@@ -1778,20 +1801,25 @@ static HRESULT WINAPI audio_renderer_render_callback_Invoke(IMFAsyncCallback *if
                     {
                         if (SUCCEEDED(IAudioClient_GetBufferSize(renderer->audio_client, &max_frames)))
                         {
-                            src_frames -= obj->u.sample.frame_offset;
-                            dst_frames = min(src_frames, max_frames);
-
-                            if (SUCCEEDED(hr = IAudioRenderClient_GetBuffer(renderer->audio_render_client, dst_frames, &dst)))
+                            if (SUCCEEDED(IAudioClient_GetCurrentPadding(renderer->audio_client, &pad_frames)))
                             {
-                                memcpy(dst, src + obj->u.sample.frame_offset * renderer->frame_size,
-                                        dst_frames * renderer->frame_size);
+                                max_frames -= pad_frames;
+                                src_frames -= obj->u.sample.frame_offset;
+                                dst_frames = min(src_frames, max_frames);
 
-                                IAudioRenderClient_ReleaseBuffer(renderer->audio_render_client, dst_frames, 0);
+                                if (SUCCEEDED(hr = IAudioRenderClient_GetBuffer(renderer->audio_render_client, dst_frames, &dst)))
+                                {
+                                    memcpy(dst, src + obj->u.sample.frame_offset * renderer->frame_size,
+                                            dst_frames * renderer->frame_size);
 
-                                obj->u.sample.frame_offset += dst_frames;
+                                    IAudioRenderClient_ReleaseBuffer(renderer->audio_render_client, dst_frames, 0);
+
+                                    obj->u.sample.frame_offset += dst_frames;
+                                    renderer->queued_frames -= dst_frames;
+                                }
+
+                                keep_sample = FAILED(hr) || src_frames > max_frames;
                             }
-
-                            keep_sample = FAILED(hr) || src_frames > max_frames;
                         }
                     }
                     IMFMediaBuffer_Unlock(buffer);
@@ -1815,7 +1843,15 @@ static HRESULT WINAPI audio_renderer_render_callback_Invoke(IMFAsyncCallback *if
 
     if (FAILED(hr = MFPutWaitingWorkItem(renderer->buffer_ready_event, 0, result, &renderer->buffer_ready_key)))
         WARN("Failed to submit wait item, hr %#x.\n", hr);
+}
 
+static HRESULT WINAPI audio_renderer_render_callback_Invoke(IMFAsyncCallback *iface, IMFAsyncResult *result)
+{
+    struct audio_renderer *renderer = impl_from_render_callback_IMFAsyncCallback(iface);
+
+    EnterCriticalSection(&renderer->cs);
+    if (!(renderer->flags & SAR_SHUT_DOWN))
+        audio_renderer_render(renderer, result);
     LeaveCriticalSection(&renderer->cs);
 
     return S_OK;
@@ -1837,7 +1873,7 @@ static HRESULT sar_create_object(IMFAttributes *attributes, void *user_context,
 
     TRACE("%p, %p, %p.\n", attributes, user_context, obj);
 
-    if (!(renderer = heap_alloc_zero(sizeof(*renderer))))
+    if (!(renderer = calloc(1, sizeof(*renderer))))
         return E_OUTOFMEMORY;
 
     renderer->IMFMediaSink_iface.lpVtbl = &audio_renderer_sink_vtbl;
@@ -1879,6 +1915,11 @@ failed:
     return hr;
 }
 
+BOOL mf_is_sar_sink(IMFMediaSink *sink)
+{
+    return sink->lpVtbl == &audio_renderer_sink_vtbl;
+}
+
 static void sar_shutdown_object(void *user_context, IUnknown *obj)
 {
     IMFMediaSink *sink;
