@@ -1125,7 +1125,6 @@ BOOL WINAPI IsColorProfileTagPresent( HPROFILE handle, TAGTYPE type, PBOOL prese
  */
 BOOL WINAPI IsColorProfileValid( HPROFILE handle, PBOOL valid )
 {
-    BOOL ret;
     struct profile *profile = grab_profile( handle );
 
     TRACE( "( %p, %p )\n", handle, valid );
@@ -1137,9 +1136,9 @@ BOOL WINAPI IsColorProfileValid( HPROFILE handle, PBOOL valid )
         release_profile( profile );
         return FALSE;
     }
-    if (profile->data) ret = *valid = TRUE;
+    *valid = !!profile->data;
     release_profile( profile );
-    return ret;
+    return *valid;
 }
 
 /******************************************************************************
@@ -1327,7 +1326,7 @@ HPROFILE WINAPI OpenColorProfileW( PPROFILE profile, DWORD access, DWORD sharing
 {
     struct profile prof;
     HPROFILE hprof;
-    void *cmsprofile = NULL;
+    cmsHPROFILE cmsprofile;
     char *data = NULL;
     HANDLE handle = INVALID_HANDLE_VALUE;
     DWORD size;
@@ -1343,7 +1342,7 @@ HPROFILE WINAPI OpenColorProfileW( PPROFILE profile, DWORD access, DWORD sharing
         if (!(data = HeapAlloc( GetProcessHeap(), 0, profile->cbDataSize ))) return NULL;
         memcpy( data, profile->pProfileData, profile->cbDataSize );
 
-        if (lcms_funcs && !(cmsprofile = lcms_funcs->open_profile( data, profile->cbDataSize )))
+        if (!(cmsprofile = cmsOpenProfileFromMem( data, profile->cbDataSize )))
         {
             HeapFree( GetProcessHeap(), 0, data );
             return FALSE;
@@ -1405,7 +1404,7 @@ HPROFILE WINAPI OpenColorProfileW( PPROFILE profile, DWORD access, DWORD sharing
             HeapFree( GetProcessHeap(), 0, data );
             return NULL;
         }
-        if (lcms_funcs && !(cmsprofile = lcms_funcs->open_profile( data, size )))
+        if (!(cmsprofile = cmsOpenProfileFromMem( data, size )))
         {
             CloseHandle( handle );
             HeapFree( GetProcessHeap(), 0, data );
@@ -1426,7 +1425,7 @@ HPROFILE WINAPI OpenColorProfileW( PPROFILE profile, DWORD access, DWORD sharing
 
     if ((hprof = create_profile( &prof ))) return hprof;
 
-    if (cmsprofile) lcms_funcs->close_profile( cmsprofile );
+    cmsCloseProfile( cmsprofile );
     HeapFree( GetProcessHeap(), 0, data );
     CloseHandle( handle );
     return NULL;
