@@ -17,7 +17,6 @@
  */
 
 #include <stdarg.h>
-#include <assert.h>
 
 #define COBJMACROS
 
@@ -66,6 +65,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 /* See jscript.h in jscript.dll. */
 #define SCRIPTLANGUAGEVERSION_HTML 0x400
 #define SCRIPTLANGUAGEVERSION_ES5  0x102
+#define SCRIPTLANGUAGEVERSION_ES6  0x103
 
 struct ScriptHost {
     IActiveScriptSite              IActiveScriptSite_iface;
@@ -154,7 +154,9 @@ static BOOL init_script_engine(ScriptHost *script_host)
     compat_mode = lock_document_mode(script_host->window->doc);
     script_mode = compat_mode < COMPAT_MODE_IE8 ? SCRIPTLANGUAGEVERSION_5_7 : SCRIPTLANGUAGEVERSION_5_8;
     if(IsEqualGUID(&script_host->guid, &CLSID_JScript)) {
-        if(compat_mode >= COMPAT_MODE_IE9)
+        if(compat_mode >= COMPAT_MODE_IE11)
+            script_mode = SCRIPTLANGUAGEVERSION_ES6;
+        else if(compat_mode >= COMPAT_MODE_IE9)
             script_mode = SCRIPTLANGUAGEVERSION_ES5;
         script_mode |= SCRIPTLANGUAGEVERSION_HTML;
     }
@@ -723,12 +725,22 @@ static void dispatch_script_readystatechange_event(HTMLScriptElement *script)
     DOMEvent *event;
     HRESULT hres;
 
-    hres = create_document_event(script->element.node.doc, EVENTID_READYSTATECHANGE, &event);
-    if(FAILED(hres))
-        return;
+    if(script->readystate != READYSTATE_LOADED ||
+       dispex_compat_mode(&script->element.node.event_target.dispex) < COMPAT_MODE_IE10) {
+        hres = create_document_event(script->element.node.doc, EVENTID_READYSTATECHANGE, &event);
+        if(SUCCEEDED(hres)) {
+            dispatch_event(&script->element.node.event_target, event);
+            IDOMEvent_Release(&event->IDOMEvent_iface);
+        }
+    }
 
-    dispatch_event(&script->element.node.event_target, event);
-    IDOMEvent_Release(&event->IDOMEvent_iface);
+    if(script->readystate == READYSTATE_LOADED) {
+        hres = create_document_event(script->element.node.doc, EVENTID_LOAD, &event);
+        if(SUCCEEDED(hres)) {
+            dispatch_event(&script->element.node.event_target, event);
+            IDOMEvent_Release(&event->IDOMEvent_iface);
+        }
+    }
 }
 
 typedef struct {
@@ -758,6 +770,10 @@ static void set_script_elem_readystate(HTMLScriptElement *script_elem, READYSTAT
 {
     script_elem->readystate = readystate;
 
+    if(readystate != READYSTATE_LOADED &&
+       dispex_compat_mode(&script_elem->element.node.event_target.dispex) >= COMPAT_MODE_IE11)
+        return;
+
     if(readystate != READYSTATE_INTERACTIVE) {
         if(!script_elem->element.node.doc->window->parser_callback_cnt) {
             fire_readystatechange_task_t *task;
@@ -808,6 +824,8 @@ typedef struct {
     BSCallback bsc;
 
     HTMLScriptElement *script_elem;
+    nsILoadGroup *load_group;
+    nsIRequest *request;
     DWORD scheme;
 
     DWORD size;
@@ -931,6 +949,13 @@ static void ScriptBSC_destroy(BSCallback *bsc)
         This->script_elem = NULL;
     }
 
+    if(This->request) {
+        ERR("Unfinished request\n");
+        nsIRequest_Release(This->request);
+    }
+    if(This->load_group)
+        nsILoadGroup_Release(This->load_group);
+
     heap_free(This->buf);
     heap_free(This);
 }
@@ -943,9 +968,19 @@ static HRESULT ScriptBSC_init_bindinfo(BSCallback *bsc)
 static HRESULT ScriptBSC_start_binding(BSCallback *bsc)
 {
     ScriptBSC *This = impl_from_BSCallback(bsc);
+    nsresult nsres;
 
     This->script_elem->binding = &This->bsc;
 
+    if(This->load_group) {
+        nsres = create_onload_blocker_request(&This->request);
+        if(NS_SUCCEEDED(nsres)) {
+            nsres = nsILoadGroup_AddRequest(This->load_group, This->request, NULL);
+            if(NS_FAILED(nsres))
+                ERR("AddRequest failed: %08x\n", nsres);
+        }
+    }
+
     /* FIXME: We should find a better to decide if 'loading' state is supposed to be used by the protocol. */
     if(This->scheme == URL_SCHEME_HTTPS || This->scheme == URL_SCHEME_HTTP)
         set_script_elem_readystate(This->script_elem, READYSTATE_LOADING);
@@ -956,6 +991,7 @@ static HRESULT ScriptBSC_start_binding(BSCallback *bsc)
 static HRESULT ScriptBSC_stop_binding(BSCallback *bsc, HRESULT result)
 {
     ScriptBSC *This = impl_from_BSCallback(bsc);
+    nsresult nsres;
 
     if(SUCCEEDED(result) && !This->script_elem)
         result = E_UNEXPECTED;
@@ -975,6 +1011,14 @@ static HRESULT ScriptBSC_stop_binding(BSCallback *bsc, HRESULT result)
         This->size = 0;
     }
 
+    if(This->request) {
+        nsres = nsILoadGroup_RemoveRequest(This->load_group, This->request, NULL, NS_OK);
+        if(NS_FAILED(nsres))
+            ERR("RemoveRequest failed: %08x\n", nsres);
+        nsIRequest_Release(This->request);
+        This->request = NULL;
+    }
+
     IHTMLScriptElement_Release(&This->script_elem->IHTMLScriptElement_iface);
     This->script_elem = NULL;
     return S_OK;
@@ -1085,6 +1129,20 @@ HRESULT load_script(HTMLScriptElement *script_elem, const WCHAR *src, BOOL async
     IHTMLScriptElement_AddRef(&script_elem->IHTMLScriptElement_iface);
     bsc->script_elem = script_elem;
 
+    if(window->bscallback && window->bscallback->nschannel &&
+       window->bscallback->nschannel->load_group) {
+        cpp_bool contains;
+        nsresult nsres;
+
+        nsres = nsIDOMNode_Contains(script_elem->element.node.doc->node.nsnode,
+                                    script_elem->element.node.nsnode, &contains);
+        if(NS_SUCCEEDED(nsres) && contains) {
+            TRACE("script %p will block load event\n", script_elem);
+            bsc->load_group = window->bscallback->nschannel->load_group;
+            nsILoadGroup_AddRef(bsc->load_group);
+        }
+    }
+
     hres = start_binding(window, &bsc->bsc, NULL);
 
     IBindStatusCallback_Release(&bsc->bsc.IBindStatusCallback_iface);
@@ -1658,12 +1716,13 @@ static BOOL use_gecko_script(IUri *uri)
         return FALSE;
 
     hres = IInternetSecurityManager_MapUrlToZone(get_security_manager(), display_uri, &zone, 0);
-    SysFreeString(display_uri);
     if(FAILED(hres)) {
         WARN("Could not map %s to zone: %08x\n", debugstr_w(display_uri), hres);
+        SysFreeString(display_uri);
         return TRUE;
     }
 
+    SysFreeString(display_uri);
     TRACE("zone %d\n", zone);
     return zone == URLZONE_UNTRUSTED;
 }
