@@ -32,8 +32,8 @@ void __cdecl _invalid_parameter(const wchar_t*, const wchar_t*,
         const wchar_t*, unsigned int, uintptr_t);
 BOOL __cdecl __uncaught_exception(void);
 
-extern void* (__cdecl *MSVCRT_operator_new)(size_t);
-extern void (__cdecl *MSVCRT_operator_delete)(void*);
+void* __cdecl operator_new(size_t);
+void __cdecl operator_delete(void*);
 
 /* basic_string<char, char_traits<char>, allocator<char>> */
 typedef struct
@@ -483,3 +483,9 @@ typedef struct {
     double real;
     double imag;
 } complex_double;
+
+void WINAPI DECLSPEC_NORETURN _CxxThrowException(void*,const cxx_exception_type*);
+void __cdecl DECLSPEC_NORETURN _Xlength_error(const char*);
+void __cdecl DECLSPEC_NORETURN _Xmem(void);
+void __cdecl DECLSPEC_NORETURN _Xout_of_range(const char*);
+void DECLSPEC_NORETURN throw_failure(const char*);
