@@ -108,9 +108,6 @@ static void* (__cdecl *p__RTDynamicCast)(void*,int,void*,void*,int);
 static char* (__cdecl *p__unDName)(char*,const char*,int,void*,void*,unsigned short int);
 
 
-/* _very_ early native versions have serious RTTI bugs, so we check */
-static void* bAncientVersion;
-
 /* Emulate a __thiscall */
 #ifdef __i386__
 
@@ -172,7 +169,6 @@ static BOOL InitFunctionPtrs(void)
     SET(p__unDName,"__unDName");
 
     /* Extremely early versions export logic_error, and crash in RTTI */
-    SETNOFAIL(bAncientVersion, "??0logic_error@@QAE@ABQBD@Z");
     if (sizeof(void *) > sizeof(int))  /* 64-bit initialization */
     {
         SETNOFAIL(poperator_new, "??_U@YAPEAX_K@Z");
@@ -433,7 +429,7 @@ static void test_exception(void)
   name = call_func1(pexception_what, &e);
   ok(e.name == name, "Bad exception name from vtable e::what()\n");
 
-  if (p__RTtypeid && !bAncientVersion)
+  if (p__RTtypeid)
   {
     /* Check the rtti */
     type_info *ti = p__RTtypeid(&e);
@@ -556,7 +552,7 @@ static void test_bad_typeid(void)
   name = call_func1(pbad_typeid_what, &e);
   ok(e.name == name, "Bad bad_typeid name from vtable e::what()\n");
 
-  if (p__RTtypeid && !bAncientVersion)
+  if (p__RTtypeid)
   {
     /* Check the rtti */
     type_info *ti = p__RTtypeid(&e);
@@ -684,7 +680,7 @@ static void test_bad_cast(void)
   name = call_func1(pbad_cast_what, &e);
   ok(e.name == name, "Bad bad_cast name from vtable e::what()\n");
 
-  if (p__RTtypeid && !bAncientVersion)
+  if (p__RTtypeid)
   {
     /* Check the rtti */
     type_info *ti = p__RTtypeid(&e);
@@ -786,7 +782,7 @@ static void test___non_rtti_object(void)
   name = call_func1(p__non_rtti_object_what, &e);
   ok(e.name == name, "Bad __non_rtti_object name from vtable e::what()\n");
 
-  if (p__RTtypeid && !bAncientVersion)
+  if (p__RTtypeid)
   {
     /* Check the rtti */
     type_info *ti = p__RTtypeid(&e);
@@ -969,8 +965,7 @@ static void test_rtti(void)
   char *base = (char*)GetModuleHandleW(NULL);
 #endif
 
-  if (bAncientVersion ||
-      !p__RTCastToVoid || !p__RTtypeid || !pexception_ctor || !pbad_typeid_ctor
+  if (!p__RTCastToVoid || !p__RTtypeid || !pexception_ctor || !pbad_typeid_ctor
       || !p__RTDynamicCast || !pexception_dtor || !pbad_typeid_dtor)
     return;
 
@@ -1106,30 +1101,10 @@ static void test_demangle_datatype(void)
         name = p__unDName(0, demangle[i].mangled, 0, malloc, free, 0x2800);
         todo_wine_if (!demangle[i].test_in_wine)
             ok(name != NULL && !strcmp(name,demangle[i].result), "Got name \"%s\" for %d\n", name, i);
-        if(name)
-            free(name);
+        free(name);
     }
 }
 
-/* Compare two strings treating multiple spaces (' ', ascii 0x20) in s2 
-   as single space. Needed for test_demangle as __unDName() returns sometimes
-   two spaces instead of one in some older native msvcrt dlls. */
-static int strcmp_space(const char *s1, const char *s2)
-{
-    const char* s2start = s2;
-    do {
-        while (*s1 == *s2 && *s1) {
-            s1++;
-            s2++;
-        }
-        if (*s2 == ' ' && s2 > s2start && *(s2 - 1) == ' ')
-            s2++;
-        else
-            break;
-    } while (*s1 && *s2);
-    return *s1 - *s2;
-}
-
 static void test_demangle(void)
 {
     static struct {const char* in; const char* out; const char *broken; unsigned int flags;} test[] = {
@@ -1325,11 +1300,11 @@ static void test_demangle(void)
 	name = p__unDName(0, test[i].in, 0, malloc, free, test[i].flags);
         ok(name != NULL, "%u: unDName failed\n", i);
         if (!name) continue;
-        ok( !strcmp_space(test[i].out, name) ||
-            broken(test[i].broken && !strcmp_space(test[i].broken, name)),
+        ok( !strcmp(test[i].out, name) ||
+            broken(test[i].broken && !strcmp(test[i].broken, name)),
            "%u: Got name \"%s\"\n", i, name );
-        ok( !strcmp_space(test[i].out, name) ||
-            broken(test[i].broken && !strcmp_space(test[i].broken, name)),
+        ok( !strcmp(test[i].out, name) ||
+            broken(test[i].broken && !strcmp(test[i].broken, name)),
            "%u: Expected \"%s\"\n", i, test[i].out );
         free(name);
     }
