@@ -44,8 +44,6 @@ WINE_DEFAULT_DEBUG_CHANNEL(msvcrt);
 #undef _time32
 #undef _wctime32
 
-BOOL WINAPI GetDaylightFlag(void);
-
 static LONGLONG init_time;
 
 void msvcrt_init_clock(void)
@@ -540,7 +538,18 @@ struct tm* CDECL _gmtime32(const __time32_t* secs)
  */
 char* CDECL _strdate(char* date)
 {
-    GetDateFormatA(LOCALE_NEUTRAL, 0, NULL, "MM'/'dd'/'yy", date, 9);
+    SYSTEMTIME st;
+
+    GetLocalTime(&st);
+    date[0] = '0' + st.wMonth / 10;
+    date[1] = '0' + st.wMonth % 10;
+    date[2] = '/';
+    date[3] = '0' + st.wDay / 10;
+    date[4] = '0' + st.wDay % 10;
+    date[5] = '/';
+    date[6] = '0' + st.wYear / 10 % 10;
+    date[7] = '0' + st.wYear % 10;
+    date[8] = 0;
     return date;
 }
 
@@ -602,7 +611,18 @@ int CDECL _wstrdate_s(wchar_t* date, size_t size)
  */
 char* CDECL _strtime(char* time)
 {
-    GetTimeFormatA(LOCALE_NEUTRAL, 0, NULL, "HH':'mm':'ss", time, 9);
+    SYSTEMTIME st;
+
+    GetLocalTime(&st);
+    time[0] = '0' + st.wHour / 10;
+    time[1] = '0' + st.wHour % 10;
+    time[2] = ':';
+    time[3] = '0' + st.wMinute / 10;
+    time[4] = '0' + st.wMinute % 10;
+    time[5] = ':';
+    time[6] = '0' + st.wSecond / 10;
+    time[7] = '0' + st.wSecond % 10;
+    time[8] = 0;
     return time;
 }
 
@@ -691,6 +711,7 @@ double CDECL _difftime32(__time32_t time1, __time32_t time2)
  */
 void CDECL _ftime64(struct __timeb64 *buf)
 {
+  TIME_ZONE_INFORMATION tzinfo;
   FILETIME ft;
   ULONGLONG time;
 
@@ -703,7 +724,7 @@ void CDECL _ftime64(struct __timeb64 *buf)
   buf->time = time / TICKSPERSEC - SECS_1601_TO_1970;
   buf->millitm = (time % TICKSPERSEC) / TICKSPERMSEC;
   buf->timezone = MSVCRT___timezone / 60;
-  buf->dstflag = GetDaylightFlag();
+  buf->dstflag = GetTimeZoneInformation( &tzinfo) == TIME_ZONE_ID_DAYLIGHT;
 }
 
 /*********************************************************************
