@@ -20,14 +20,10 @@
 
 #define COBJMACROS
 
-#include "config.h"
-
 #include <stdarg.h>
-#ifdef HAVE_LIBXML2
-# include <libxml/parser.h>
-# include <libxml/xmlerror.h>
-# include <libxml/HTMLtree.h>
-#endif
+#include <libxml/parser.h>
+#include <libxml/xmlerror.h>
+#include <libxml/HTMLtree.h>
 
 #include "windef.h"
 #include "winbase.h"
@@ -39,8 +35,6 @@
 
 #include "wine/debug.h"
 
-#ifdef HAVE_LIBXML2
-
 WINE_DEFAULT_DEBUG_CHANNEL(msxml);
 
 static const xmlChar xmlns[] = "xmlns";
@@ -738,5 +732,3 @@ IUnknown* create_attribute( xmlNodePtr attribute, BOOL floating )
 
     return (IUnknown*)&This->IXMLDOMAttribute_iface;
 }
-
-#endif
