@@ -100,11 +100,7 @@ struct norm_table
     /* WORD[]       composition character sequences */
 };
 
-LCID user_lcid = 0, system_lcid = 0;
-
 static NLSTABLEINFO nls_info;
-static HMODULE kernel32_handle;
-static CPTABLEINFO unix_table;
 static struct norm_table *norm_tables[16];
 
 
@@ -534,134 +530,6 @@ static unsigned int compose_string( const struct norm_table *info, WCHAR *str, u
 }
 
 
-void init_unix_codepage(void)
-{
-    USHORT *data = unix_funcs->get_unix_codepage_data();
-    if (data) RtlInitCodePageTable( data, &unix_table );
-}
-
-
-static LCID locale_to_lcid( WCHAR *win_name )
-{
-    WCHAR *p;
-    LCID lcid;
-
-    if (!RtlLocaleNameToLcid( win_name, &lcid, 0 )) return lcid;
-
-    /* try neutral name */
-    if ((p = wcsrchr( win_name, '-' )))
-    {
-        *p = 0;
-        if (!RtlLocaleNameToLcid( win_name, &lcid, 2 ))
-        {
-            if (SUBLANGID(lcid) == SUBLANG_NEUTRAL)
-                lcid = MAKELANGID( PRIMARYLANGID(lcid), SUBLANG_DEFAULT );
-            return lcid;
-        }
-    }
-    return 0;
-}
-
-
-/******************************************************************
- *		init_locale
- */
-void init_locale( HMODULE module )
-{
-    WCHAR system_locale[LOCALE_NAME_MAX_LENGTH];
-    WCHAR user_locale[LOCALE_NAME_MAX_LENGTH];
-
-    kernel32_handle = module;
-
-    unix_funcs->get_locales( system_locale, user_locale );
-    system_lcid = locale_to_lcid( system_locale );
-    user_lcid = locale_to_lcid( user_locale );
-    if (!system_lcid) system_lcid = MAKELCID( MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT), SORT_DEFAULT );
-    if (!user_lcid) user_lcid = system_lcid;
-
-    NtSetDefaultUILanguage( LANGIDFROMLCID(user_lcid) );
-    NtSetDefaultLocale( TRUE, user_lcid );
-    NtSetDefaultLocale( FALSE, system_lcid );
-    TRACE( "system=%04x user=%04x\n", system_lcid, user_lcid );
-}
-
-
-/******************************************************************
- *      ntdll_umbstowcs
- */
-DWORD ntdll_umbstowcs( const char *src, DWORD srclen, WCHAR *dst, DWORD dstlen )
-{
-    DWORD reslen;
-
-    if (unix_table.CodePage)
-        RtlCustomCPToUnicodeN( &unix_table, dst, dstlen * sizeof(WCHAR), &reslen, src, srclen );
-    else
-        RtlUTF8ToUnicodeN( dst, dstlen * sizeof(WCHAR), &reslen, src, srclen );
-    return reslen / sizeof(WCHAR);
-}
-
-
-/******************************************************************
- *      ntdll_wcstoumbs
- */
-int ntdll_wcstoumbs( const WCHAR *src, DWORD srclen, char *dst, DWORD dstlen, BOOL strict )
-{
-    DWORD i, reslen;
-
-    if (!unix_table.CodePage)
-        RtlUnicodeToUTF8N( dst, dstlen, &reslen, src, srclen * sizeof(WCHAR) );
-    else if (!strict)
-        RtlUnicodeToCustomCPN( &unix_table, dst, dstlen, &reslen, src, srclen * sizeof(WCHAR) );
-    else  /* do it by hand to make sure every character roundtrips correctly */
-    {
-        if (unix_table.DBCSOffsets)
-        {
-            const unsigned short *uni2cp = unix_table.WideCharTable;
-            for (i = dstlen; srclen && i; i--, srclen--, src++)
-            {
-                unsigned short ch = uni2cp[*src];
-                if (ch >> 8)
-                {
-                    if (unix_table.DBCSOffsets[unix_table.DBCSOffsets[ch >> 8] + (ch & 0xff)] != *src)
-                        return -1;
-                    if (i == 1) break;  /* do not output a partial char */
-                    i--;
-                    *dst++ = ch >> 8;
-                }
-                else
-                {
-                    if (unix_table.MultiByteTable[ch] != *src) return -1;
-                    *dst++ = (char)ch;
-                }
-            }
-            reslen = dstlen - i;
-        }
-        else
-        {
-            const unsigned char *uni2cp = unix_table.WideCharTable;
-            reslen = min( srclen, dstlen );
-            for (i = 0; i < reslen; i++)
-            {
-                unsigned char ch = uni2cp[src[i]];
-                if (unix_table.MultiByteTable[ch] != src[i]) return -1;
-                dst[i] = ch;
-            }
-        }
-    }
-    return reslen;
-}
-
-
-/******************************************************************
- *      __wine_get_unix_codepage   (NTDLL.@)
- */
-UINT CDECL __wine_get_unix_codepage(void)
-{
-    if (!unix_table.CodePage) return CP_UTF8;
-    return unix_table.CodePage;
-}
-
-
 static NTSTATUS get_dummy_preferred_ui_language( DWORD flags, LANGID lang, ULONG *count,
                                                  WCHAR *buffer, ULONG *size )
 {
@@ -1358,6 +1226,9 @@ NTSTATUS WINAPI RtlLocaleNameToLcid( const WCHAR *name, LCID *lcid, ULONG flags
             case MAKELANGID( LANG_CHINESE, SUBLANG_CHINESE_HONGKONG ):
                 *lcid = MAKELCID( 0x7c04, SORT_DEFAULT );
                 break;
+            case MAKELANGID( LANG_SERBIAN, SUBLANG_NEUTRAL ):
+                *lcid = LANG_SERBIAN_NEUTRAL;
+                break;
             default:
                 *lcid = MAKELANGID( PRIMARYLANGID(id), SUBLANG_NEUTRAL );
                 break;
