@@ -26,30 +26,23 @@
 #define BIONIC_IOCTL_NO_SIGNEDNESS_OVERLOAD  /* work around ioctl breakage on Android */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <stdarg.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <fcntl.h>
-#ifdef HAVE_SYS_IOCTL_H
+#include <dlfcn.h>
 #include <sys/ioctl.h>
-#endif
-#ifdef HAVE_SYS_MMAN_H
 #include <sys/mman.h>
-#endif
 #include <errno.h>
-#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
-#endif
 #ifdef HAVE_ASM_TYPES_H
 #include <asm/types.h>
 #endif
 #ifdef HAVE_LINUX_VIDEODEV2_H
 #include <linux/videodev2.h>
 #endif
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif
 
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
@@ -59,7 +52,7 @@
 
 #ifdef HAVE_LINUX_VIDEODEV2_H
 
-WINE_DEFAULT_DEBUG_CHANNEL(qcap);
+WINE_DEFAULT_DEBUG_CHANNEL(quartz);
 WINE_DECLARE_DEBUG_CHANNEL(winediag);
 
 static typeof(open) *video_open = open;
@@ -118,7 +111,12 @@ static int xioctl(int fd, int request, void * arg)
     return r;
 }
 
-static void CDECL v4l_device_destroy(struct video_capture_device *device)
+static struct video_capture_device *get_device( video_capture_device_t dev )
+{
+    return (struct video_capture_device *)(ULONG_PTR)dev;
+}
+
+static void device_destroy(struct video_capture_device *device)
 {
     if (device->fd != -1)
         video_close(device->fd);
@@ -148,17 +146,17 @@ static const struct caps *find_caps(struct video_capture_device *device, const A
     return NULL;
 }
 
-static HRESULT CDECL v4l_device_check_format(struct video_capture_device *device, const AM_MEDIA_TYPE *mt)
+static NTSTATUS v4l_device_check_format( void *args )
 {
-    TRACE("device %p, mt %p.\n", device, mt);
+    const struct check_format_params *params = args;
+    struct video_capture_device *device = get_device(params->device);
 
-    if (!mt)
-        return E_POINTER;
+    TRACE("device %p, mt %p.\n", device, params->mt);
 
-    if (!IsEqualGUID(&mt->majortype, &MEDIATYPE_Video))
+    if (!IsEqualGUID(&params->mt->majortype, &MEDIATYPE_Video))
         return E_FAIL;
 
-    if (find_caps(device, mt))
+    if (find_caps(device, params->mt))
         return S_OK;
 
     return E_FAIL;
@@ -202,11 +200,13 @@ static HRESULT set_caps(struct video_capture_device *device, const struct caps *
     return S_OK;
 }
 
-static HRESULT CDECL v4l_device_set_format(struct video_capture_device *device, const AM_MEDIA_TYPE *mt)
+static NTSTATUS v4l_device_set_format( void *args )
 {
+    const struct set_format_params *params = args;
+    struct video_capture_device *device = get_device(params->device);
     const struct caps *caps;
 
-    caps = find_caps(device, mt);
+    caps = find_caps(device, params->mt);
     if (!caps)
         return E_FAIL;
 
@@ -216,29 +216,34 @@ static HRESULT CDECL v4l_device_set_format(struct video_capture_device *device,
     return set_caps(device, caps);
 }
 
-static void CDECL v4l_device_get_format(struct video_capture_device *device, AM_MEDIA_TYPE *mt, VIDEOINFOHEADER *format)
+static NTSTATUS v4l_device_get_format( void *args )
 {
-    *mt = device->current_caps->media_type;
-    *format = device->current_caps->video_info;
+    const struct get_format_params *params = args;
+    struct video_capture_device *device = get_device(params->device);
+
+    *params->mt = device->current_caps->media_type;
+    *params->format = device->current_caps->video_info;
+    return S_OK;
 }
 
-static HRESULT CDECL v4l_device_get_media_type(struct video_capture_device *device,
-        unsigned int index, AM_MEDIA_TYPE *mt, VIDEOINFOHEADER *format)
+static NTSTATUS v4l_device_get_media_type( void *args )
 {
+    const struct get_media_type_params *params = args;
+    struct video_capture_device *device = get_device(params->device);
     unsigned int caps_count = (device->current_caps) ? 1 : device->caps_count;
 
-    if (index >= caps_count)
+    if (params->index >= caps_count)
         return VFW_S_NO_MORE_ITEMS;
 
     if (device->current_caps)
     {
-        *mt = device->current_caps->media_type;
-        *format = device->current_caps->video_info;
+        *params->mt = device->current_caps->media_type;
+        *params->format = device->current_caps->video_info;
     }
     else
     {
-        *mt = device->caps[index].media_type;
-        *format = device->caps[index].video_info;
+        *params->mt = device->caps[params->index].media_type;
+        *params->format = device->caps[params->index].video_info;
     }
     return S_OK;
 }
@@ -261,12 +266,13 @@ static __u32 v4l2_cid_from_qcap_property(VideoProcAmpProperty property)
     }
 }
 
-static HRESULT CDECL v4l_device_get_prop_range(struct video_capture_device *device, VideoProcAmpProperty property,
-        LONG *min, LONG *max, LONG *step, LONG *default_value, LONG *flags)
+static NTSTATUS v4l_device_get_prop_range( void *args )
 {
+    const struct get_prop_range_params *params = args;
+    struct video_capture_device *device = get_device(params->device);
     struct v4l2_queryctrl ctrl;
 
-    ctrl.id = v4l2_cid_from_qcap_property(property);
+    ctrl.id = v4l2_cid_from_qcap_property(params->property);
 
     if (xioctl(device->fd, VIDIOC_QUERYCTRL, &ctrl) == -1)
     {
@@ -274,20 +280,21 @@ static HRESULT CDECL v4l_device_get_prop_range(struct video_capture_device *devi
         return E_PROP_ID_UNSUPPORTED;
     }
 
-    *min = ctrl.minimum;
-    *max = ctrl.maximum;
-    *step = ctrl.step;
-    *default_value = ctrl.default_value;
-    *flags = VideoProcAmp_Flags_Manual;
+    *params->min = ctrl.minimum;
+    *params->max = ctrl.maximum;
+    *params->step = ctrl.step;
+    *params->default_value = ctrl.default_value;
+    *params->flags = VideoProcAmp_Flags_Manual;
     return S_OK;
 }
 
-static HRESULT CDECL v4l_device_get_prop(struct video_capture_device *device,
-        VideoProcAmpProperty property, LONG *value, LONG *flags)
+static NTSTATUS v4l_device_get_prop( void *args )
 {
+    const struct get_prop_params *params = args;
+    struct video_capture_device *device = get_device(params->device);
     struct v4l2_control ctrl;
 
-    ctrl.id = v4l2_cid_from_qcap_property(property);
+    ctrl.id = v4l2_cid_from_qcap_property(params->property);
 
     if (xioctl(device->fd, VIDIOC_G_CTRL, &ctrl) == -1)
     {
@@ -295,19 +302,20 @@ static HRESULT CDECL v4l_device_get_prop(struct video_capture_device *device,
         return E_FAIL;
     }
 
-    *value = ctrl.value;
-    *flags = VideoProcAmp_Flags_Manual;
+    *params->value = ctrl.value;
+    *params->flags = VideoProcAmp_Flags_Manual;
 
     return S_OK;
 }
 
-static HRESULT CDECL v4l_device_set_prop(struct video_capture_device *device,
-        VideoProcAmpProperty property, LONG value, LONG flags)
+static NTSTATUS v4l_device_set_prop( void *args )
 {
+    const struct set_prop_params *params = args;
+    struct video_capture_device *device = get_device(params->device);
     struct v4l2_control ctrl;
 
-    ctrl.id = v4l2_cid_from_qcap_property(property);
-    ctrl.value = value;
+    ctrl.id = v4l2_cid_from_qcap_property(params->property);
+    ctrl.value = params->value;
 
     if (xioctl(device->fd, VIDIOC_S_CTRL, &ctrl) == -1)
     {
@@ -337,8 +345,11 @@ static void reverse_image(struct video_capture_device *device, LPBYTE output, co
     }
 }
 
-static BOOL CDECL v4l_device_read_frame(struct video_capture_device *device, BYTE *data)
+static NTSTATUS v4l_device_read_frame( void *args )
 {
+    const struct read_frame_params *params = args;
+    struct video_capture_device *device = get_device(params->device);
+
     while (video_read(device->fd, device->image_data, device->image_size) < 0)
     {
         if (errno != EAGAIN)
@@ -348,7 +359,7 @@ static BOOL CDECL v4l_device_read_frame(struct video_capture_device *device, BYT
         }
     }
 
-    reverse_image(device, data, device->image_data);
+    reverse_image(device, params->data, device->image_data);
     return TRUE;
 }
 
@@ -388,21 +399,29 @@ static void fill_caps(__u32 pixelformat, __u32 width, __u32 height,
     caps->pixelformat = pixelformat;
 }
 
-static void CDECL v4l_device_get_caps(struct video_capture_device *device, LONG index,
-        AM_MEDIA_TYPE *type, VIDEOINFOHEADER *format, VIDEO_STREAM_CONFIG_CAPS *vscc)
+static NTSTATUS v4l_device_get_caps( void *args )
 {
-    *vscc = device->caps[index].config;
-    *type = device->caps[index].media_type;
-    *format = device->caps[index].video_info;
+    const struct get_caps_params *params = args;
+    struct video_capture_device *device = get_device(params->device);
+
+    *params->caps   = device->caps[params->index].config;
+    *params->mt     = device->caps[params->index].media_type;
+    *params->format = device->caps[params->index].video_info;
+    return S_OK;
 }
 
-static LONG CDECL v4l_device_get_caps_count(struct video_capture_device *device)
+static NTSTATUS v4l_device_get_caps_count( void *args )
 {
-    return device->caps_count;
+    const struct get_caps_count_params *params = args;
+    struct video_capture_device *device = get_device(params->device);
+
+    *params->count = device->caps_count;
+    return S_OK;
 }
 
-struct video_capture_device * CDECL v4l_device_create(USHORT index)
+static NTSTATUS v4l_device_create( void *args )
 {
+    const struct create_params *params = args;
     struct v4l2_frmsizeenum frmsize = {0};
     struct video_capture_device *device;
     struct v4l2_capability caps = {{0}};
@@ -415,9 +434,9 @@ struct video_capture_device * CDECL v4l_device_create(USHORT index)
     have_libv4l2 = video_init();
 
     if (!(device = calloc(1, sizeof(*device))))
-        return NULL;
+        return E_OUTOFMEMORY;
 
-    sprintf(path, "/dev/video%i", index);
+    sprintf(path, "/dev/video%i", params->index);
     TRACE("Opening device %s.\n", path);
 #ifdef O_CLOEXEC
     if ((fd = video_open(path, O_RDWR | O_NONBLOCK | O_CLOEXEC)) == -1 && errno == EINVAL)
@@ -542,35 +561,302 @@ struct video_capture_device * CDECL v4l_device_create(USHORT index)
             device->current_caps->video_info.bmiHeader.biWidth,
             device->current_caps->video_info.bmiHeader.biHeight);
 
-    return device;
+    *params->device = (ULONG_PTR)device;
+    return S_OK;
 
 error:
-    v4l_device_destroy(device);
-    return NULL;
+    device_destroy(device);
+    return E_FAIL;
 }
 
-const struct video_capture_funcs v4l_funcs =
+static NTSTATUS v4l_device_destroy( void *args )
 {
-    .create = v4l_device_create,
-    .destroy = v4l_device_destroy,
-    .check_format = v4l_device_check_format,
-    .set_format = v4l_device_set_format,
-    .get_format = v4l_device_get_format,
-    .get_media_type = v4l_device_get_media_type,
-    .get_caps = v4l_device_get_caps,
-    .get_caps_count = v4l_device_get_caps_count,
-    .get_prop_range = v4l_device_get_prop_range,
-    .get_prop = v4l_device_get_prop,
-    .set_prop = v4l_device_set_prop,
-    .read_frame = v4l_device_read_frame,
+    const struct destroy_params *params = args;
+
+    device_destroy( get_device(params->device) );
+    return S_OK;
+}
+
+const unixlib_entry_t __wine_unix_call_funcs[] =
+{
+    v4l_device_create,
+    v4l_device_destroy,
+    v4l_device_check_format,
+    v4l_device_set_format,
+    v4l_device_get_format,
+    v4l_device_get_media_type,
+    v4l_device_get_caps,
+    v4l_device_get_caps_count,
+    v4l_device_get_prop_range,
+    v4l_device_get_prop,
+    v4l_device_set_prop,
+    v4l_device_read_frame,
 };
 
-NTSTATUS CDECL __wine_init_unix_lib(HMODULE module, DWORD reason, const void *ptr_in, void *ptr_out)
-{
-    if (reason != DLL_PROCESS_ATTACH) return STATUS_SUCCESS;
+#ifdef _WIN64
 
-    *(const struct video_capture_funcs **)ptr_out = &v4l_funcs;
-    return STATUS_SUCCESS;
+typedef ULONG PTR32;
+
+struct am_media_type32
+{
+    GUID       majortype;
+    GUID       subtype;
+    BOOL       bFixedSizeSamples;
+    BOOL       bTemporalCompression;
+    ULONG      lSampleSize;
+    GUID       formattype;
+    PTR32      pUnk;
+    ULONG      cbFormat;
+    PTR32      pbFormat;
+};
+
+static AM_MEDIA_TYPE *get_media_type( const struct am_media_type32 *mt32, AM_MEDIA_TYPE *mt )
+{
+    mt->majortype            = mt32->majortype;
+    mt->subtype              = mt32->subtype;
+    mt->bFixedSizeSamples    = mt32->bFixedSizeSamples;
+    mt->bTemporalCompression = mt32->bTemporalCompression;
+    mt->lSampleSize          = mt32->lSampleSize;
+    mt->formattype           = mt32->formattype;
+    mt->pUnk                 = NULL;
+    mt->cbFormat             = mt32->cbFormat;
+    mt->pbFormat             = ULongToPtr(mt32->pbFormat);
+    return mt;
 }
 
+static void put_media_type( const AM_MEDIA_TYPE *mt, struct am_media_type32 *mt32 )
+{
+    mt32->majortype            = mt->majortype;
+    mt32->subtype              = mt->subtype;
+    mt32->bFixedSizeSamples    = mt->bFixedSizeSamples;
+    mt32->bTemporalCompression = mt->bTemporalCompression;
+    mt32->lSampleSize          = mt->lSampleSize;
+    mt32->formattype           = mt->formattype;
+}
+
+static NTSTATUS wow64_v4l_device_create( void *args )
+{
+    struct
+    {
+        unsigned int index;
+        PTR32 device;
+    } const *params32 = args;
+
+    struct create_params params =
+    {
+        params32->index,
+        ULongToPtr(params32->device)
+    };
+
+    return v4l_device_create( &params );
+}
+
+static NTSTATUS wow64_v4l_device_check_format( void *args )
+{
+    struct
+    {
+        video_capture_device_t device;
+        PTR32                  mt;
+    } const *params32 = args;
+
+    AM_MEDIA_TYPE mt;
+    struct check_format_params params =
+    {
+        params32->device,
+        get_media_type( ULongToPtr(params32->mt), &mt )
+    };
+
+    return v4l_device_check_format( &params );
+}
+
+static NTSTATUS wow64_v4l_device_set_format( void *args )
+{
+    struct
+    {
+        video_capture_device_t device;
+        PTR32                  mt;
+    } const *params32 = args;
+
+    AM_MEDIA_TYPE mt;
+    struct set_format_params params =
+    {
+        params32->device,
+        get_media_type( ULongToPtr(params32->mt), &mt )
+    };
+
+    return v4l_device_set_format( &params );
+}
+
+static NTSTATUS wow64_v4l_device_get_format( void *args )
+{
+    struct
+    {
+        video_capture_device_t device;
+        PTR32                  mt;
+        PTR32                  format;
+    } const *params32 = args;
+
+    AM_MEDIA_TYPE mt;
+    struct get_format_params params =
+    {
+        params32->device,
+        &mt,
+        ULongToPtr(params32->format)
+    };
+
+    NTSTATUS status = v4l_device_get_format( &params );
+    if (!status) put_media_type( &mt, ULongToPtr(params32->mt) );
+    return status;
+}
+
+static NTSTATUS wow64_v4l_device_get_media_type( void *args )
+{
+    struct
+    {
+        video_capture_device_t device;
+        unsigned int           index;
+        PTR32                  mt;
+        PTR32                  format;
+    } const *params32 = args;
+
+    AM_MEDIA_TYPE mt;
+    struct get_media_type_params params =
+    {
+        params32->device,
+        params32->index,
+        &mt,
+        ULongToPtr(params32->format)
+    };
+
+    NTSTATUS status = v4l_device_get_media_type( &params );
+    if (!status) put_media_type( &mt, ULongToPtr(params32->mt) );
+    return status;
+}
+
+static NTSTATUS wow64_v4l_device_get_caps( void *args )
+{
+    struct
+    {
+        video_capture_device_t device;
+        unsigned int           index;
+        PTR32                  mt;
+        PTR32                  format;
+        PTR32                  caps;
+    } const *params32 = args;
+
+    AM_MEDIA_TYPE mt;
+    struct get_caps_params params =
+    {
+        params32->device,
+        params32->index,
+        &mt,
+        ULongToPtr(params32->format),
+        ULongToPtr(params32->caps)
+    };
+
+    NTSTATUS status = v4l_device_get_caps( &params );
+    if (!status) put_media_type( &mt, ULongToPtr(params32->mt) );
+    return status;
+}
+
+static NTSTATUS wow64_v4l_device_get_caps_count( void *args )
+{
+    struct
+    {
+        video_capture_device_t device;
+        PTR32                  count;
+    } const *params32 = args;
+
+    struct get_caps_count_params params =
+    {
+        params32->device,
+        ULongToPtr(params32->count)
+    };
+
+    return v4l_device_get_caps_count( &params );
+}
+
+static NTSTATUS wow64_v4l_device_get_prop_range( void *args )
+{
+    struct
+    {
+        video_capture_device_t device;
+        VideoProcAmpProperty   property;
+        PTR32                  min;
+        PTR32                  max;
+        PTR32                  step;
+        PTR32                  default_value;
+        PTR32                  flags;
+    } const *params32 = args;
+
+    struct get_prop_range_params params =
+    {
+        params32->device,
+        params32->property,
+        ULongToPtr(params32->min),
+        ULongToPtr(params32->max),
+        ULongToPtr(params32->step),
+        ULongToPtr(params32->default_value),
+        ULongToPtr(params32->flags)
+    };
+
+    return v4l_device_get_prop_range( &params );
+}
+
+static NTSTATUS wow64_v4l_device_get_prop( void *args )
+{
+    struct
+    {
+        video_capture_device_t device;
+        VideoProcAmpProperty   property;
+        PTR32                  value;
+        PTR32                  flags;
+    } const *params32 = args;
+
+    struct get_prop_params params =
+    {
+        params32->device,
+        params32->property,
+        ULongToPtr(params32->value),
+        ULongToPtr(params32->flags)
+    };
+
+    return v4l_device_get_prop( &params );
+}
+
+static NTSTATUS wow64_v4l_device_read_frame( void *args )
+{
+    struct
+    {
+        video_capture_device_t device;
+        PTR32                  data;
+    } const *params32 = args;
+
+    struct read_frame_params params =
+    {
+        params32->device,
+        ULongToPtr(params32->data)
+    };
+
+    return v4l_device_read_frame( &params );
+}
+
+const unixlib_entry_t __wine_unix_call_wow64_funcs[] =
+{
+    wow64_v4l_device_create,
+    v4l_device_destroy,
+    wow64_v4l_device_check_format,
+    wow64_v4l_device_set_format,
+    wow64_v4l_device_get_format,
+    wow64_v4l_device_get_media_type,
+    wow64_v4l_device_get_caps,
+    wow64_v4l_device_get_caps_count,
+    wow64_v4l_device_get_prop_range,
+    wow64_v4l_device_get_prop,
+    v4l_device_set_prop,
+    wow64_v4l_device_read_frame,
+};
+
+#endif /* _WIN64 */
+
 #endif /* HAVE_LINUX_VIDEODEV2_H */
