@@ -24,7 +24,6 @@
 #include "initguid.h"
 #include "rtworkq.h"
 #include "wine/debug.h"
-#include "wine/heap.h"
 #include "wine/list.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mfplat);
@@ -58,6 +57,7 @@ static struct queue_handle user_queues[MAX_USER_QUEUE_HANDLES];
 static struct queue_handle *next_free_user_queue;
 static struct queue_handle *next_unused_user_queue = user_queues;
 static WORD queue_generation;
+static DWORD shared_mt_queue;
 
 static CRITICAL_SECTION queues_section;
 static CRITICAL_SECTION_DEBUG queues_critsect_debug =
@@ -217,8 +217,9 @@ static HRESULT unlock_user_queue(DWORD queue)
     {
         if (--entry->refcount == 0)
         {
+            if (shared_mt_queue == queue) shared_mt_queue = 0;
             shutdown_queue((struct queue *)entry->obj);
-            heap_free(entry->obj);
+            free(entry->obj);
             entry->obj = next_free_user_queue;
             next_free_user_queue = entry;
         }
@@ -553,7 +554,7 @@ static ULONG WINAPI work_item_Release(IUnknown *iface)
         if (item->reply_result)
             IRtwqAsyncResult_Release(item->reply_result);
         IRtwqAsyncResult_Release(item->result);
-        heap_free(item);
+        free(item);
     }
 
     return refcount;
@@ -572,7 +573,7 @@ static struct work_item * alloc_work_item(struct queue *queue, LONG priority, IR
     DWORD flags = 0, queue_id = 0;
     struct work_item *item;
 
-    item = heap_alloc_zero(sizeof(*item));
+    item = calloc(1, sizeof(*item));
 
     item->IUnknown_iface.lpVtbl = &work_item_vtbl;
     item->result = result;
@@ -896,8 +897,7 @@ static HRESULT alloc_user_queue(const struct queue_desc *desc, DWORD *queue_id)
     if (platform_lock <= 0)
         return RTWQ_E_SHUTDOWN;
 
-    queue = heap_alloc_zero(sizeof(*queue));
-    if (!queue)
+    if (!(queue = calloc(1, sizeof(*queue))))
         return E_OUTOFMEMORY;
 
     init_work_queue(desc, queue);
@@ -912,7 +912,7 @@ static HRESULT alloc_user_queue(const struct queue_desc *desc, DWORD *queue_id)
     else
     {
         LeaveCriticalSection(&queues_section);
-        heap_free(queue);
+        free(queue);
         WARN("Out of user queue handles.\n");
         return E_OUTOFMEMORY;
     }
@@ -986,7 +986,7 @@ static ULONG WINAPI async_result_Release(IRtwqAsyncResult *iface)
             IUnknown_Release(result->state);
         if (result->result.hEvent)
             CloseHandle(result->result.hEvent);
-        heap_free(result);
+        free(result);
 
         RtwqUnlockPlatform();
     }
@@ -1072,8 +1072,7 @@ static HRESULT create_async_result(IUnknown *object, IRtwqAsyncCallback *callbac
     if (!out)
         return E_INVALIDARG;
 
-    result = heap_alloc_zero(sizeof(*result));
-    if (!result)
+    if (!(result = calloc(1, sizeof(*result))))
         return E_OUTOFMEMORY;
 
     RtwqLockPlatform();
@@ -1265,7 +1264,7 @@ static ULONG WINAPI periodic_callback_Release(IRtwqAsyncCallback *iface)
     TRACE("%p, %u.\n", iface, refcount);
 
     if (!refcount)
-        heap_free(callback);
+        free(callback);
 
     return refcount;
 }
@@ -1304,8 +1303,7 @@ static HRESULT create_periodic_callback_obj(RTWQPERIODICCALLBACK callback, IRtwq
 {
     struct periodic_callback *object;
 
-    object = heap_alloc(sizeof(*object));
-    if (!object)
+    if (!(object = calloc(1, sizeof(*object))))
         return E_OUTOFMEMORY;
 
     object->IRtwqAsyncCallback_iface.lpVtbl = &periodic_callback_vtbl;
@@ -1438,9 +1436,37 @@ HRESULT WINAPI RtwqSetLongRunning(DWORD queue_id, BOOL enable)
 
 HRESULT WINAPI RtwqLockSharedWorkQueue(const WCHAR *usageclass, LONG priority, DWORD *taskid, DWORD *queue)
 {
-    FIXME("%s, %d, %p, %p.\n", debugstr_w(usageclass), priority, taskid, queue);
+    struct queue_desc desc;
+    HRESULT hr;
 
-    return RtwqAllocateWorkQueue(RTWQ_STANDARD_WORKQUEUE, queue);
+    TRACE("%s, %d, %p, %p.\n", debugstr_w(usageclass), priority, taskid, queue);
+
+    if (!usageclass)
+        return E_POINTER;
+
+    if (!*usageclass && taskid)
+        return E_INVALIDARG;
+
+    if (*usageclass)
+        FIXME("Class name is ignored.\n");
+
+    EnterCriticalSection(&queues_section);
+
+    if (shared_mt_queue)
+        hr = lock_user_queue(shared_mt_queue);
+    else
+    {
+        desc.queue_type = RTWQ_MULTITHREADED_WORKQUEUE;
+        desc.ops = &pool_queue_ops;
+        desc.target_queue = 0;
+        hr = alloc_user_queue(&desc, &shared_mt_queue);
+    }
+
+    *queue = shared_mt_queue;
+
+    LeaveCriticalSection(&queues_section);
+
+    return hr;
 }
 
 HRESULT WINAPI RtwqSetDeadline(DWORD queue_id, LONGLONG deadline, HANDLE *request)
