@@ -27,6 +27,7 @@
 #include "winuser.h"
 #include "initguid.h"
 #include "ocidl.h"
+#include "featurestagingapi.h"
 #include "shellscalingapi.h"
 #include "shlwapi.h"
 
@@ -44,8 +45,6 @@ BOOL WINAPI DllMain(HINSTANCE instance, DWORD reason, void *reserved)
 
     switch (reason)
     {
-        case DLL_WINE_PREATTACH:
-            return FALSE;  /* prefer native version */
         case DLL_PROCESS_ATTACH:
             DisableThreadLibraryCalls(instance);
             shcore_tls = TlsAlloc();
@@ -2507,3 +2506,21 @@ BOOL WINAPI IsOS(DWORD feature)
 
     return FALSE;
 }
+
+/*************************************************************************
+ * SubscribeFeatureStateChangeNotification        [SHCORE.@]
+ */
+void WINAPI SubscribeFeatureStateChangeNotification(FEATURE_STATE_CHANGE_SUBSCRIPTION *subscription,
+                                                    FEATURE_STATE_CHANGE_CALLBACK *callback, void *context)
+{
+    FIXME("(%p, %p, %p) stub\n", subscription, callback, context);
+}
+
+/*************************************************************************
+ * GetFeatureEnabledState        [SHCORE.@]
+ */
+FEATURE_ENABLED_STATE WINAPI GetFeatureEnabledState(UINT32 feature, FEATURE_CHANGE_TIME change_time)
+{
+    FIXME("(%u, %u) stub\n", feature, change_time);
+    return FEATURE_ENABLED_STATE_DEFAULT;
+}
