@@ -19,9 +19,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
@@ -245,13 +242,6 @@ static HRESULT WINAPI ISF_MyComputer_fnParseDisplayName (IShellFolder2 *iface,
 /* retrieve a map of drives that should be displayed */
 static DWORD get_drive_map(void)
 {
-    static const WCHAR policiesW[] = {'S','o','f','t','w','a','r','e','\\',
-                                      'M','i','c','r','o','s','o','f','t','\\',
-                                      'W','i','n','d','o','w','s','\\',
-                                      'C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\',
-                                      'P','o','l','i','c','i','e','s','\\',
-                                      'E','x','p','l','o','r','e','r',0};
-    static const WCHAR nodrivesW[] = {'N','o','D','r','i','v','e','s',0};
     static DWORD drive_mask;
     static BOOL init_done = FALSE;
 
@@ -260,17 +250,17 @@ static DWORD get_drive_map(void)
         DWORD type, size, data, mask = 0;
         HKEY hkey;
 
-        if (!RegOpenKeyW( HKEY_LOCAL_MACHINE, policiesW, &hkey ))
+        if (!RegOpenKeyW( HKEY_LOCAL_MACHINE, L"Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer", &hkey ))
         {
             size = sizeof(data);
-            if (!RegQueryValueExW( hkey, nodrivesW, NULL, &type, (LPBYTE)&data, &size ) && type == REG_DWORD)
+            if (!RegQueryValueExW( hkey, L"NoDrives", NULL, &type, (LPBYTE)&data, &size ) && type == REG_DWORD)
                 mask |= data;
             RegCloseKey( hkey );
         }
-        if (!RegOpenKeyW( HKEY_CURRENT_USER, policiesW, &hkey ))
+        if (!RegOpenKeyW( HKEY_CURRENT_USER, L"Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer", &hkey ))
         {
             size = sizeof(data);
-            if (!RegQueryValueExW( hkey, nodrivesW, NULL, &type, (LPBYTE)&data, &size ) && type == REG_DWORD)
+            if (!RegQueryValueExW( hkey, L"NoDrives", NULL, &type, (LPBYTE)&data, &size ) && type == REG_DWORD)
                 mask |= data;
             RegCloseKey( hkey );
         }
@@ -284,12 +274,6 @@ static DWORD get_drive_map(void)
 /**************************************************************************
  *  CreateMyCompEnumList()
  */
-static const WCHAR MyComputer_NameSpaceW[] = { 'S','O','F','T','W','A','R','E',
- '\\','M','i','c','r','o','s','o','f','t','\\','W','i','n','d','o','w','s','\\',
- 'C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\','E','x','p','l',
- 'o','r','e','r','\\','M','y','C','o','m','p','u','t','e','r','\\','N','a','m',
- 'e','s','p','a','c','e','\0' };
-
 static BOOL CreateMyCompEnumList(IEnumIDListImpl *list, DWORD dwFlags)
 {
     BOOL ret = TRUE;
@@ -299,7 +283,7 @@ static BOOL CreateMyCompEnumList(IEnumIDListImpl *list, DWORD dwFlags)
     /* enumerate the folders */
     if (dwFlags & SHCONTF_FOLDERS)
     {
-        WCHAR wszDriveName[] = {'A', ':', '\\', '\0'};
+        WCHAR wszDriveName[] = L"A:\\";
         DWORD dwDrivemap = get_drive_map();
         HKEY hkey;
         UINT i;
@@ -315,7 +299,8 @@ static BOOL CreateMyCompEnumList(IEnumIDListImpl *list, DWORD dwFlags)
         TRACE("-- (%p)-> enumerate (mycomputer shell extensions)\n",list);
         for (i=0; i<2; i++) {
             if (ret && !RegOpenKeyExW(i == 0 ? HKEY_LOCAL_MACHINE : HKEY_CURRENT_USER,
-                                      MyComputer_NameSpaceW, 0, KEY_READ, &hkey))
+                                      L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MyComputer\\Namespace",
+                                      0, KEY_READ, &hkey))
             {
                 WCHAR iid[50];
                 int i=0;
@@ -379,7 +364,7 @@ static HRESULT WINAPI ISF_MyComputer_fnBindToObject (IShellFolder2 *iface,
     TRACE("(%p)->(pidl=%p,%p,%s,%p)\n", This,
           pidl, pbcReserved, shdebugstr_guid (riid), ppvOut);
 
-    return SHELL32_BindToChild (This->pidlRoot, NULL, pidl, riid, ppvOut);
+    return SHELL32_BindToChild (This->pidlRoot, &CLSID_ShellFSFolder, NULL, pidl, riid, ppvOut);
 }
 
 /**************************************************************************
@@ -616,13 +601,6 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDisplayNameOf (IShellFolder2 *iface,
             {
                 if ((GET_SHGDN_FOR (dwFlags) & (SHGDN_FORPARSING | SHGDN_FORADDRESSBAR)) == SHGDN_FORPARSING)
                 {
-                    static const WCHAR clsidW[] =
-                     { 'C','L','S','I','D','\\',0 };
-                    static const WCHAR shellfolderW[] =
-                     { '\\','s','h','e','l','l','f','o','l','d','e','r',0 };
-                    static const WCHAR wantsForParsingW[] =
-                     { 'W','a','n','t','s','F','o','r','P','a','r','s','i','n',
-                     'g',0 };
                     BOOL bWantsForParsing = FALSE;
                     WCHAR szRegPath[100];
                     LONG r;
@@ -638,11 +616,10 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDisplayNameOf (IShellFolder2 *iface,
                      * Get the "WantsFORPARSING" flag from the registry
                      */
 
-                    lstrcpyW (szRegPath, clsidW);
+                    lstrcpyW (szRegPath, L"CLSID\\");
                     SHELL32_GUIDToStringW (clsid, &szRegPath[6]);
-                    lstrcatW (szRegPath, shellfolderW);
-                    r = SHGetValueW (HKEY_CLASSES_ROOT, szRegPath, 
-                                     wantsForParsingW, NULL, NULL, NULL);
+                    lstrcatW (szRegPath, L"\\shellfolder");
+                    r = SHGetValueW (HKEY_CLASSES_ROOT, szRegPath, L"WantsForParsing", NULL, NULL, NULL);
                     if (r == ERROR_SUCCESS)
                         bWantsForParsing = TRUE;
 
@@ -693,16 +670,14 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDisplayNameOf (IShellFolder2 *iface,
             /* long view "lw_name (C:)" */
             if (!(dwFlags & SHGDN_FORPARSING))
             {
-                static const WCHAR wszOpenBracket[] = {' ','(',0};
-                static const WCHAR wszCloseBracket[] = {')',0};
                 WCHAR wszDrive[32 /* label */ + 6 /* ' (C:)'\0 */] = {0};
 
                 GetVolumeInformationW (pszPath, wszDrive, ARRAY_SIZE(wszDrive) - 5, NULL, NULL,
                         NULL, NULL, 0);
-                strcatW (wszDrive, wszOpenBracket);
-                lstrcpynW (wszDrive + strlenW(wszDrive), pszPath, 3);
-                strcatW (wszDrive, wszCloseBracket);
-                strcpyW (pszPath, wszDrive);
+                lstrcatW (wszDrive, L" (");
+                lstrcpynW (wszDrive + lstrlenW(wszDrive), pszPath, 3);
+                lstrcatW (wszDrive, L")");
+                lstrcpyW (pszPath, wszDrive);
             }
         }
         else 
@@ -834,13 +809,6 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDetailsOf (IShellFolder2 *iface,
 
     switch (iColumn)
     {
-        case 0:        /* name */
-            hr = IShellFolder2_GetDisplayNameOf (iface, pidl,
-                       SHGDN_NORMAL | SHGDN_INFOLDER, &psd->str);
-            break;
-        case 1:        /* type */
-            _ILGetFileType (pidl, psd->str.u.cStr, MAX_PATH);
-            break;
         case 2:        /* total size */
             if (_ILIsDrive (pidl))
             {
@@ -857,6 +825,9 @@ static HRESULT WINAPI ISF_MyComputer_fnGetDetailsOf (IShellFolder2 *iface,
                 StrFormatByteSizeA (ulBytes.u.LowPart, psd->str.u.cStr, MAX_PATH);
             }
             break;
+
+        default:
+            return shellfolder_get_file_details( iface, pidl, mycomputer_header, iColumn, psd );
     }
 
     return hr;
