@@ -125,33 +125,18 @@ HWINSTA WINAPI CreateWindowStationA( LPCSTR name, DWORD flags, ACCESS_MASK acces
 HWINSTA WINAPI CreateWindowStationW( LPCWSTR name, DWORD flags, ACCESS_MASK access,
                                      LPSECURITY_ATTRIBUTES sa )
 {
-    HANDLE ret;
-    DWORD len = name ? lstrlenW(name) : 0;
+    OBJECT_ATTRIBUTES attr;
+    UNICODE_STRING str;
 
-    if (len >= MAX_PATH)
-    {
-        SetLastError( ERROR_FILENAME_EXCED_RANGE );
-        return 0;
-    }
-    if (!len)
-    {
-        name = get_winstation_default_name();
-        len = lstrlenW( name );
-    }
-    SERVER_START_REQ( create_winstation )
-    {
-        req->flags      = 0;
-        req->access     = access;
-        req->attributes = OBJ_CASE_INSENSITIVE |
-                          ((flags & CWF_CREATE_ONLY) ? 0 : OBJ_OPENIF) |
-                          ((sa && sa->bInheritHandle) ? OBJ_INHERIT : 0);
-        req->rootdir    = wine_server_obj_handle( get_winstations_dir_handle() );
-        wine_server_add_data( req, name, len * sizeof(WCHAR) );
-        wine_server_call_err( req );
-        ret = wine_server_ptr_handle( reply->handle );
-    }
-    SERVER_END_REQ;
-    return ret;
+    RtlInitUnicodeString( &str, name );
+    if (!str.Length) RtlInitUnicodeString( &str, get_winstation_default_name() );
+
+    InitializeObjectAttributes( &attr, &str, OBJ_CASE_INSENSITIVE,
+                                get_winstations_dir_handle(), sa );
+    if (!(flags & CWF_CREATE_ONLY)) attr.Attributes |= OBJ_OPENIF;
+    if (sa && sa->bInheritHandle) attr.Attributes |= OBJ_INHERIT;
+
+    return NtUserCreateWindowStation( &attr, access, 0, 0, 0, 0, 0 );
 }
 
 
@@ -178,78 +163,17 @@ HWINSTA WINAPI OpenWindowStationA( LPCSTR name, BOOL inherit, ACCESS_MASK access
  */
 HWINSTA WINAPI OpenWindowStationW( LPCWSTR name, BOOL inherit, ACCESS_MASK access )
 {
-    HANDLE ret = 0;
-    DWORD len = name ? lstrlenW(name) : 0;
-    if (len >= MAX_PATH)
-    {
-        SetLastError( ERROR_FILENAME_EXCED_RANGE );
-        return 0;
-    }
-    if (!len)
-    {
-        name = get_winstation_default_name();
-        len = lstrlenW( name );
-    }
-    SERVER_START_REQ( open_winstation )
-    {
-        req->access     = access;
-        req->attributes = OBJ_CASE_INSENSITIVE | (inherit ? OBJ_INHERIT : 0);
-        req->rootdir    = wine_server_obj_handle( get_winstations_dir_handle() );
-        wine_server_add_data( req, name, len * sizeof(WCHAR) );
-        if (!wine_server_call_err( req )) ret = wine_server_ptr_handle( reply->handle );
-    }
-    SERVER_END_REQ;
-    return ret;
-}
+    OBJECT_ATTRIBUTES attr;
+    UNICODE_STRING str;
 
+    RtlInitUnicodeString( &str, name );
+    if (!str.Length) RtlInitUnicodeString( &str, get_winstation_default_name() );
 
-/***********************************************************************
- *              CloseWindowStation  (USER32.@)
- */
-BOOL WINAPI CloseWindowStation( HWINSTA handle )
-{
-    BOOL ret;
-    SERVER_START_REQ( close_winstation )
-    {
-        req->handle = wine_server_obj_handle( handle );
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
-    return ret;
-}
+    InitializeObjectAttributes( &attr, &str, OBJ_CASE_INSENSITIVE,
+                                get_winstations_dir_handle(), NULL );
+    if (inherit) attr.Attributes |= OBJ_INHERIT;
 
-
-/******************************************************************************
- *              GetProcessWindowStation  (USER32.@)
- */
-HWINSTA WINAPI GetProcessWindowStation(void)
-{
-    HWINSTA ret = 0;
-
-    SERVER_START_REQ( get_process_winstation )
-    {
-        if (!wine_server_call_err( req ))
-            ret = wine_server_ptr_handle( reply->handle );
-    }
-    SERVER_END_REQ;
-    return ret;
-}
-
-
-/***********************************************************************
- *              SetProcessWindowStation  (USER32.@)
- */
-BOOL WINAPI SetProcessWindowStation( HWINSTA handle )
-{
-    BOOL ret;
-
-    SERVER_START_REQ( set_process_winstation )
-    {
-        req->handle = wine_server_obj_handle( handle );
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
-    return ret;
+    return NtUserOpenWindowStation( &attr, access );
 }
 
 
@@ -329,31 +253,20 @@ HDESK WINAPI CreateDesktopA( LPCSTR name, LPCSTR device, LPDEVMODEA devmode,
 HDESK WINAPI CreateDesktopW( LPCWSTR name, LPCWSTR device, LPDEVMODEW devmode,
                              DWORD flags, ACCESS_MASK access, LPSECURITY_ATTRIBUTES sa )
 {
-    HANDLE ret;
-    DWORD len = name ? lstrlenW(name) : 0;
+    OBJECT_ATTRIBUTES attr;
+    UNICODE_STRING str;
 
     if (device || devmode)
     {
         SetLastError( ERROR_INVALID_PARAMETER );
         return 0;
     }
-    if (len >= MAX_PATH)
-    {
-        SetLastError( ERROR_FILENAME_EXCED_RANGE );
-        return 0;
-    }
-    SERVER_START_REQ( create_desktop )
-    {
-        req->flags      = flags;
-        req->access     = access;
-        req->attributes = OBJ_CASE_INSENSITIVE | OBJ_OPENIF |
-                          ((sa && sa->bInheritHandle) ? OBJ_INHERIT : 0);
-        wine_server_add_data( req, name, len * sizeof(WCHAR) );
-        wine_server_call_err( req );
-        ret = wine_server_ptr_handle( reply->handle );
-    }
-    SERVER_END_REQ;
-    return ret;
+
+    RtlInitUnicodeString( &str, name );
+    InitializeObjectAttributes( &attr, &str, OBJ_CASE_INSENSITIVE | OBJ_OPENIF,
+                                get_winstations_dir_handle(), NULL );
+    if (sa && sa->bInheritHandle) attr.Attributes |= OBJ_INHERIT;
+    return NtUserCreateDesktopEx( &attr, NULL, devmode, flags, access, 0 );
 }
 
 
@@ -377,24 +290,13 @@ HDESK WINAPI OpenDesktopA( LPCSTR name, DWORD flags, BOOL inherit, ACCESS_MASK a
 
 HDESK open_winstation_desktop( HWINSTA hwinsta, LPCWSTR name, DWORD flags, BOOL inherit, ACCESS_MASK access )
 {
-    HANDLE ret = 0;
-    DWORD len = name ? lstrlenW(name) : 0;
-    if (len >= MAX_PATH)
-    {
-        SetLastError( ERROR_FILENAME_EXCED_RANGE );
-        return 0;
-    }
-    SERVER_START_REQ( open_desktop )
-    {
-        req->winsta     = wine_server_obj_handle( hwinsta );
-        req->flags      = flags;
-        req->access     = access;
-        req->attributes = OBJ_CASE_INSENSITIVE | (inherit ? OBJ_INHERIT : 0);
-        wine_server_add_data( req, name, len * sizeof(WCHAR) );
-        if (!wine_server_call_err( req )) ret = wine_server_ptr_handle( reply->handle );
-    }
-    SERVER_END_REQ;
-    return ret;
+    OBJECT_ATTRIBUTES attr;
+    UNICODE_STRING str;
+
+    RtlInitUnicodeString( &str, name );
+    InitializeObjectAttributes( &attr, &str, OBJ_CASE_INSENSITIVE, hwinsta, NULL );
+    if (inherit) attr.Attributes |= OBJ_INHERIT;
+    return NtUserOpenDesktop( &attr, flags, access );
 }
 
 
@@ -407,52 +309,13 @@ HDESK WINAPI OpenDesktopW( LPCWSTR name, DWORD flags, BOOL inherit, ACCESS_MASK
 }
 
 
-/***********************************************************************
- *              CloseDesktop  (USER32.@)
- */
-BOOL WINAPI CloseDesktop( HDESK handle )
-{
-    BOOL ret;
-    SERVER_START_REQ( close_desktop )
-    {
-        req->handle = wine_server_obj_handle( handle );
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
-    return ret;
-}
-
-
-/******************************************************************************
- *              GetThreadDesktop   (USER32.@)
- */
-HDESK WINAPI GetThreadDesktop( DWORD thread )
-{
-    HDESK ret = 0;
-
-    SERVER_START_REQ( get_thread_desktop )
-    {
-        req->tid = thread;
-        if (!wine_server_call_err( req )) ret = wine_server_ptr_handle( reply->handle );
-    }
-    SERVER_END_REQ;
-    return ret;
-}
-
-
 /******************************************************************************
  *              SetThreadDesktop   (USER32.@)
  */
 BOOL WINAPI SetThreadDesktop( HDESK handle )
 {
-    BOOL ret;
+    BOOL ret = NtUserSetThreadDesktop( handle );
 
-    SERVER_START_REQ( set_thread_desktop )
-    {
-        req->handle = wine_server_obj_handle( handle );
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
     if (ret)  /* reset the desktop windows */
     {
         struct user_thread_info *thread_info = get_user_thread_info();
@@ -488,7 +351,7 @@ BOOL WINAPI EnumDesktopsW( HWINSTA winsta, DESKTOPENUMPROCW func, LPARAM lparam
     NTSTATUS status;
 
     if (!winsta)
-        winsta = GetProcessWindowStation();
+        winsta = NtUserGetProcessWindowStation();
 
     while (ret)
     {
@@ -515,31 +378,6 @@ BOOL WINAPI EnumDesktopsW( HWINSTA winsta, DESKTOPENUMPROCW func, LPARAM lparam
 }
 
 
-/******************************************************************************
- *              OpenInputDesktop   (USER32.@)
- */
-HDESK WINAPI OpenInputDesktop( DWORD flags, BOOL inherit, ACCESS_MASK access )
-{
-    HANDLE ret = 0;
-
-    TRACE( "(%x,%i,%x)\n", flags, inherit, access );
-
-    if (flags)
-        FIXME( "partial stub flags %08x\n", flags );
-
-    SERVER_START_REQ( open_input_desktop )
-    {
-        req->flags      = flags;
-        req->access     = access;
-        req->attributes = inherit ? OBJ_INHERIT : 0;
-        if (!wine_server_call_err( req )) ret = wine_server_ptr_handle( reply->handle );
-    }
-    SERVER_END_REQ;
-
-    return ret;
-}
-
-
 /***********************************************************************
  *              GetUserObjectInformationA   (USER32.@)
  */
@@ -551,7 +389,7 @@ BOOL WINAPI GetUserObjectInformationA( HANDLE handle, INT index, LPVOID info, DW
         WCHAR buffer[MAX_PATH];
         DWORD lenA, lenW;
 
-        if (!GetUserObjectInformationW( handle, index, buffer, sizeof(buffer), &lenW )) return FALSE;
+        if (!NtUserGetObjectInformation( handle, index, buffer, sizeof(buffer), &lenW )) return FALSE;
         lenA = WideCharToMultiByte( CP_ACP, 0, buffer, -1, NULL, 0, NULL, NULL );
         if (needed) *needed = lenA;
         if (lenA > len)
@@ -566,98 +404,7 @@ BOOL WINAPI GetUserObjectInformationA( HANDLE handle, INT index, LPVOID info, DW
         if (info) WideCharToMultiByte( CP_ACP, 0, buffer, -1, info, len, NULL, NULL );
         return TRUE;
     }
-    return GetUserObjectInformationW( handle, index, info, len, needed );
-}
-
-
-/***********************************************************************
- *              GetUserObjectInformationW   (USER32.@)
- */
-BOOL WINAPI GetUserObjectInformationW( HANDLE handle, INT index, LPVOID info, DWORD len, LPDWORD needed )
-{
-    BOOL ret;
-
-    switch(index)
-    {
-    case UOI_FLAGS:
-        {
-            USEROBJECTFLAGS *obj_flags = info;
-            if (needed) *needed = sizeof(*obj_flags);
-            if (len < sizeof(*obj_flags))
-            {
-                SetLastError( ERROR_BUFFER_OVERFLOW );
-                return FALSE;
-            }
-            SERVER_START_REQ( set_user_object_info )
-            {
-                req->handle    = wine_server_obj_handle( handle );
-                req->flags     = 0;
-                ret = !wine_server_call_err( req );
-                if (ret)
-                {
-                    /* FIXME: inherit flag */
-                    obj_flags->dwFlags = reply->old_obj_flags;
-                }
-            }
-            SERVER_END_REQ;
-        }
-        return ret;
-
-    case UOI_TYPE:
-        SERVER_START_REQ( set_user_object_info )
-        {
-            req->handle = wine_server_obj_handle( handle );
-            req->flags  = 0;
-            ret = !wine_server_call_err( req );
-            if (ret)
-            {
-                size_t size = reply->is_desktop ? sizeof(L"Desktop") : sizeof(L"WindowStation");
-                if (needed) *needed = size;
-                if (len < size)
-                {
-                    SetLastError( ERROR_INSUFFICIENT_BUFFER );
-                    ret = FALSE;
-                }
-                else memcpy( info, reply->is_desktop ? L"Desktop" : L"WindowStation", size );
-            }
-        }
-        SERVER_END_REQ;
-        return ret;
-
-    case UOI_NAME:
-        {
-            WCHAR buffer[MAX_PATH];
-            SERVER_START_REQ( set_user_object_info )
-            {
-                req->handle = wine_server_obj_handle( handle );
-                req->flags  = 0;
-                wine_server_set_reply( req, buffer, sizeof(buffer) - sizeof(WCHAR) );
-                ret = !wine_server_call_err( req );
-                if (ret)
-                {
-                    size_t size = wine_server_reply_size( reply );
-                    buffer[size / sizeof(WCHAR)] = 0;
-                    size += sizeof(WCHAR);
-                    if (needed) *needed = size;
-                    if (len < size)
-                    {
-                        SetLastError( ERROR_INSUFFICIENT_BUFFER );
-                        ret = FALSE;
-                    }
-                    else memcpy( info, buffer, size );
-                }
-            }
-            SERVER_END_REQ;
-        }
-        return ret;
-
-    case UOI_USER_SID:
-        FIXME( "not supported index %d\n", index );
-        /* fall through */
-    default:
-        SetLastError( ERROR_INVALID_PARAMETER );
-        return FALSE;
-    }
+    return NtUserGetObjectInformation( handle, index, info, len, needed );
 }
 
 
@@ -666,33 +413,7 @@ BOOL WINAPI GetUserObjectInformationW( HANDLE handle, INT index, LPVOID info, DW
  */
 BOOL WINAPI SetUserObjectInformationA( HANDLE handle, INT index, LPVOID info, DWORD len )
 {
-    return SetUserObjectInformationW( handle, index, info, len );
-}
-
-
-/******************************************************************************
- *              SetUserObjectInformationW   (USER32.@)
- */
-BOOL WINAPI SetUserObjectInformationW( HANDLE handle, INT index, LPVOID info, DWORD len )
-{
-    BOOL ret;
-    const USEROBJECTFLAGS *obj_flags = info;
-
-    if (index != UOI_FLAGS || !info || len < sizeof(*obj_flags))
-    {
-        SetLastError( ERROR_INVALID_PARAMETER );
-        return FALSE;
-    }
-    /* FIXME: inherit flag */
-    SERVER_START_REQ( set_user_object_info )
-    {
-        req->handle    = wine_server_obj_handle( handle );
-        req->flags     = SET_USER_OBJECT_SET_FLAGS;
-        req->obj_flags = obj_flags->dwFlags;
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
-    return ret;
+    return NtUserSetObjectInformation( handle, index, info, len );
 }
 
 
