@@ -22,8 +22,8 @@
 
 #include "config.h"
 
-#include "macdrv.h"
 #define OEMRESOURCE
+#include "macdrv.h"
 #include "winuser.h"
 #include "winreg.h"
 #include "wine/server.h"
@@ -165,7 +165,7 @@ static void send_mouse_input(HWND hwnd, macdrv_window cocoa_window, UINT flags,
     input.mi.time           = time;
     input.mi.dwExtraInfo    = 0;
 
-    __wine_send_input(top_level_hwnd, &input);
+    __wine_send_input(top_level_hwnd, &input, NULL);
 }
 
 
