@@ -1798,7 +1798,6 @@ UINT CDECL macdrv_ShowWindow(HWND hwnd, INT cmd, RECT *rect, UINT swp)
           hwnd, data ? data->cocoa_window : NULL, cmd, wine_dbgstr_rect(rect), swp);
 
     if (!data || !data->cocoa_window) goto done;
-    if (IsRectEmpty(rect)) goto done;
     if (GetWindowLongW(hwnd, GWL_STYLE) & WS_MINIMIZE)
     {
         if (rect->left != -32000 || rect->top != -32000)
@@ -2053,7 +2052,7 @@ static inline RECT get_surface_rect(const RECT *visible_rect)
 /***********************************************************************
  *              WindowPosChanging   (MACDRV.@)
  */
-void CDECL macdrv_WindowPosChanging(HWND hwnd, HWND insert_after, UINT swp_flags,
+BOOL CDECL macdrv_WindowPosChanging(HWND hwnd, HWND insert_after, UINT swp_flags,
                                     const RECT *window_rect, const RECT *client_rect,
                                     RECT *visible_rect, struct window_surface **surface)
 {
@@ -2065,7 +2064,7 @@ void CDECL macdrv_WindowPosChanging(HWND hwnd, HWND insert_after, UINT swp_flags
           swp_flags, wine_dbgstr_rect(window_rect), wine_dbgstr_rect(client_rect),
           wine_dbgstr_rect(visible_rect), surface);
 
-    if (!data && !(data = macdrv_create_win_data(hwnd, window_rect, client_rect))) return;
+    if (!data && !(data = macdrv_create_win_data(hwnd, window_rect, client_rect))) return TRUE;
 
     *visible_rect = *window_rect;
     macdrv_window_to_mac_rect(data, style, visible_rect, window_rect, client_rect);
@@ -2098,6 +2097,7 @@ void CDECL macdrv_WindowPosChanging(HWND hwnd, HWND insert_after, UINT swp_flags
 
 done:
     release_win_data(data);
+    return TRUE;
 }
 
 
@@ -2406,6 +2406,8 @@ void macdrv_app_activated(void)
  */
 void macdrv_app_deactivated(void)
 {
+    ClipCursor(NULL);
+
     if (GetActiveWindow() == GetForegroundWindow())
     {
         TRACE("setting fg to desktop\n");
@@ -2436,6 +2438,21 @@ void macdrv_window_minimize_requested(HWND hwnd)
 }
 
 
+/***********************************************************************
+ *              macdrv_window_did_minimize
+ *
+ * Handler for WINDOW_DID_MINIMIZE events.
+ */
+void macdrv_window_did_minimize(HWND hwnd)
+{
+    TRACE("win %p\n", hwnd);
+
+    /* If all our windows are minimized, disable cursor clipping. */
+    if (!macdrv_is_any_wine_window_visible())
+        ClipCursor(NULL);
+}
+
+
 /***********************************************************************
  *              macdrv_window_did_unminimize
  *
@@ -2871,6 +2888,8 @@ BOOL query_resize_start(HWND hwnd)
 {
     TRACE("hwnd %p\n", hwnd);
 
+    ClipCursor(NULL);
+
     sync_window_min_max_info(hwnd);
     SendMessageW(hwnd, WM_ENTERSIZEMOVE, 0, 0);
 
