@@ -21,7 +21,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #define NONAMELESSSTRUCT
 #define NONAMELESSUNION
@@ -38,6 +37,7 @@ WINE_DECLARE_DEBUG_CHANNEL(winediag);
 #include <assert.h>
 #include <X11/Xlib.h>
 #include <X11/extensions/Xrandr.h>
+#include <dlfcn.h>
 #include "x11drv.h"
 
 #define VK_NO_PROTOTYPES
@@ -68,6 +68,7 @@ MAKE_FUNCPTR(XRRFreeOutputInfo)
 MAKE_FUNCPTR(XRRFreeScreenResources)
 MAKE_FUNCPTR(XRRGetCrtcInfo)
 MAKE_FUNCPTR(XRRGetOutputInfo)
+MAKE_FUNCPTR(XRRGetOutputProperty)
 MAKE_FUNCPTR(XRRGetScreenResources)
 MAKE_FUNCPTR(XRRGetScreenResourcesCurrent)
 MAKE_FUNCPTR(XRRGetScreenSizeRange)
@@ -112,6 +113,7 @@ static int load_xrandr(void)
         LOAD_FUNCPTR(XRRFreeScreenResources);
         LOAD_FUNCPTR(XRRGetCrtcInfo);
         LOAD_FUNCPTR(XRRGetOutputInfo);
+        LOAD_FUNCPTR(XRRGetOutputProperty);
         LOAD_FUNCPTR(XRRGetScreenResources);
         LOAD_FUNCPTR(XRRGetScreenResourcesCurrent);
         LOAD_FUNCPTR(XRRGetScreenSizeRange);
@@ -324,6 +326,32 @@ static LONG xrandr10_set_current_mode( ULONG_PTR id, DEVMODEW *mode )
 
 #ifdef HAVE_XRRGETPROVIDERRESOURCES
 
+static struct current_mode
+{
+    ULONG_PTR id;
+    BOOL loaded;
+    DEVMODEW mode;
+} *current_modes;
+static int current_mode_count;
+
+static CRITICAL_SECTION current_modes_section;
+static CRITICAL_SECTION_DEBUG current_modes_critsect_debug =
+{
+    0, 0, &current_modes_section,
+    {&current_modes_critsect_debug.ProcessLocksList, &current_modes_critsect_debug.ProcessLocksList},
+     0, 0, {(DWORD_PTR)(__FILE__ ": current_modes_section")}
+};
+static CRITICAL_SECTION current_modes_section = {&current_modes_critsect_debug, -1, 0, 0, 0, 0};
+
+static void xrandr14_invalidate_current_mode_cache(void)
+{
+    EnterCriticalSection( &current_modes_section );
+    heap_free( current_modes);
+    current_modes = NULL;
+    current_mode_count = 0;
+    LeaveCriticalSection( &current_modes_section );
+}
+
 static XRRScreenResources *xrandr_get_screen_resources(void)
 {
     XRRScreenResources *resources = pXRRGetScreenResourcesCurrent( gdi_display, root_window );
@@ -440,6 +468,25 @@ static void get_screen_size( XRRScreenResources *resources, unsigned int *width,
     }
 }
 
+static unsigned int get_edid( RROutput output, unsigned char **prop )
+{
+    int result, actual_format;
+    unsigned long bytes_after, len;
+    Atom actual_type;
+
+    result = pXRRGetOutputProperty( gdi_display, output, x11drv_atom(EDID), 0, 128, FALSE, FALSE,
+                                    AnyPropertyType, &actual_type, &actual_format, &len,
+                                    &bytes_after, prop );
+
+    if (result != Success)
+    {
+        WARN("Could not retrieve EDID property for output %#lx.\n", output);
+        *prop = NULL;
+        return 0;
+    }
+    return len;
+}
+
 static void set_screen_size( int width, int height )
 {
     int screen = default_visual.screen;
@@ -591,13 +638,16 @@ static BOOL is_crtc_primary( RECT primary, const XRRCrtcInfo *crtc )
 
 VK_DEFINE_NON_DISPATCHABLE_HANDLE(VkDisplayKHR)
 
-static BOOL get_gpu_properties_from_vulkan( struct x11drv_gpu *gpu, const XRRProviderInfo *provider_info )
+static BOOL get_gpu_properties_from_vulkan( struct gdi_gpu *gpu, const XRRProviderInfo *provider_info )
 {
     static const char *extensions[] =
     {
         VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME,
+        VK_KHR_EXTERNAL_MEMORY_CAPABILITIES_EXTENSION_NAME,
         "VK_EXT_acquire_xlib_display",
         "VK_EXT_direct_mode_display",
+        "VK_KHR_display",
+        VK_KHR_SURFACE_EXTENSION_NAME,
     };
     const struct vulkan_funcs *vulkan_funcs = get_vulkan_driver( WINE_VULKAN_DRIVER_VERSION );
     VkResult (*pvkGetRandROutputDisplayEXT)( VkPhysicalDevice, Display *, RROutput, VkDisplayKHR * );
@@ -696,10 +746,10 @@ done:
 
 /* Get a list of GPUs reported by XRandR 1.4. Set get_properties to FALSE if GPU properties are
  * not needed to avoid unnecessary querying */
-static BOOL xrandr14_get_gpus2( struct x11drv_gpu **new_gpus, int *count, BOOL get_properties )
+static BOOL xrandr14_get_gpus2( struct gdi_gpu **new_gpus, int *count, BOOL get_properties )
 {
     static const WCHAR wine_adapterW[] = {'W','i','n','e',' ','A','d','a','p','t','e','r',0};
-    struct x11drv_gpu *gpus = NULL;
+    struct gdi_gpu *gpus = NULL;
     XRRScreenResources *screen_resources = NULL;
     XRRProviderResources *provider_resources = NULL;
     XRRProviderInfo *provider_info = NULL;
@@ -770,7 +820,7 @@ static BOOL xrandr14_get_gpus2( struct x11drv_gpu **new_gpus, int *count, BOOL g
     /* Make primary GPU the first */
     if (primary_provider > 0)
     {
-        struct x11drv_gpu tmp = gpus[0];
+        struct gdi_gpu tmp = gpus[0];
         gpus[0] = gpus[primary_provider];
         gpus[primary_provider] = tmp;
     }
@@ -791,19 +841,19 @@ done:
     return ret;
 }
 
-static BOOL xrandr14_get_gpus( struct x11drv_gpu **new_gpus, int *count )
+static BOOL xrandr14_get_gpus( struct gdi_gpu **new_gpus, int *count )
 {
     return xrandr14_get_gpus2( new_gpus, count, TRUE );
 }
 
-static void xrandr14_free_gpus( struct x11drv_gpu *gpus )
+static void xrandr14_free_gpus( struct gdi_gpu *gpus )
 {
     heap_free( gpus );
 }
 
-static BOOL xrandr14_get_adapters( ULONG_PTR gpu_id, struct x11drv_adapter **new_adapters, int *count )
+static BOOL xrandr14_get_adapters( ULONG_PTR gpu_id, struct gdi_adapter **new_adapters, int *count )
 {
-    struct x11drv_adapter *adapters = NULL;
+    struct gdi_adapter *adapters = NULL;
     XRRScreenResources *screen_resources = NULL;
     XRRProviderInfo *provider_info = NULL;
     XRRCrtcInfo *enum_crtc_info, *crtc_info = NULL;
@@ -937,7 +987,7 @@ static BOOL xrandr14_get_adapters( ULONG_PTR gpu_id, struct x11drv_adapter **new
     /* Make primary adapter the first */
     if (primary_adapter)
     {
-        struct x11drv_adapter tmp = adapters[0];
+        struct gdi_adapter tmp = adapters[0];
         adapters[0] = adapters[primary_adapter];
         adapters[primary_adapter] = tmp;
     }
@@ -962,17 +1012,17 @@ done:
     return ret;
 }
 
-static void xrandr14_free_adapters( struct x11drv_adapter *adapters )
+static void xrandr14_free_adapters( struct gdi_adapter *adapters )
 {
     heap_free( adapters );
 }
 
-static BOOL xrandr14_get_monitors( ULONG_PTR adapter_id, struct x11drv_monitor **new_monitors, int *count )
+static BOOL xrandr14_get_monitors( ULONG_PTR adapter_id, struct gdi_monitor **new_monitors, int *count )
 {
     static const WCHAR generic_nonpnp_monitorW[] = {
         'G','e','n','e','r','i','c',' ',
         'N','o','n','-','P','n','P',' ','M','o','n','i','t','o','r',0};
-    struct x11drv_monitor *realloc_monitors, *monitors = NULL;
+    struct gdi_monitor *realloc_monitors, *monitors = NULL;
     XRRScreenResources *screen_resources = NULL;
     XRROutputInfo *output_info = NULL, *enum_output_info = NULL;
     XRRCrtcInfo *crtc_info = NULL, *enum_crtc_info;
@@ -1007,6 +1057,7 @@ static BOOL xrandr14_get_monitors( ULONG_PTR adapter_id, struct x11drv_monitor *
     {
         lstrcpyW( monitors[monitor_count].name, generic_nonpnp_monitorW );
         monitors[monitor_count].state_flags = DISPLAY_DEVICE_ATTACHED;
+        monitors[monitor_count].edid_len = get_edid( adapter_id, &monitors[monitor_count].edid );
         monitor_count = 1;
     }
     /* Active monitors, need to find other monitors with the same coordinates as mirrored */
@@ -1062,6 +1113,9 @@ static BOOL xrandr14_get_monitors( ULONG_PTR adapter_id, struct x11drv_monitor *
 
                     if (is_crtc_primary( primary_rect, crtc_info ))
                         primary_index = monitor_count;
+
+                    monitors[monitor_count].edid_len = get_edid( screen_resources->outputs[i],
+                                                                 &monitors[monitor_count].edid );
                     monitor_count++;
                 }
 
@@ -1075,7 +1129,7 @@ static BOOL xrandr14_get_monitors( ULONG_PTR adapter_id, struct x11drv_monitor *
         /* Make sure the first monitor is the primary */
         if (primary_index)
         {
-            struct x11drv_monitor tmp = monitors[0];
+            struct gdi_monitor tmp = monitors[0];
             monitors[0] = monitors[primary_index];
             monitors[primary_index] = tmp;
         }
@@ -1102,19 +1156,32 @@ done:
         pXRRFreeOutputInfo( enum_output_info );
     if (!ret)
     {
+        for (i = 0; i < monitor_count; i++)
+        {
+            if (monitors[i].edid)
+                XFree( monitors[i].edid );
+        }
         heap_free( monitors );
         ERR("Failed to get monitors\n");
     }
     return ret;
 }
 
-static void xrandr14_free_monitors( struct x11drv_monitor *monitors )
+static void xrandr14_free_monitors( struct gdi_monitor *monitors, int count )
 {
+    int i;
+
+    for (i = 0; i < count; i++)
+    {
+        if (monitors[i].edid)
+            XFree( monitors[i].edid );
+    }
     heap_free( monitors );
 }
 
 static BOOL xrandr14_device_change_handler( HWND hwnd, XEvent *event )
 {
+    xrandr14_invalidate_current_mode_cache();
     if (hwnd == GetDesktopWindow() && GetWindowThreadProcessId( hwnd, NULL ) == GetCurrentThreadId())
     {
         /* Don't send a WM_DISPLAYCHANGE message here because this event may be a result from
@@ -1148,10 +1215,11 @@ static void xrandr14_register_event_handlers(void)
 /* XRandR 1.4 display settings handler */
 static BOOL xrandr14_get_id( const WCHAR *device_name, ULONG_PTR *id )
 {
-    INT gpu_count, adapter_count, display_count = 0;
+    struct current_mode *tmp_modes, *new_current_modes = NULL;
+    INT gpu_count, adapter_count, new_current_mode_count = 0;
     INT gpu_idx, adapter_idx, display_idx;
-    struct x11drv_adapter *adapters;
-    struct x11drv_gpu *gpus;
+    struct gdi_adapter *adapters;
+    struct gdi_gpu *gpus;
     WCHAR *end;
 
     /* Parse \\.\DISPLAY%d */
@@ -1159,31 +1227,60 @@ static BOOL xrandr14_get_id( const WCHAR *device_name, ULONG_PTR *id )
     if (*end)
         return FALSE;
 
-    if (!xrandr14_get_gpus2( &gpus, &gpu_count, FALSE ))
-        return FALSE;
-
-    for (gpu_idx = 0; gpu_idx < gpu_count; ++gpu_idx)
+    /* Update cache */
+    EnterCriticalSection( &current_modes_section );
+    if (!current_modes)
     {
-        if (!xrandr14_get_adapters( gpus[gpu_idx].id, &adapters, &adapter_count ))
+        if (!xrandr14_get_gpus2( &gpus, &gpu_count, FALSE ))
         {
-            xrandr14_free_gpus( gpus );
+            LeaveCriticalSection( &current_modes_section );
             return FALSE;
         }
 
-        adapter_idx = display_idx - display_count;
-        if (adapter_idx < adapter_count)
+        for (gpu_idx = 0; gpu_idx < gpu_count; ++gpu_idx)
         {
-            *id = adapters[adapter_idx].id;
-            xrandr14_free_adapters( adapters );
-            xrandr14_free_gpus( gpus );
-            return TRUE;
-        }
+            if (!xrandr14_get_adapters( gpus[gpu_idx].id, &adapters, &adapter_count ))
+                break;
 
-        display_count += adapter_count;
-        xrandr14_free_adapters( adapters );
+            if (!new_current_modes)
+                tmp_modes = heap_alloc( adapter_count * sizeof(*tmp_modes) );
+            else
+                tmp_modes = heap_realloc( new_current_modes, (new_current_mode_count + adapter_count) * sizeof(*tmp_modes) );
+
+            if (!tmp_modes)
+            {
+                xrandr14_free_adapters( adapters );
+                break;
+            }
+            new_current_modes = tmp_modes;
+
+            for (adapter_idx = 0; adapter_idx < adapter_count; ++adapter_idx)
+            {
+                new_current_modes[new_current_mode_count + adapter_idx].id = adapters[adapter_idx].id;
+                new_current_modes[new_current_mode_count + adapter_idx].loaded = FALSE;
+            }
+            new_current_mode_count += adapter_count;
+            xrandr14_free_adapters( adapters );
+        }
+        xrandr14_free_gpus( gpus );
+
+        if (new_current_modes)
+        {
+            heap_free( current_modes );
+            current_modes = new_current_modes;
+            current_mode_count = new_current_mode_count;
+        }
     }
-    xrandr14_free_gpus( gpus );
-    return FALSE;
+
+    if (display_idx >= current_mode_count)
+    {
+        LeaveCriticalSection( &current_modes_section );
+        return FALSE;
+    }
+
+    *id = current_modes[display_idx].id;
+    LeaveCriticalSection( &current_modes_section );
+    return TRUE;
 }
 
 static void add_xrandr14_mode( DEVMODEW *mode, XRRModeInfo *info, DWORD depth, DWORD frequency,
@@ -1333,6 +1430,7 @@ static void xrandr14_free_modes( DEVMODEW *modes )
 
 static BOOL xrandr14_get_current_mode( ULONG_PTR id, DEVMODEW *mode )
 {
+    struct current_mode *mode_ptr = NULL;
     XRRScreenResources *screen_resources;
     XRROutputInfo *output_info = NULL;
     RROutput output = (RROutput)id;
@@ -1342,6 +1440,23 @@ static BOOL xrandr14_get_current_mode( ULONG_PTR id, DEVMODEW *mode )
     RECT primary;
     INT mode_idx;
 
+    EnterCriticalSection( &current_modes_section );
+    for (mode_idx = 0; mode_idx < current_mode_count; ++mode_idx)
+    {
+        if (current_modes[mode_idx].id != id)
+            continue;
+
+        if (!current_modes[mode_idx].loaded)
+        {
+            mode_ptr = &current_modes[mode_idx];
+            break;
+        }
+
+        memcpy( mode, &current_modes[mode_idx].mode, sizeof(*mode) );
+        LeaveCriticalSection( &current_modes_section );
+        return TRUE;
+    }
+
     screen_resources = xrandr_get_screen_resources();
     if (!screen_resources)
         goto done;
@@ -1400,7 +1515,16 @@ static BOOL xrandr14_get_current_mode( ULONG_PTR id, DEVMODEW *mode )
     mode->u1.s2.dmPosition.x = crtc_info->x - primary.left;
     mode->u1.s2.dmPosition.y = crtc_info->y - primary.top;
     ret = TRUE;
+
 done:
+    if (ret && mode_ptr)
+    {
+        memcpy( &mode_ptr->mode, mode, sizeof(*mode) );
+        mode_ptr->mode.dmSize = sizeof(*mode);
+        mode_ptr->mode.dmDriverExtra = 0;
+        mode_ptr->loaded = TRUE;
+    }
+    LeaveCriticalSection( &current_modes_section );
     if (crtc_info)
         pXRRFreeCrtcInfo( crtc_info );
     if (output_info)
@@ -1517,6 +1641,7 @@ done:
     if (output_info)
         pXRRFreeOutputInfo( output_info );
     pXRRFreeScreenResources( screen_resources );
+    xrandr14_invalidate_current_mode_cache();
     return ret;
 }
 
