@@ -48,8 +48,10 @@ void send_callback( struct object_header *hdr, DWORD status, void *info, DWORD b
 {
     if (hdr->callback && (hdr->notify_mask & status))
     {
-        TRACE("%p, 0x%08x, %p, %u\n", hdr, status, info, buflen);
+        TRACE("%p, 0x%08x, %p, %u, %u\n", hdr, status, info, buflen, hdr->recursion_count);
+        InterlockedIncrement( &hdr->recursion_count );
         hdr->callback( hdr->handle, hdr->context, status, info, buflen );
+        InterlockedDecrement( &hdr->recursion_count );
         TRACE("returning from 0x%08x callback\n", status);
     }
 }
@@ -74,12 +76,23 @@ static void session_destroy( struct object_header *hdr )
 
     session->cs.DebugInfo->Spare[0] = 0;
     DeleteCriticalSection( &session->cs );
-    heap_free( session->agent );
-    heap_free( session->proxy_server );
-    heap_free( session->proxy_bypass );
-    heap_free( session->proxy_username );
-    heap_free( session->proxy_password );
-    heap_free( session );
+    free( session->agent );
+    free( session->proxy_server );
+    free( session->proxy_bypass );
+    free( session->proxy_username );
+    free( session->proxy_password );
+    free( session );
+}
+
+static BOOL validate_buffer( void *buffer, DWORD *buflen, DWORD required )
+{
+    if (!buffer || *buflen < required)
+    {
+        *buflen = required;
+        SetLastError( ERROR_INSUFFICIENT_BUFFER );
+        return FALSE;
+    }
+    return TRUE;
 }
 
 static BOOL session_query_option( struct object_header *hdr, DWORD option, void *buffer, DWORD *buflen )
@@ -90,38 +103,43 @@ static BOOL session_query_option( struct object_header *hdr, DWORD option, void
     {
     case WINHTTP_OPTION_REDIRECT_POLICY:
     {
-        if (!buffer || *buflen < sizeof(DWORD))
-        {
-            *buflen = sizeof(DWORD);
-            SetLastError( ERROR_INSUFFICIENT_BUFFER );
-            return FALSE;
-        }
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
 
         *(DWORD *)buffer = hdr->redirect_policy;
         *buflen = sizeof(DWORD);
         return TRUE;
     }
     case WINHTTP_OPTION_RESOLVE_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = session->resolve_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
 
     case WINHTTP_OPTION_CONNECT_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = session->connect_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
 
     case WINHTTP_OPTION_SEND_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = session->send_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
 
     case WINHTTP_OPTION_RECEIVE_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = session->receive_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
 
     case WINHTTP_OPTION_RECEIVE_RESPONSE_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = session->receive_response_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
@@ -239,7 +257,7 @@ HINTERNET WINAPI WinHttpOpen( LPCWSTR agent, DWORD access, LPCWSTR proxy, LPCWST
 
     TRACE("%s, %u, %s, %s, 0x%08x\n", debugstr_w(agent), access, debugstr_w(proxy), debugstr_w(bypass), flags);
 
-    if (!(session = heap_alloc_zero( sizeof(struct session) ))) return NULL;
+    if (!(session = calloc( 1, sizeof(*session) ))) return NULL;
 
     session->hdr.type = WINHTTP_HANDLE_TYPE_SESSION;
     session->hdr.vtbl = &session_vtbl;
@@ -299,11 +317,11 @@ static void connect_destroy( struct object_header *hdr )
 
     release_object( &connect->session->hdr );
 
-    heap_free( connect->hostname );
-    heap_free( connect->servername );
-    heap_free( connect->username );
-    heap_free( connect->password );
-    heap_free( connect );
+    free( connect->hostname );
+    free( connect->servername );
+    free( connect->username );
+    free( connect->password );
+    free( connect );
 }
 
 static BOOL connect_query_option( struct object_header *hdr, DWORD option, void *buffer, DWORD *buflen )
@@ -314,38 +332,43 @@ static BOOL connect_query_option( struct object_header *hdr, DWORD option, void
     {
     case WINHTTP_OPTION_PARENT_HANDLE:
     {
-        if (!buffer || *buflen < sizeof(HINTERNET))
-        {
-            *buflen = sizeof(HINTERNET);
-            SetLastError( ERROR_INSUFFICIENT_BUFFER );
-            return FALSE;
-        }
+        if (!validate_buffer( buffer, buflen, sizeof(HINTERNET) )) return FALSE;
 
         *(HINTERNET *)buffer = ((struct object_header *)connect->session)->handle;
         *buflen = sizeof(HINTERNET);
         return TRUE;
     }
     case WINHTTP_OPTION_RESOLVE_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = connect->session->resolve_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
 
     case WINHTTP_OPTION_CONNECT_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = connect->session->connect_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
 
     case WINHTTP_OPTION_SEND_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = connect->session->send_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
 
     case WINHTTP_OPTION_RECEIVE_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = connect->session->receive_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
 
     case WINHTTP_OPTION_RECEIVE_RESPONSE_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = connect->session->receive_response_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
@@ -462,16 +485,14 @@ BOOL set_server_for_hostname( struct connect *connect, const WCHAR *server, INTE
             if (!connect->servername || wcsnicmp( connect->servername,
                 session->proxy_server, colon - session->proxy_server - 1 ))
             {
-                heap_free( connect->servername );
+                free( connect->servername );
                 connect->resolved = FALSE;
-                if (!(connect->servername = heap_alloc(
-                    (colon - session->proxy_server + 1) * sizeof(WCHAR) )))
+                if (!(connect->servername = malloc( (colon - session->proxy_server + 1) * sizeof(WCHAR) )))
                 {
                     ret = FALSE;
                     goto end;
                 }
-                memcpy( connect->servername, session->proxy_server,
-                    (colon - session->proxy_server) * sizeof(WCHAR) );
+                memcpy( connect->servername, session->proxy_server, (colon - session->proxy_server) * sizeof(WCHAR) );
                 connect->servername[colon - session->proxy_server] = 0;
                 if (*(colon + 1))
                     connect->serverport = wcstol( colon + 1, NULL, 10 );
@@ -481,10 +502,9 @@ BOOL set_server_for_hostname( struct connect *connect, const WCHAR *server, INTE
         }
         else
         {
-            if (!connect->servername || wcsicmp( connect->servername,
-                session->proxy_server ))
+            if (!connect->servername || wcsicmp( connect->servername, session->proxy_server ))
             {
-                heap_free( connect->servername );
+                free( connect->servername );
                 connect->resolved = FALSE;
                 if (!(connect->servername = strdupW( session->proxy_server )))
                 {
@@ -497,7 +517,7 @@ BOOL set_server_for_hostname( struct connect *connect, const WCHAR *server, INTE
     }
     else if (server)
     {
-        heap_free( connect->servername );
+        free( connect->servername );
         connect->resolved = FALSE;
         if (!(connect->servername = strdupW( server )))
         {
@@ -537,7 +557,7 @@ HINTERNET WINAPI WinHttpConnect( HINTERNET hsession, LPCWSTR server, INTERNET_PO
         SetLastError( ERROR_WINHTTP_INCORRECT_HANDLE_TYPE );
         return NULL;
     }
-    if (!(connect = heap_alloc_zero( sizeof(struct connect) )))
+    if (!(connect = calloc( 1, sizeof(*connect) )))
     {
         release_object( &session->hdr );
         return NULL;
@@ -588,27 +608,27 @@ static void request_destroy( struct object_header *hdr )
     destroy_authinfo( request->authinfo );
     destroy_authinfo( request->proxy_authinfo );
 
-    heap_free( request->verb );
-    heap_free( request->path );
-    heap_free( request->version );
-    heap_free( request->raw_headers );
-    heap_free( request->status_text );
+    free( request->verb );
+    free( request->path );
+    free( request->version );
+    free( request->raw_headers );
+    free( request->status_text );
     for (i = 0; i < request->num_headers; i++)
     {
-        heap_free( request->headers[i].field );
-        heap_free( request->headers[i].value );
+        free( request->headers[i].field );
+        free( request->headers[i].value );
     }
-    heap_free( request->headers );
+    free( request->headers );
     for (i = 0; i < TARGET_MAX; i++)
     {
         for (j = 0; j < SCHEME_MAX; j++)
         {
-            heap_free( request->creds[i][j].username );
-            heap_free( request->creds[i][j].password );
+            free( request->creds[i][j].username );
+            free( request->creds[i][j].password );
         }
     }
 
-    heap_free( request );
+    free( request );
 }
 
 static void str_to_buffer( WCHAR *buffer, const WCHAR *str, LPDWORD buflen )
@@ -672,12 +692,7 @@ static BOOL request_query_option( struct object_header *hdr, DWORD option, void
         DWORD flags;
         int bits;
 
-        if (!buffer || *buflen < sizeof(flags))
-        {
-            *buflen = sizeof(flags);
-            SetLastError( ERROR_INSUFFICIENT_BUFFER );
-            return FALSE;
-        }
+        if (!validate_buffer( buffer, buflen, sizeof(flags) )) return FALSE;
 
         flags = request->security_flags;
         if (request->netconn)
@@ -698,12 +713,7 @@ static BOOL request_query_option( struct object_header *hdr, DWORD option, void
     {
         const CERT_CONTEXT *cert;
 
-        if (!buffer || *buflen < sizeof(cert))
-        {
-            *buflen = sizeof(cert);
-            SetLastError( ERROR_INSUFFICIENT_BUFFER );
-            return FALSE;
-        }
+        if (!validate_buffer( buffer, buflen, sizeof(cert) )) return FALSE;
 
         if (!(cert = CertDuplicateCertificateContext( request->server_cert ))) return FALSE;
         *(CERT_CONTEXT **)buffer = (CERT_CONTEXT *)cert;
@@ -718,13 +728,7 @@ static BOOL request_query_option( struct object_header *hdr, DWORD option, void
 
         FIXME("partial stub\n");
 
-        if (!buffer || *buflen < sizeof(*ci))
-        {
-            *buflen = sizeof(*ci);
-            SetLastError( ERROR_INSUFFICIENT_BUFFER );
-            return FALSE;
-        }
-        if (!cert) return FALSE;
+        if (!validate_buffer( buffer, buflen, sizeof(*ci) ) || !cert) return FALSE;
 
         ci->ftExpiry = cert->pCertInfo->NotAfter;
         ci->ftStart  = cert->pCertInfo->NotBefore;
@@ -744,12 +748,7 @@ static BOOL request_query_option( struct object_header *hdr, DWORD option, void
     }
     case WINHTTP_OPTION_SECURITY_KEY_BITNESS:
     {
-        if (!buffer || *buflen < sizeof(DWORD))
-        {
-            *buflen = sizeof(DWORD);
-            SetLastError( ERROR_INSUFFICIENT_BUFFER );
-            return FALSE;
-        }
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
 
         *(DWORD *)buffer = request->netconn ? netconn_get_cipher_strength( request->netconn ) : 0;
         *buflen = sizeof(DWORD);
@@ -762,12 +761,8 @@ static BOOL request_query_option( struct object_header *hdr, DWORD option, void
         socklen_t len = sizeof(local);
         const struct sockaddr *remote = (const struct sockaddr *)&request->connect->sockaddr;
 
-        if (!buffer || *buflen < sizeof(*info))
-        {
-            *buflen = sizeof(*info);
-            SetLastError( ERROR_INSUFFICIENT_BUFFER );
-            return FALSE;
-        }
+        if (!validate_buffer( buffer, buflen, sizeof(*info) )) return FALSE;
+
         if (!request->netconn)
         {
             SetLastError( ERROR_WINHTTP_INCORRECT_HANDLE_STATE );
@@ -780,26 +775,36 @@ static BOOL request_query_option( struct object_header *hdr, DWORD option, void
         return TRUE;
     }
     case WINHTTP_OPTION_RESOLVE_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = request->resolve_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
 
     case WINHTTP_OPTION_CONNECT_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = request->connect_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
 
     case WINHTTP_OPTION_SEND_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = request->send_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
 
     case WINHTTP_OPTION_RECEIVE_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = request->receive_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
 
     case WINHTTP_OPTION_RECEIVE_RESPONSE_TIMEOUT:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = request->receive_response_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
@@ -821,10 +826,20 @@ static BOOL request_query_option( struct object_header *hdr, DWORD option, void
         return TRUE;
 
     case WINHTTP_OPTION_MAX_HTTP_AUTOMATIC_REDIRECTS:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
         *(DWORD *)buffer = request->max_redirects;
         *buflen = sizeof(DWORD);
         return TRUE;
 
+    case WINHTTP_OPTION_HTTP_PROTOCOL_USED:
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD) )) return FALSE;
+
+        FIXME("WINHTTP_OPTION_HTTP_PROTOCOL_USED\n");
+        *(DWORD *)buffer = 0;
+        *buflen = sizeof(DWORD);
+        return TRUE;
+
     default:
         FIXME("unimplemented option %u\n", option);
         SetLastError( ERROR_INVALID_PARAMETER );
@@ -835,7 +850,7 @@ static BOOL request_query_option( struct object_header *hdr, DWORD option, void
 static WCHAR *buffer_to_str( WCHAR *buffer, DWORD buflen )
 {
     WCHAR *ret;
-    if ((ret = heap_alloc( (buflen + 1) * sizeof(WCHAR))))
+    if ((ret = malloc( (buflen + 1) * sizeof(WCHAR))))
     {
         memcpy( ret, buffer, buflen * sizeof(WCHAR) );
         ret[buflen] = 0;
@@ -950,7 +965,7 @@ static BOOL request_set_option( struct object_header *hdr, DWORD option, void *b
     {
         struct connect *connect = request->connect;
 
-        heap_free( connect->username );
+        free( connect->username );
         if (!(connect->username = buffer_to_str( buffer, buflen ))) return FALSE;
         return TRUE;
     }
@@ -958,7 +973,7 @@ static BOOL request_set_option( struct object_header *hdr, DWORD option, void *b
     {
         struct connect *connect = request->connect;
 
-        heap_free( connect->password );
+        free( connect->password );
         if (!(connect->password = buffer_to_str( buffer, buflen ))) return FALSE;
         return TRUE;
     }
@@ -966,7 +981,7 @@ static BOOL request_set_option( struct object_header *hdr, DWORD option, void *b
     {
         struct session *session = request->connect->session;
 
-        heap_free( session->proxy_username );
+        free( session->proxy_username );
         if (!(session->proxy_username = buffer_to_str( buffer, buflen ))) return FALSE;
         return TRUE;
     }
@@ -974,7 +989,7 @@ static BOOL request_set_option( struct object_header *hdr, DWORD option, void *b
     {
         struct session *session = request->connect->session;
 
-        heap_free( session->proxy_password );
+        free( session->proxy_password );
         if (!(session->proxy_password = buffer_to_str( buffer, buflen ))) return FALSE;
         return TRUE;
     }
@@ -992,7 +1007,7 @@ static BOOL request_set_option( struct object_header *hdr, DWORD option, void *b
             CertFreeCertificateContext( request->client_cert );
             request->client_cert = NULL;
         }
-        else if (buflen >= sizeof(cert))
+        else if (buflen >= sizeof(*cert))
         {
             if (!(cert = CertDuplicateCertificateContext( buffer ))) return FALSE;
             CertFreeCertificateContext( request->client_cert );
@@ -1044,6 +1059,23 @@ static BOOL request_set_option( struct object_header *hdr, DWORD option, void *b
         SetLastError(ERROR_INVALID_PARAMETER);
         return FALSE;
 
+    case WINHTTP_OPTION_MAX_RESPONSE_HEADER_SIZE:
+        FIXME("WINHTTP_OPTION_MAX_RESPONSE_HEADER_SIZE\n");
+        return TRUE;
+
+    case WINHTTP_OPTION_MAX_RESPONSE_DRAIN_SIZE:
+        FIXME("WINHTTP_OPTION_MAX_RESPONSE_DRAIN_SIZE\n");
+        return TRUE;
+
+    case WINHTTP_OPTION_ENABLE_HTTP_PROTOCOL:
+        if (buflen == sizeof(DWORD))
+        {
+            FIXME("WINHTTP_OPTION_ENABLE_HTTP_PROTOCOL %08x\n", *(DWORD *)buffer);
+            return TRUE;
+        }
+        SetLastError(ERROR_INVALID_PARAMETER);
+        return FALSE;
+
     default:
         FIXME("unimplemented option %u\n", option);
         SetLastError( ERROR_WINHTTP_INVALID_OPTION );
@@ -1077,7 +1109,7 @@ static WCHAR *get_request_path( const WCHAR *object )
     WCHAR *p, *ret;
 
     if (!object || object[0] != '/') len++;
-    if (!(p = ret = heap_alloc( (len + 1) * sizeof(WCHAR) ))) return NULL;
+    if (!(p = ret = malloc( (len + 1) * sizeof(WCHAR) ))) return NULL;
     if (!object || object[0] != '/') *p++ = '/';
     if (object) lstrcpyW( p, object );
     ret[len] = 0;
@@ -1115,7 +1147,7 @@ HINTERNET WINAPI WinHttpOpenRequest( HINTERNET hconnect, LPCWSTR verb, LPCWSTR o
         SetLastError( ERROR_WINHTTP_INCORRECT_HANDLE_TYPE );
         return NULL;
     }
-    if (!(request = heap_alloc_zero( sizeof(struct request) )))
+    if (!(request = calloc( 1, sizeof(*request) )))
     {
         release_object( &connect->hdr );
         return NULL;
@@ -1194,12 +1226,7 @@ static BOOL query_option( struct object_header *hdr, DWORD option, void *buffer,
     {
     case WINHTTP_OPTION_CONTEXT_VALUE:
     {
-        if (!buffer || *buflen < sizeof(DWORD_PTR))
-        {
-            *buflen = sizeof(DWORD_PTR);
-            SetLastError( ERROR_INSUFFICIENT_BUFFER );
-            return FALSE;
-        }
+        if (!validate_buffer( buffer, buflen, sizeof(DWORD_PTR) )) return FALSE;
 
         *(DWORD_PTR *)buffer = hdr->context;
         *buflen = sizeof(DWORD_PTR);
@@ -1306,16 +1333,16 @@ static IP_ADAPTER_ADDRESSES *get_adapters(void)
                                     GAA_FLAG_SKIP_DNS_SERVER | GAA_FLAG_SKIP_FRIENDLY_NAME;
     IP_ADAPTER_ADDRESSES *tmp, *ret;
 
-    if (!(ret = heap_alloc( size ))) return NULL;
+    if (!(ret = malloc( size ))) return NULL;
     err = GetAdaptersAddresses( AF_UNSPEC, flags, NULL, ret, &size );
     while (err == ERROR_BUFFER_OVERFLOW)
     {
-        if (!(tmp = heap_realloc( ret, size ))) break;
+        if (!(tmp = realloc( ret, size ))) break;
         ret = tmp;
         err = GetAdaptersAddresses( AF_UNSPEC, flags, NULL, ret, &size );
     }
     if (err == ERROR_SUCCESS) return ret;
-    heap_free( ret );
+    free( ret );
     return NULL;
 }
 
@@ -1345,12 +1372,12 @@ static WCHAR *detect_autoproxyconfig_url_dhcp(void)
         /* FIXME: also skip adapters where DHCP is disabled */
 
         size = 256;
-        if (!(buf = heap_alloc( size ))) goto done;
+        if (!(buf = malloc( size ))) goto done;
         err = DhcpRequestParams( DHCPCAPI_REQUEST_SYNCHRONOUS, NULL, name, NULL, send_params, recv_params,
                                  buf, &size, NULL );
         while (err == ERROR_MORE_DATA)
         {
-            if (!(tmp = heap_realloc( buf, size ))) goto done;
+            if (!(tmp = realloc( buf, size ))) goto done;
             buf = tmp;
             err = DhcpRequestParams( DHCPCAPI_REQUEST_SYNCHRONOUS, NULL, name, NULL, send_params, recv_params,
                                      buf, &size, NULL );
@@ -1358,7 +1385,7 @@ static WCHAR *detect_autoproxyconfig_url_dhcp(void)
         if (err == ERROR_SUCCESS && param.nBytesData)
         {
             int len = MultiByteToWideChar( CP_ACP, 0, (const char *)param.Data, param.nBytesData, NULL, 0 );
-            if ((ret = heap_alloc( (len + 1) * sizeof(WCHAR) )))
+            if ((ret = malloc( (len + 1) * sizeof(WCHAR) )))
             {
                 MultiByteToWideChar( CP_ACP, 0,  (const char *)param.Data, param.nBytesData, ret, len );
                 ret[len] = 0;
@@ -1369,8 +1396,8 @@ static WCHAR *detect_autoproxyconfig_url_dhcp(void)
     }
 
 done:
-    heap_free( buf );
-    heap_free( adapters );
+    free( buf );
+    free( adapters );
     return ret;
 }
 
@@ -1381,10 +1408,10 @@ static char *get_computer_name( COMPUTER_NAME_FORMAT format )
 
     GetComputerNameExA( format, NULL, &size );
     if (GetLastError() != ERROR_MORE_DATA) return NULL;
-    if (!(ret = heap_alloc( size ))) return NULL;
+    if (!(ret = malloc( size ))) return NULL;
     if (!GetComputerNameExA( format, ret, &size ))
     {
-        heap_free( ret );
+        free( ret );
         return NULL;
     }
     return ret;
@@ -1432,25 +1459,28 @@ static WCHAR *detect_autoproxyconfig_url_dns(void)
     if (!(fqdn = get_computer_name( ComputerNamePhysicalDnsFullyQualified ))) return NULL;
     if (!(domain = get_computer_name( ComputerNamePhysicalDnsDomain )))
     {
-        heap_free( fqdn );
+        free( fqdn );
         return NULL;
     }
     p = fqdn;
     while ((p = strchr( p, '.' )) && is_domain_suffix( p + 1, domain ))
     {
         char *name;
-        struct addrinfo *ai;
+        struct addrinfo *ai, hints;
         int res;
 
-        if (!(name = heap_alloc( sizeof("wpad") + strlen(p) )))
+        if (!(name = malloc( sizeof("wpad") + strlen(p) )))
         {
-            heap_free( fqdn );
-            heap_free( domain );
+            free( fqdn );
+            free( domain );
             return NULL;
         }
         strcpy( name, "wpad" );
         strcat( name, p );
-        res = getaddrinfo( name, NULL, NULL, &ai );
+        memset( &hints, 0, sizeof(hints) );
+        hints.ai_flags  = AI_ALL | AI_DNS_ONLY;
+        hints.ai_family = AF_UNSPEC;
+        res = getaddrinfo( name, NULL, &hints, &ai );
         if (!res)
         {
             ret = build_wpad_url( name, ai );
@@ -1458,15 +1488,15 @@ static WCHAR *detect_autoproxyconfig_url_dns(void)
             if (ret)
             {
                 TRACE("returning %s\n", debugstr_w(ret));
-                heap_free( name );
+                free( name );
                 break;
             }
         }
-       heap_free( name );
+       free( name );
        p++;
     }
-    heap_free( domain );
-    heap_free( fqdn );
+    free( domain );
+    free( fqdn );
     return ret;
 }
 
@@ -1547,7 +1577,7 @@ BOOL WINAPI WinHttpGetDefaultProxyConfiguration( WINHTTP_PROXY_INFO *info )
         if (!l && type == REG_BINARY &&
             size >= sizeof(struct connection_settings_header) + 2 * sizeof(DWORD))
         {
-            BYTE *buf = heap_alloc( size );
+            BYTE *buf = malloc( size );
 
             if (buf)
             {
@@ -1605,7 +1635,7 @@ BOOL WINAPI WinHttpGetDefaultProxyConfiguration( WINHTTP_PROXY_INFO *info )
                         }
                     }
                 }
-                heap_free( buf );
+                free( buf );
             }
         }
         RegCloseKey( key );
@@ -1673,7 +1703,7 @@ BOOL WINAPI WinHttpGetIEProxyConfigForCurrentUser( WINHTTP_CURRENT_USER_IE_PROXY
         ret = TRUE;
         goto done;
     }
-    if (!(hdr = heap_alloc( size ))) goto done;
+    if (!(hdr = malloc( size ))) goto done;
     if (RegQueryValueExW( hkey, L"DefaultConnectionSettings", NULL, &type, (BYTE *)hdr, &size ) ||
         hdr->magic != WININET_SETTINGS_MAGIC)
     {
@@ -1713,7 +1743,7 @@ BOOL WINAPI WinHttpGetIEProxyConfigForCurrentUser( WINHTTP_CURRENT_USER_IE_PROXY
 
 done:
     RegCloseKey( hkey );
-    heap_free( hdr );
+    free( hdr );
     if (!ret)
     {
         GlobalFree( config->lpszAutoConfigUrl );
@@ -1777,7 +1807,7 @@ static char *download_script( const WCHAR *url, DWORD *out_size )
     uc.dwHostNameLength = -1;
     uc.dwUrlPathLength = -1;
     if (!WinHttpCrackUrl( url, 0, 0, &uc )) return NULL;
-    if (!(hostname = heap_alloc( (uc.dwHostNameLength + 1) * sizeof(WCHAR) ))) return NULL;
+    if (!(hostname = malloc( (uc.dwHostNameLength + 1) * sizeof(WCHAR) ))) return NULL;
     memcpy( hostname, uc.lpszHostName, uc.dwHostNameLength * sizeof(WCHAR) );
     hostname[uc.dwHostNameLength] = 0;
 
@@ -1787,12 +1817,12 @@ static char *download_script( const WCHAR *url, DWORD *out_size )
     if (!(req = WinHttpOpenRequest( con, NULL, uc.lpszUrlPath, NULL, NULL, acceptW, flags ))) goto done;
     if (!WinHttpSendRequest( req, NULL, 0, NULL, 0, 0, 0 )) goto done;
 
-    if (!(WinHttpReceiveResponse( req, 0 ))) goto done;
+    if (!WinHttpReceiveResponse( req, 0 )) goto done;
     if (!WinHttpQueryHeaders( req, WINHTTP_QUERY_STATUS_CODE|WINHTTP_QUERY_FLAG_NUMBER, NULL, &status,
         &size, NULL ) || status != HTTP_STATUS_OK) goto done;
 
     size = 4096;
-    if (!(buffer = heap_alloc( size ))) goto done;
+    if (!(buffer = malloc( size ))) goto done;
     to_read = size;
     offset = 0;
     for (;;)
@@ -1806,7 +1836,7 @@ static char *download_script( const WCHAR *url, DWORD *out_size )
         {
             to_read = size;
             size *= 2;
-            if (!(tmp = heap_realloc( buffer, size ))) goto done;
+            if (!(tmp = realloc( buffer, size ))) goto done;
             buffer = tmp;
         }
     }
@@ -1815,7 +1845,7 @@ done:
     WinHttpCloseHandle( req );
     WinHttpCloseHandle( con );
     WinHttpCloseHandle( ses );
-    heap_free( hostname );
+    free( hostname );
     if (!buffer) SetLastError( ERROR_WINHTTP_UNABLE_TO_DOWNLOAD_SCRIPT );
     return buffer;
 }
@@ -1846,7 +1876,7 @@ static BOOL run_script( char *script, DWORD size, const WCHAR *url, WINHTTP_PROX
     if (!(urlA = strdupWA( url ))) return FALSE;
     if (!(ret = InternetInitializeAutoProxyDll( 0, NULL, NULL, NULL, &buffer )))
     {
-        heap_free( urlA );
+        free( urlA );
         return FALSE;
     }
 
@@ -1854,7 +1884,7 @@ static BOOL run_script( char *script, DWORD size, const WCHAR *url, WINHTTP_PROX
     uc.dwStructSize = sizeof(uc);
     uc.dwHostNameLength = -1;
 
-    if (WinHttpCrackUrl( url, 0, 0, &uc ))
+    if ((ret = WinHttpCrackUrl( url, 0, 0, &uc )))
     {
         char *hostnameA = strdupWA_sized( uc.lpszHostName, uc.dwHostNameLength );
 
@@ -1862,13 +1892,14 @@ static BOOL run_script( char *script, DWORD size, const WCHAR *url, WINHTTP_PROX
                         hostnameA, strlen(hostnameA), &result, &len_result )))
         {
             ret = parse_script_result( result, info );
-            heap_free( result );
+            free( result );
         }
 
-        heap_free( hostnameA );
+        free( hostnameA );
     }
-    heap_free( urlA );
-    return InternetDeInitializeAutoProxyDll( NULL, 0 );
+    free( urlA );
+    InternetDeInitializeAutoProxyDll( NULL, 0 );
+    return ret;
 }
 
 /***********************************************************************
@@ -1917,7 +1948,7 @@ BOOL WINAPI WinHttpGetProxyForUrl( HINTERNET hsession, LPCWSTR url, WINHTTP_AUTO
     if ((script = download_script( pac_url, &size )))
     {
         ret = run_script( script, size, url, info );
-        heap_free( script );
+        free( script );
     }
 
 done:
@@ -1989,8 +2020,7 @@ BOOL WINAPI WinHttpSetDefaultProxyConfiguration( WINHTTP_PROXY_INFO *info )
             if (info->lpszProxyBypass)
                 size += lstrlenW( info->lpszProxyBypass );
         }
-        buf = heap_alloc( size );
-        if (buf)
+        if ((buf = malloc( size )))
         {
             struct connection_settings_header *hdr =
                 (struct connection_settings_header *)buf;
@@ -2027,7 +2057,7 @@ BOOL WINAPI WinHttpSetDefaultProxyConfiguration( WINHTTP_PROXY_INFO *info )
             l = RegSetValueExW( key, L"WinHttpSettings", 0, REG_BINARY, buf, size );
             if (!l)
                 ret = TRUE;
-            heap_free( buf );
+            free( buf );
         }
         RegCloseKey( key );
     }
