@@ -122,15 +122,15 @@ static void WCMD_output_asis_len(const WCHAR *message, DWORD len, HANDLE device)
 
 void WINAPIV WCMD_output (const WCHAR *format, ...) {
 
-  __ms_va_list ap;
+  va_list ap;
   WCHAR* string;
   DWORD len;
 
-  __ms_va_start(ap,format);
+  va_start(ap,format);
   string = NULL;
   len = FormatMessageW(FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ALLOCATE_BUFFER,
                        format, 0, 0, (LPWSTR)&string, 0, &ap);
-  __ms_va_end(ap);
+  va_end(ap);
   if (len == 0 && GetLastError() != ERROR_NO_WORK_DONE)
     WINE_FIXME("Could not format string: le=%u, fmt=%s\n", GetLastError(), wine_dbgstr_w(format));
   else
@@ -147,15 +147,15 @@ void WINAPIV WCMD_output (const WCHAR *format, ...) {
 
 void WINAPIV WCMD_output_stderr (const WCHAR *format, ...) {
 
-  __ms_va_list ap;
+  va_list ap;
   WCHAR* string;
   DWORD len;
 
-  __ms_va_start(ap,format);
+  va_start(ap,format);
   string = NULL;
   len = FormatMessageW(FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ALLOCATE_BUFFER,
                        format, 0, 0, (LPWSTR)&string, 0, &ap);
-  __ms_va_end(ap);
+  va_end(ap);
   if (len == 0 && GetLastError() != ERROR_NO_WORK_DONE)
     WINE_FIXME("Could not format string: le=%u, fmt=%s\n", GetLastError(), wine_dbgstr_w(format));
   else
@@ -172,14 +172,14 @@ void WINAPIV WCMD_output_stderr (const WCHAR *format, ...) {
 
 WCHAR* WINAPIV WCMD_format_string (const WCHAR *format, ...)
 {
-  __ms_va_list ap;
+  va_list ap;
   WCHAR* string;
   DWORD len;
 
-  __ms_va_start(ap,format);
+  va_start(ap,format);
   len = FormatMessageW(FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ALLOCATE_BUFFER,
                        format, 0, 0, (LPWSTR)&string, 0, &ap);
-  __ms_va_end(ap);
+  va_end(ap);
   if (len == 0 && GetLastError() != ERROR_NO_WORK_DONE) {
     WINE_FIXME("Could not format string: le=%u, fmt=%s\n", GetLastError(), wine_dbgstr_w(format));
     string = (WCHAR*)LocalAlloc(LMEM_FIXED, 2);
@@ -212,7 +212,7 @@ void WCMD_leave_paged_mode(void)
 }
 
 /***************************************************************************
- * WCMD_Readfile
+ * WCMD_ReadFile
  *
  *	Read characters in from a console/file, returning result in Unicode
  */
@@ -473,7 +473,8 @@ WCHAR *WCMD_skip_leading_spaces (WCHAR *string) {
  *  Checks if the string located at ptr matches a keyword (of length len)
  *  followed by a whitespace character (space or tab)
  */
-BOOL WCMD_keyword_ws_found(const WCHAR *keyword, int len, const WCHAR *ptr) {
+BOOL WCMD_keyword_ws_found(const WCHAR *keyword, const WCHAR *ptr) {
+    const int len = lstrlenW(keyword);
     return (CompareStringW(LOCALE_USER_DEFAULT, NORM_IGNORECASE | SORT_STRINGSORT,
                            ptr, len, keyword, len) == CSTR_EQUAL)
             && ((*(ptr + len) == ' ') || (*(ptr + len) == '\t'));
@@ -1813,11 +1814,6 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
     CMD_LIST *lastEntry = NULL;
     CMD_DELIMITERS prevDelim = CMD_NONE;
     static WCHAR    *extraSpace = NULL;  /* Deliberately never freed */
-    static const WCHAR remCmd[] = {'r','e','m'};
-    static const WCHAR forCmd[] = {'f','o','r'};
-    static const WCHAR ifCmd[]  = {'i','f'};
-    static const WCHAR ifElse[] = {'e','l','s','e'};
-    static const WCHAR setCmd[] = {'s','e','t'};
     BOOL      inOneLine = FALSE;
     BOOL      inFor = FALSE;
     BOOL      inIn  = FALSE;
@@ -1870,10 +1866,7 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
 
     /* Show prompt before batch line IF echo is on and in batch program */
     if (context && echo_mode && *curPos && (*curPos != '@')) {
-      static const WCHAR echoDot[] = {'e','c','h','o','.'};
-      static const WCHAR echoCol[] = {'e','c','h','o',':'};
-      static const WCHAR echoSlash[] = {'e','c','h','o','/'};
-      const DWORD len = ARRAY_SIZE(echoDot);
+      const DWORD len = lstrlenW(L"echo.");
       DWORD curr_size = lstrlenW(curPos);
       DWORD min_len = (curr_size < len ? curr_size : len);
       WCMD_show_prompt(TRUE);
@@ -1881,11 +1874,11 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
       /* I don't know why Windows puts a space here but it does */
       /* Except for lines starting with 'echo.', 'echo:' or 'echo/'. Ask MS why */
       if (CompareStringW(LOCALE_SYSTEM_DEFAULT, NORM_IGNORECASE,
-                         curPos, min_len, echoDot, len) != CSTR_EQUAL
+                         curPos, min_len, L"echo.", len) != CSTR_EQUAL
           && CompareStringW(LOCALE_SYSTEM_DEFAULT, NORM_IGNORECASE,
-                         curPos, min_len, echoCol, len) != CSTR_EQUAL
+                         curPos, min_len, L"echo:", len) != CSTR_EQUAL
           && CompareStringW(LOCALE_SYSTEM_DEFAULT, NORM_IGNORECASE,
-                         curPos, min_len, echoSlash, len) != CSTR_EQUAL)
+                         curPos, min_len, L"echo/", len) != CSTR_EQUAL)
       {
           WCMD_output_asis(L" ");
       }
@@ -1921,13 +1914,11 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
 
       /* Certain commands need special handling */
       if (curStringLen == 0 && curCopyTo == curString) {
-        static const WCHAR forDO[] = {'d','o'};
-
         /* If command starts with 'rem ' or identifies a label, ignore any &&, ( etc. */
-        if (WCMD_keyword_ws_found(remCmd, ARRAY_SIZE(remCmd), curPos) || *curPos == ':') {
+        if (WCMD_keyword_ws_found(L"rem", curPos) || *curPos == ':') {
           inOneLine = TRUE;
 
-        } else if (WCMD_keyword_ws_found(forCmd, ARRAY_SIZE(forCmd), curPos)) {
+        } else if (WCMD_keyword_ws_found(L"for", curPos)) {
           inFor = TRUE;
 
         /* If command starts with 'if ' or 'else ', handle ('s mid line. We should ensure this
@@ -1935,14 +1926,14 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
            should suffice for now.
            To be able to handle ('s in the condition part take as much as evaluate_if_condition
            would take and skip parsing it here. */
-        } else if (WCMD_keyword_ws_found(ifCmd, ARRAY_SIZE(ifCmd), curPos)) {
+        } else if (WCMD_keyword_ws_found(L"if", curPos)) {
           int negate; /* Negate condition */
           int test;   /* Condition evaluation result */
           WCHAR *p, *command;
 
           inIf = TRUE;
 
-          p = curPos+(ARRAY_SIZE(ifCmd));
+          p = curPos+(lstrlenW(L"if"));
           while (*p == ' ' || *p == '\t')
             p++;
           WCMD_parse (p, quals, param1, param2);
@@ -1960,11 +1951,11 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
               curPos+=if_condition_len;
           }
 
-          if (WCMD_keyword_ws_found(setCmd, ARRAY_SIZE(setCmd), curPos))
+          if (WCMD_keyword_ws_found(L"set", curPos))
               ignoreBracket = TRUE;
 
-        } else if (WCMD_keyword_ws_found(ifElse, ARRAY_SIZE(ifElse), curPos)) {
-          const int keyw_len = ARRAY_SIZE(ifElse) + 1;
+        } else if (WCMD_keyword_ws_found(L"else", curPos)) {
+          const int keyw_len = lstrlenW(L"else") + 1;
           inElse = TRUE;
           lastWasElse = TRUE;
           onlyWhiteSpace = TRUE;
@@ -1985,8 +1976,8 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
         /* In a for loop, the DO command will follow a close bracket followed by
            whitespace, followed by DO, ie closeBracket inserts a NULL entry, curLen
            is then 0, and all whitespace is skipped                                */
-        } else if (inFor && WCMD_keyword_ws_found(forDO, ARRAY_SIZE(forDO), curPos)) {
-          const int keyw_len = ARRAY_SIZE(forDO) + 1;
+        } else if (inFor && WCMD_keyword_ws_found(L"do", curPos)) {
+          const int keyw_len = lstrlenW(L"do") + 1;
           WINE_TRACE("Found 'DO '\n");
           lastWasDo = TRUE;
           onlyWhiteSpace = TRUE;
@@ -1999,12 +1990,10 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
 
         /* Special handling for the 'FOR' command */
         if (inFor && lastWasWhiteSpace) {
-          static const WCHAR forIN[] = {'i','n'};
-
           WINE_TRACE("Found 'FOR ', comparing next parm: '%s'\n", wine_dbgstr_w(curPos));
 
-          if (WCMD_keyword_ws_found(forIN, ARRAY_SIZE(forIN), curPos)) {
-            const int keyw_len = ARRAY_SIZE(forIN) + 1;
+          if (WCMD_keyword_ws_found(L"in", curPos)) {
+            const int keyw_len = lstrlenW(L"in") + 1;
             WINE_TRACE("Found 'IN '\n");
             lastWasIn = TRUE;
             onlyWhiteSpace = TRUE;
@@ -2711,7 +2700,7 @@ int __cdecl wmain (int argc, WCHAR *argvW[])
                   size = sizeof(DWORD);
                   RegQueryValueExW(key, L"DefaultColor", NULL, NULL, (BYTE *)&value, &size);
               } else if (type == REG_SZ) {
-                  size = ARRAY_SIZE(strvalue);
+                  size = sizeof(strvalue);
                   RegQueryValueExW(key, L"DefaultColor", NULL, NULL, (BYTE *)strvalue, &size);
                   value = wcstoul(strvalue, NULL, 10);
               }
@@ -2730,7 +2719,7 @@ int __cdecl wmain (int argc, WCHAR *argvW[])
                   size = sizeof(DWORD);
                   RegQueryValueExW(key, L"DefaultColor", NULL, NULL, (BYTE *)&value, &size);
               } else if (type == REG_SZ) {
-                  size = ARRAY_SIZE(strvalue);
+                  size = sizeof(strvalue);
                   RegQueryValueExW(key, L"DefaultColor", NULL, NULL, (BYTE *)strvalue, &size);
                   value = wcstoul(strvalue, NULL, 10);
               }
