@@ -836,6 +836,7 @@ static HMODULE load_graphics_driver( const WCHAR *driver, const GUID *guid )
         if (!wcscmp( name, L"null" ))
         {
             TRACE( "display %s using null driver\n", debugstr_guid(guid) );
+            wcscpy( libname, L"null" );
             null_driver = TRUE;
             break;
         }
@@ -959,6 +960,7 @@ void manage_desktop( WCHAR *arg )
     const WCHAR *name = NULL;
     BOOL enable_shell = FALSE;
     void (WINAPI *pShellDDEInit)( BOOL ) = NULL;
+    HMODULE shell32;
 
     /* get the rest of the command line (if any) */
     while (*p && !is_whitespace(*p)) p++;
@@ -1028,20 +1030,15 @@ void manage_desktop( WCHAR *arg )
         initialize_display_settings();
         initialize_appbar();
 
-        if (graphics_driver)
+        if (using_root) enable_shell = FALSE;
+
+        initialize_systray( graphics_driver, using_root, enable_shell );
+        if (!using_root) initialize_launchers( hwnd );
+
+        if ((shell32 = LoadLibraryW( L"shell32.dll" )) &&
+            (pShellDDEInit = (void *)GetProcAddress( shell32, (LPCSTR)188)))
         {
-            HMODULE shell32;
-
-            if (using_root) enable_shell = FALSE;
-
-            initialize_systray( graphics_driver, using_root, enable_shell );
-            if (!using_root) initialize_launchers( hwnd );
-
-            if ((shell32 = LoadLibraryW( L"shell32.dll" )) &&
-                (pShellDDEInit = (void *)GetProcAddress( shell32, (LPCSTR)188)))
-            {
-                pShellDDEInit( TRUE );
-            }
+            pShellDDEInit( TRUE );
         }
     }
 
@@ -1219,6 +1216,7 @@ static HRESULT WINAPI shellwindows_Register(IShellWindows *iface,
     window->hwnd = hwnd;
     window->class = class;
     *cookie = window->cookie = ++cookie_counter;
+    window->pidl = NULL;
 
     LeaveCriticalSection(&sw->cs);
     return S_OK;
