@@ -29,7 +29,6 @@
 #include <stdlib.h>
 #include <assert.h>
 #include "wine/heap.h"
-#include "wine/unicode.h"
 #include "winecfg.h"
 #include "resource.h"
 
@@ -37,90 +36,90 @@ WINE_DEFAULT_DEBUG_CHANNEL(winecfg);
 
 struct win_version
 {
-    const char *szVersion;
-    const char *szDescription;
-    DWORD       dwMajorVersion;
-    DWORD       dwMinorVersion;
-    DWORD       dwBuildNumber;
-    DWORD       dwPlatformId;
-    const char *szCSDVersion;
-    WORD        wServicePackMajor;
-    WORD        wServicePackMinor;
-    const char *szProductType;
+    const WCHAR *szVersion;
+    const WCHAR *szDescription;
+    DWORD        dwMajorVersion;
+    DWORD        dwMinorVersion;
+    DWORD        dwBuildNumber;
+    DWORD        dwPlatformId;
+    const WCHAR *szCSDVersion;
+    WORD         wServicePackMajor;
+    WORD         wServicePackMinor;
+    const WCHAR *szProductType;
 };
 
 static const struct win_version win_versions[] =
 {
-    { "win10",       "Windows 10",       10,  0, 0x4563,VER_PLATFORM_WIN32_NT, "", 0, 0, "WinNT"},
-    { "win81",       "Windows 8.1",       6,  3, 0x2580,VER_PLATFORM_WIN32_NT, "", 0, 0, "WinNT"},
-    { "win8",        "Windows 8",         6,  2, 0x23F0,VER_PLATFORM_WIN32_NT, "", 0, 0, "WinNT"},
-    { "win2008r2",   "Windows 2008 R2",   6,  1, 0x1DB1,VER_PLATFORM_WIN32_NT, "Service Pack 1", 1, 0, "ServerNT"},
-    { "win7",        "Windows 7",         6,  1, 0x1DB1,VER_PLATFORM_WIN32_NT, "Service Pack 1", 1, 0, "WinNT"},
-    { "win2008",     "Windows 2008",      6,  0, 0x1772,VER_PLATFORM_WIN32_NT, "Service Pack 2", 2, 0, "ServerNT"},
-    { "vista",       "Windows Vista",     6,  0, 0x1772,VER_PLATFORM_WIN32_NT, "Service Pack 2", 2, 0, "WinNT"},
-    { "win2003",     "Windows 2003",      5,  2, 0xECE, VER_PLATFORM_WIN32_NT, "Service Pack 2", 2, 0, "ServerNT"},
+    { L"win10",     L"Windows 10",       10,  0, 0x4563,VER_PLATFORM_WIN32_NT, L"", 0, 0, L"WinNT"},
+    { L"win81",     L"Windows 8.1",       6,  3, 0x2580,VER_PLATFORM_WIN32_NT, L"", 0, 0, L"WinNT"},
+    { L"win8",      L"Windows 8",         6,  2, 0x23F0,VER_PLATFORM_WIN32_NT, L"", 0, 0, L"WinNT"},
+    { L"win2008r2", L"Windows 2008 R2",   6,  1, 0x1DB1,VER_PLATFORM_WIN32_NT, L"Service Pack 1", 1, 0, L"ServerNT"},
+    { L"win7",      L"Windows 7",         6,  1, 0x1DB1,VER_PLATFORM_WIN32_NT, L"Service Pack 1", 1, 0, L"WinNT"},
+    { L"win2008",   L"Windows 2008",      6,  0, 0x1772,VER_PLATFORM_WIN32_NT, L"Service Pack 2", 2, 0, L"ServerNT"},
+    { L"vista",     L"Windows Vista",     6,  0, 0x1772,VER_PLATFORM_WIN32_NT, L"Service Pack 2", 2, 0, L"WinNT"},
+    { L"win2003",   L"Windows 2003",      5,  2, 0xECE, VER_PLATFORM_WIN32_NT, L"Service Pack 2", 2, 0, L"ServerNT"},
 #ifdef _WIN64
-    { "winxp64",     "Windows XP",        5,  2, 0xECE, VER_PLATFORM_WIN32_NT, "Service Pack 2", 2, 0, "WinNT"},
+    { L"winxp64",   L"Windows XP",        5,  2, 0xECE, VER_PLATFORM_WIN32_NT, L"Service Pack 2", 2, 0, L"WinNT"},
 #else
-    { "winxp",       "Windows XP",        5,  1, 0xA28, VER_PLATFORM_WIN32_NT, "Service Pack 3", 3, 0, "WinNT"},
-    { "win2k",       "Windows 2000",      5,  0, 0x893, VER_PLATFORM_WIN32_NT, "Service Pack 4", 4, 0, "WinNT"},
-    { "winme",       "Windows ME",        4, 90, 0xBB8, VER_PLATFORM_WIN32_WINDOWS, " ", 0, 0, ""},
-    { "win98",       "Windows 98",        4, 10, 0x8AE, VER_PLATFORM_WIN32_WINDOWS, " A ", 0, 0, ""},
-    { "win95",       "Windows 95",        4,  0, 0x3B6, VER_PLATFORM_WIN32_WINDOWS, "", 0, 0, ""},
-    { "nt40",        "Windows NT 4.0",    4,  0, 0x565, VER_PLATFORM_WIN32_NT, "Service Pack 6a", 6, 0, "WinNT"},
-    { "nt351",       "Windows NT 3.51",   3, 51, 0x421, VER_PLATFORM_WIN32_NT, "Service Pack 5", 5, 0, "WinNT"},
-    { "win31",       "Windows 3.1",       3, 10,     0, VER_PLATFORM_WIN32s, "Win32s 1.3", 0, 0, ""},
-    { "win30",       "Windows 3.0",       3,  0,     0, VER_PLATFORM_WIN32s, "Win32s 1.3", 0, 0, ""},
-    { "win20",       "Windows 2.0",       2,  0,     0, VER_PLATFORM_WIN32s, "Win32s 1.3", 0, 0, ""}
+    { L"winxp",     L"Windows XP",        5,  1, 0xA28, VER_PLATFORM_WIN32_NT, L"Service Pack 3", 3, 0, L"WinNT"},
+    { L"win2k",     L"Windows 2000",      5,  0, 0x893, VER_PLATFORM_WIN32_NT, L"Service Pack 4", 4, 0, L"WinNT"},
+    { L"winme",     L"Windows ME",        4, 90, 0xBB8, VER_PLATFORM_WIN32_WINDOWS, L" ", 0, 0, L""},
+    { L"win98",     L"Windows 98",        4, 10, 0x8AE, VER_PLATFORM_WIN32_WINDOWS, L" A ", 0, 0, L""},
+    { L"win95",     L"Windows 95",        4,  0, 0x3B6, VER_PLATFORM_WIN32_WINDOWS, L"", 0, 0, L""},
+    { L"nt40",      L"Windows NT 4.0",    4,  0, 0x565, VER_PLATFORM_WIN32_NT, L"Service Pack 6a", 6, 0, L"WinNT"},
+    { L"nt351",     L"Windows NT 3.51",   3, 51, 0x421, VER_PLATFORM_WIN32_NT, L"Service Pack 5", 5, 0, L"WinNT"},
+    { L"win31",     L"Windows 3.1",       3, 10,     0, VER_PLATFORM_WIN32s, L"Win32s 1.3", 0, 0, L""},
+    { L"win30",     L"Windows 3.0",       3,  0,     0, VER_PLATFORM_WIN32s, L"Win32s 1.3", 0, 0, L""},
+    { L"win20",     L"Windows 2.0",       2,  0,     0, VER_PLATFORM_WIN32s, L"Win32s 1.3", 0, 0, L""}
 #endif
 };
 
-#define DEFAULT_WIN_VERSION   "win7"
+#define DEFAULT_WIN_VERSION   L"win7"
 
-static const char szKey9x[] = "Software\\Microsoft\\Windows\\CurrentVersion";
-static const char szKeyNT[] = "Software\\Microsoft\\Windows NT\\CurrentVersion";
-static const char szKeyProdNT[] = "System\\CurrentControlSet\\Control\\ProductOptions";
+static const WCHAR szKey9x[] = L"Software\\Microsoft\\Windows\\CurrentVersion";
+static const WCHAR szKeyNT[] = L"Software\\Microsoft\\Windows NT\\CurrentVersion";
+static const WCHAR szKeyProdNT[] = L"System\\CurrentControlSet\\Control\\ProductOptions";
 
 static int get_registry_version(void)
 {
     int i, best = -1, platform, major, minor = 0, build = 0;
-    char *p, *ver, *type = NULL;
+    WCHAR *p, *ver, *type = NULL;
 
-    if ((ver = get_reg_key( HKEY_LOCAL_MACHINE, szKeyNT, "CurrentVersion", NULL )))
+    if ((ver = get_reg_key( HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentVersion", NULL )))
     {
-        char *build_str;
+        WCHAR *build_str;
 
         platform = VER_PLATFORM_WIN32_NT;
 
-        build_str = get_reg_key( HKEY_LOCAL_MACHINE, szKeyNT, "CurrentBuildNumber", NULL );
-        build = atoi(build_str);
+        build_str = get_reg_key( HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentBuildNumber", NULL );
+        build = wcstol(build_str, NULL, 10);
 
-        type = get_reg_key( HKEY_LOCAL_MACHINE, szKeyProdNT, "ProductType", NULL );
+        type = get_reg_key( HKEY_LOCAL_MACHINE, szKeyProdNT, L"ProductType", NULL );
     }
-    else if ((ver = get_reg_key( HKEY_LOCAL_MACHINE, szKey9x, "VersionNumber", NULL )))
+    else if ((ver = get_reg_key( HKEY_LOCAL_MACHINE, szKey9x, L"VersionNumber", NULL )))
         platform = VER_PLATFORM_WIN32_WINDOWS;
     else
         return -1;
 
-    if ((p = strchr( ver, '.' )))
+    if ((p = wcschr( ver, '.' )))
     {
-        char *minor_str = p;
+        WCHAR *minor_str = p;
         *minor_str++ = 0;
-        if ((p = strchr( minor_str, '.' )))
+        if ((p = wcschr( minor_str, '.' )))
         {
-            char *build_str = p;
+            WCHAR *build_str = p;
             *build_str++ = 0;
-            build = atoi(build_str);
+            build = wcstol(build_str, NULL, 10);
         }
-        minor = atoi(minor_str);
+        minor = wcstol(minor_str, NULL, 10);
     }
-    major = atoi(ver);
+    major = wcstol(ver, NULL, 10);
 
     for (i = 0; i < ARRAY_SIZE(win_versions); i++)
     {
         if (win_versions[i].dwPlatformId != platform) continue;
         if (win_versions[i].dwMajorVersion != major) continue;
-        if (type && strcasecmp(win_versions[i].szProductType, type)) continue;
+        if (type && wcsicmp(win_versions[i].szProductType, type)) continue;
         best = i;
         if ((win_versions[i].dwMinorVersion == minor) &&
             (win_versions[i].dwBuildNumber == build))
@@ -132,10 +131,10 @@ static int get_registry_version(void)
 static void update_comboboxes(HWND dialog)
 {
     int i, ver;
-    char *winver;
+    WCHAR *winver;
 
     /* retrieve the registry values */
-    winver = get_reg_key(config_key, keypath(""), "Version", "");
+    winver = get_reg_key(config_key, keypath(L""), L"Version", L"");
     ver = get_registry_version();
 
     if (!winver || !winver[0])
@@ -148,19 +147,19 @@ static void update_comboboxes(HWND dialog)
             SendDlgItemMessageW(dialog, IDC_WINVER, CB_SETCURSEL, 0, 0);
             return;
         }
-        if (ver != -1) winver = strdupA( win_versions[ver].szVersion );
-        else winver = strdupA(DEFAULT_WIN_VERSION);
+        if (ver != -1) winver = strdupW( win_versions[ver].szVersion );
+        else winver = strdupW(DEFAULT_WIN_VERSION);
     }
-    WINE_TRACE("winver is %s\n", winver);
+    WINE_TRACE("winver is %s\n", debugstr_w(winver));
 
     /* normalize the version strings */
     for (i = 0; i < ARRAY_SIZE(win_versions); i++)
     {
-        if (!strcasecmp (win_versions[i].szVersion, winver))
+        if (!wcsicmp(win_versions[i].szVersion, winver))
         {
             SendDlgItemMessageW(dialog, IDC_WINVER, CB_SETCURSEL,
                                 i + (current_app?1:0), 0);
-            WINE_TRACE("match with %s\n", win_versions[i].szVersion);
+            WINE_TRACE("match with %s\n", debugstr_w(win_versions[i].szVersion));
             break;
 	}
     }
@@ -185,7 +184,7 @@ init_comboboxes (HWND dialog)
 
     for (i = 0; i < ARRAY_SIZE(win_versions); i++)
     {
-      SendDlgItemMessageA(dialog, IDC_WINVER, CB_ADDSTRING,
+      SendDlgItemMessageW(dialog, IDC_WINVER, CB_ADDSTRING,
                           0, (LPARAM) win_versions[i].szDescription);
     }
 }
@@ -224,7 +223,7 @@ static void init_appsheet(HWND dialog)
   add_listview_item(listview, appname, NULL);
 
   /* because this list is only populated once, it's safe to bypass the settings list here  */
-  if (RegOpenKeyA(config_key, "AppDefaults", &key) == ERROR_SUCCESS)
+  if (RegOpenKeyW(config_key, L"AppDefaults", &key) == ERROR_SUCCESS)
   {
       i = 0;
       size = ARRAY_SIZE(appname);
@@ -320,16 +319,14 @@ static BOOL list_contains_file(HWND listview, WCHAR *filename)
 
 static void on_add_app_click(HWND dialog)
 {
-  static const WCHAR filterW[] = {'%','s','%','c','*','.','e','x','e',';','*','.','e','x','e','.','s','o','%','c',0};
   WCHAR filetitle[MAX_PATH];
   WCHAR file[MAX_PATH];
   WCHAR programsFilter[100], filter[MAX_PATH];
   WCHAR selectExecutableStr[100];
-  static const WCHAR pathC[] = { 'c',':','\\',0 };
 
   OPENFILENAMEW ofn = { sizeof(OPENFILENAMEW),
 		       dialog, /*hInst*/0, 0, NULL, 0, 0, NULL,
-		       0, NULL, 0, pathC, 0,
+		       0, NULL, 0, L"C:\\", 0,
 		       OFN_SHOWHELP | OFN_HIDEREADONLY | OFN_ENABLESIZING,
                        0, 0, NULL, 0, NULL };
 
@@ -337,7 +334,7 @@ static void on_add_app_click(HWND dialog)
       ARRAY_SIZE(selectExecutableStr));
   LoadStringW (GetModuleHandleW(NULL), IDS_EXECUTABLE_FILTER, programsFilter,
       ARRAY_SIZE(programsFilter));
-  snprintfW( filter, MAX_PATH, filterW, programsFilter, 0, 0 );
+  swprintf( filter, MAX_PATH, L"%s%c*.exe;*.exe.so%c", programsFilter, 0, 0 );
 
   ofn.lpstrTitle = selectExecutableStr;
   ofn.lpstrFilter = filter;
@@ -388,7 +385,7 @@ static void on_remove_app_click(HWND dialog)
 
     assert( selection != 0 ); /* user cannot click this button when "default settings" is selected  */
 
-    set_reg_key(config_key, keypath(""), NULL, NULL); /* delete the section  */
+    set_reg_key(config_key, keypath(L""), NULL, NULL); /* delete the section  */
     SendMessageW(listview, LVM_GETITEMW, 0, (LPARAM) &item);
     HeapFree (GetProcessHeap(), 0, (void*)item.lParam);
     SendMessageW(listview, LVM_DELETEITEM, selection, 0);
@@ -403,85 +400,85 @@ static void on_remove_app_click(HWND dialog)
 
 static void set_winver(const struct win_version *version)
 {
-    static const char szKeyWindNT[] = "System\\CurrentControlSet\\Control\\Windows";
-    static const char szKeyEnvNT[]  = "System\\CurrentControlSet\\Control\\Session Manager\\Environment";
-    char Buffer[40];
+    static const WCHAR szKeyWindNT[] = L"System\\CurrentControlSet\\Control\\Windows";
+    static const WCHAR szKeyEnvNT[]  = L"System\\CurrentControlSet\\Control\\Session Manager\\Environment";
+    WCHAR buffer[40];
 
     switch (version->dwPlatformId)
     {
         case VER_PLATFORM_WIN32_WINDOWS:
-            snprintf(Buffer, sizeof(Buffer), "%d.%d.%d", version->dwMajorVersion,
+            swprintf(buffer, ARRAY_SIZE(buffer), L"%d.%d.%d", version->dwMajorVersion,
                         version->dwMinorVersion, version->dwBuildNumber);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, "VersionNumber", Buffer);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, "SubVersionNumber", version->szCSDVersion);
-            snprintf(Buffer, sizeof(Buffer), "Microsoft %s", version->szDescription);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, "ProductName", Buffer);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, L"VersionNumber", buffer);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, L"SubVersionNumber", version->szCSDVersion);
+            swprintf(buffer, ARRAY_SIZE(buffer), L"Microsoft %s", version->szDescription);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, L"ProductName", buffer);
 
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CSDVersion", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CurrentVersion", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CurrentMajorVersionNumber", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CurrentMinorVersionNumber", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CurrentBuild", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CurrentBuildNumber", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "ProductName", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyProdNT, "ProductType", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyWindNT, "CSDVersion", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyEnvNT, "OS", NULL);
-            set_reg_key(config_key, keypath(""), "Version", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CSDVersion", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentVersion", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentMajorVersionNumber", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentMinorVersionNumber", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentBuild", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentBuildNumber", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"ProductName", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyProdNT, L"ProductType", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyWindNT, L"CSDVersion", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyEnvNT, L"OS", NULL);
+            set_reg_key(config_key, keypath(L""), L"Version", NULL);
             break;
 
         case VER_PLATFORM_WIN32_NT:
-            snprintf(Buffer, sizeof(Buffer), "%d.%d", version->dwMajorVersion,
+            swprintf(buffer, ARRAY_SIZE(buffer), L"%d.%d", version->dwMajorVersion,
                         version->dwMinorVersion);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CurrentVersion", Buffer);
-            set_reg_key_dword(HKEY_LOCAL_MACHINE, szKeyNT, "CurrentMajorVersionNumber", version->dwMajorVersion);
-            set_reg_key_dword(HKEY_LOCAL_MACHINE, szKeyNT, "CurrentMinorVersionNumber", version->dwMinorVersion);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CSDVersion", version->szCSDVersion);
-            snprintf(Buffer, sizeof(Buffer), "%d", version->dwBuildNumber);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CurrentBuild", Buffer);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CurrentBuildNumber", Buffer);
-            snprintf(Buffer, sizeof(Buffer), "Microsoft %s", version->szDescription);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "ProductName", Buffer);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyProdNT, "ProductType", version->szProductType);
-            set_reg_key_dword(HKEY_LOCAL_MACHINE, szKeyWindNT, "CSDVersion",
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentVersion", buffer);
+            set_reg_key_dword(HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentMajorVersionNumber", version->dwMajorVersion);
+            set_reg_key_dword(HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentMinorVersionNumber", version->dwMinorVersion);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CSDVersion", version->szCSDVersion);
+            swprintf(buffer, ARRAY_SIZE(buffer), L"%d", version->dwBuildNumber);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentBuild", buffer);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentBuildNumber", buffer);
+            swprintf(buffer, ARRAY_SIZE(buffer), L"Microsoft %s", version->szDescription);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"ProductName", buffer);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyProdNT, L"ProductType", version->szProductType);
+            set_reg_key_dword(HKEY_LOCAL_MACHINE, szKeyWindNT, L"CSDVersion",
                                 MAKEWORD( version->wServicePackMinor,
                                         version->wServicePackMajor ));
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyEnvNT, "OS", "Windows_NT");
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyEnvNT, L"OS", L"Windows_NT");
 
-            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, "VersionNumber", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, "SubVersionNumber", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, "ProductName", NULL);
-            set_reg_key(config_key, keypath(""), "Version", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, L"VersionNumber", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, L"SubVersionNumber", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, L"ProductName", NULL);
+            set_reg_key(config_key, keypath(L""), L"Version", NULL);
             break;
 
         case VER_PLATFORM_WIN32s:
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CSDVersion", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CurrentVersion", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CurrentBuild", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "CurrentBuildNumber", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, "ProductName", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyProdNT, "ProductType", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyWindNT, "CSDVersion", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKeyEnvNT, "OS", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, "VersionNumber", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, "SubVersionNumber", NULL);
-            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, "ProductName", NULL);
-            set_reg_key(config_key, keypath(""), "Version", version->szVersion);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CSDVersion", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentVersion", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentBuild", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"CurrentBuildNumber", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyNT, L"ProductName", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyProdNT, L"ProductType", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyWindNT, L"CSDVersion", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKeyEnvNT, L"OS", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, L"VersionNumber", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, L"SubVersionNumber", NULL);
+            set_reg_key(HKEY_LOCAL_MACHINE, szKey9x, L"ProductName", NULL);
+            set_reg_key(config_key, keypath(L""), L"Version", version->szVersion);
             break;
     }
 }
 
-BOOL set_winver_from_string(const char *version)
+BOOL set_winver_from_string(const WCHAR *version)
 {
     int i;
 
-    WINE_TRACE("desired winver: '%s'\n", version);
+    WINE_TRACE("desired winver: %s\n", debugstr_w(version));
 
     for (i = 0; i < ARRAY_SIZE(win_versions); i++)
     {
-        if (!lstrcmpiA(win_versions[i].szVersion, version))
+        if (!wcsicmp(win_versions[i].szVersion, version))
         {
-            WINE_TRACE("match with %s\n", win_versions[i].szVersion);
+            WINE_TRACE("match with %s\n", debugstr_w(win_versions[i].szVersion));
             set_winver(&win_versions[i]);
             apply();
             return TRUE;
@@ -497,25 +494,21 @@ void print_windows_versions(void)
 
     for (i = 0; i < ARRAY_SIZE(win_versions); i++)
     {
-        printf("  %10s  %s\n", win_versions[i].szVersion, win_versions[i].szDescription);
+        wprintf(L"  %10s  %s\n", win_versions[i].szVersion, win_versions[i].szDescription);
     }
 }
 
 void print_current_winver(void)
 {
-    char *winver = get_reg_key(config_key, keypath(""), "Version", "");
+    WCHAR *winver = get_reg_key(config_key, keypath(L""), L"Version", L"");
 
     if (!winver || !winver[0])
     {
         int ver = get_registry_version();
-
-        if (ver == -1)
-            printf(DEFAULT_WIN_VERSION "\n");
-        else
-            printf("%s\n", win_versions[ver].szVersion);
+        wprintf(L"%s\n", ver == -1 ? DEFAULT_WIN_VERSION : win_versions[ver].szVersion);
     }
     else
-        printf("%s\n", winver);
+        wprintf(L"%s\n", winver);
 
     heap_free(winver);
 }
@@ -529,12 +522,12 @@ static void on_winver_change(HWND dialog)
         if (!selection)
         {
             WINE_TRACE("default selected so removing current setting\n");
-            set_reg_key(config_key, keypath(""), "Version", NULL);
+            set_reg_key(config_key, keypath(L""), L"Version", NULL);
         }
         else
         {
-            WINE_TRACE("setting Version key to value '%s'\n", win_versions[selection-1].szVersion);
-            set_reg_key(config_key, keypath(""), "Version", win_versions[selection-1].szVersion);
+            WINE_TRACE("setting Version key to value %s\n", debugstr_w(win_versions[selection-1].szVersion));
+            set_reg_key(config_key, keypath(L""), L"Version", win_versions[selection-1].szVersion);
         }
     }
     else /* global version only */
