@@ -26,6 +26,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <sys/types.h>
 #include <unistd.h>
 
 #include "ntstatus.h"
@@ -97,6 +98,21 @@ const PSID security_high_label_sid = (PSID)&high_label_sid;
 
 static luid_t prev_luid_value = { 1000, 0 };
 
+static const WCHAR token_name[] = {'T','o','k','e','n'};
+
+struct type_descr token_type =
+{
+    { token_name, sizeof(token_name) },   /* name */
+    TOKEN_ALL_ACCESS | SYNCHRONIZE,       /* valid_access */
+    {                                     /* mapping */
+        STANDARD_RIGHTS_READ | TOKEN_QUERY_SOURCE | TOKEN_QUERY | TOKEN_DUPLICATE,
+        STANDARD_RIGHTS_WRITE | TOKEN_ADJUST_SESSIONID | TOKEN_ADJUST_DEFAULT | TOKEN_ADJUST_GROUPS
+        | TOKEN_ADJUST_PRIVILEGES,
+        STANDARD_RIGHTS_EXECUTE | TOKEN_IMPERSONATE | TOKEN_ASSIGN_PRIMARY,
+        TOKEN_ALL_ACCESS
+    },
+};
+
 struct token
 {
     struct object  obj;             /* object header */
@@ -107,10 +123,12 @@ struct token
     SID           *user;            /* SID of user this token represents */
     SID           *owner;           /* SID of owner (points to user or one of groups) */
     SID           *primary_group;   /* SID of user's primary group (points to one of groups) */
-    unsigned       primary;         /* is this a primary or impersonation token? */
+    unsigned int   primary;         /* is this a primary or impersonation token? */
+    unsigned int   session_id;      /* token session id */
     ACL           *default_dacl;    /* the default DACL to assign to objects created by this user */
     TOKEN_SOURCE   source;          /* source of the token */
     int            impersonation_level; /* impersonation level this token is capable of if non-primary token */
+    int            elevation;       /* elevation type */
 };
 
 struct privilege
@@ -135,22 +153,20 @@ struct group
 };
 
 static void token_dump( struct object *obj, int verbose );
-static struct object_type *token_get_type( struct object *obj );
-static unsigned int token_map_access( struct object *obj, unsigned int access );
 static void token_destroy( struct object *obj );
 
 static const struct object_ops token_ops =
 {
     sizeof(struct token),      /* size */
+    &token_type,               /* type */
     token_dump,                /* dump */
-    token_get_type,            /* get_type */
     no_add_queue,              /* add_queue */
     NULL,                      /* remove_queue */
     NULL,                      /* signaled */
     NULL,                      /* satisfied */
     no_signal,                 /* signal */
     no_get_fd,                 /* get_fd */
-    token_map_access,          /* map_access */
+    default_map_access,        /* map_access */
     default_get_sd,            /* get_sd */
     default_set_sd,            /* set_sd */
     no_get_full_name,          /* get_full_name */
@@ -171,22 +187,6 @@ static void token_dump( struct object *obj, int verbose )
              token->token_id.low_part, token->primary, token->impersonation_level );
 }
 
-static struct object_type *token_get_type( struct object *obj )
-{
-    static const WCHAR name[] = {'T','o','k','e','n'};
-    static const struct unicode_str str = { name, sizeof(name) };
-    return get_object_type( &str );
-}
-
-static unsigned int token_map_access( struct object *obj, unsigned int access )
-{
-    if (access & GENERIC_READ)    access |= TOKEN_READ;
-    if (access & GENERIC_WRITE)   access |= TOKEN_WRITE;
-    if (access & GENERIC_EXECUTE) access |= STANDARD_RIGHTS_EXECUTE;
-    if (access & GENERIC_ALL)     access |= TOKEN_ALL_ACCESS;
-    return access & ~(GENERIC_READ | GENERIC_WRITE | GENERIC_EXECUTE | GENERIC_ALL);
-}
-
 static SID *security_sid_alloc( const SID_IDENTIFIER_AUTHORITY *idauthority, int subauthcount, const unsigned int subauth[] )
 {
     int i;
@@ -463,16 +463,6 @@ ACL *replace_security_labels( const ACL *old_sacl, const ACL *new_sacl )
     return replaced_acl;
 }
 
-/* maps from generic rights to specific rights as given by a mapping */
-static inline void map_generic_mask(unsigned int *mask, const GENERIC_MAPPING *mapping)
-{
-    if (*mask & GENERIC_READ) *mask |= mapping->GenericRead;
-    if (*mask & GENERIC_WRITE) *mask |= mapping->GenericWrite;
-    if (*mask & GENERIC_EXECUTE) *mask |= mapping->GenericExecute;
-    if (*mask & GENERIC_ALL) *mask |= mapping->GenericAll;
-    *mask &= 0x0FFFFFFF;
-}
-
 static inline int is_equal_luid( const LUID *luid1, const LUID *luid2 )
 {
     return (luid1->LowPart == luid2->LowPart && luid1->HighPart == luid2->HighPart);
@@ -549,12 +539,12 @@ static void token_destroy( struct object *obj )
  *  modified_id may be NULL, indicating that a new modified_id luid should be
  *   allocated.
  */
-static struct token *create_token( unsigned primary, const SID *user,
+static struct token *create_token( unsigned int primary, unsigned int session_id, const SID *user,
                                    const SID_AND_ATTRIBUTES *groups, unsigned int group_count,
                                    const LUID_AND_ATTRIBUTES *privs, unsigned int priv_count,
                                    const ACL *default_dacl, TOKEN_SOURCE source,
                                    const luid_t *modified_id,
-                                   int impersonation_level )
+                                   int impersonation_level, int elevation )
 {
     struct token *token = alloc_object( &token_ops );
     if (token)
@@ -569,6 +559,7 @@ static struct token *create_token( unsigned primary, const SID *user,
         list_init( &token->privileges );
         list_init( &token->groups );
         token->primary = primary;
+        token->session_id = session_id;
         /* primary tokens don't have impersonation levels */
         if (primary)
             token->impersonation_level = -1;
@@ -576,6 +567,7 @@ static struct token *create_token( unsigned primary, const SID *user,
             token->impersonation_level = impersonation_level;
         token->default_dacl = NULL;
         token->primary_group = NULL;
+        token->elevation = elevation;
 
         /* copy user */
         token->user = memdup( user, security_sid_len( user ));
@@ -688,10 +680,10 @@ struct token *token_duplicate( struct token *src_token, unsigned primary,
         return NULL;
     }
 
-    token = create_token( primary, src_token->user, NULL, 0,
+    token = create_token( primary, src_token->session_id, src_token->user, NULL, 0,
                           NULL, 0, src_token->default_dacl,
                           src_token->source, modified_id,
-                          impersonation_level );
+                          impersonation_level, src_token->elevation );
     if (!token) return token;
 
     /* copy groups */
@@ -841,7 +833,7 @@ struct token *get_token_obj( struct process *process, obj_handle_t handle, unsig
     return (struct token *)get_handle_obj( process, handle, access, &token_ops );
 }
 
-struct token *token_create_admin( void )
+struct token *token_create_admin( unsigned primary, int impersonation_level, int elevation, unsigned int session_id )
 {
     struct token *token = NULL;
     static const SID_IDENTIFIER_AUTHORITY nt_authority = { SECURITY_NT_AUTHORITY };
@@ -901,9 +893,9 @@ struct token *token_create_admin( void )
             { logon_sid, SE_GROUP_ENABLED|SE_GROUP_ENABLED_BY_DEFAULT|SE_GROUP_MANDATORY|SE_GROUP_LOGON_ID },
         };
         static const TOKEN_SOURCE admin_source = {"SeMgr", {0, 0}};
-        token = create_token( TRUE, user_sid, admin_groups, ARRAY_SIZE( admin_groups ),
+        token = create_token( primary, session_id, user_sid, admin_groups, ARRAY_SIZE( admin_groups ),
                               admin_privs, ARRAY_SIZE( admin_privs ), default_dacl,
-                              admin_source, NULL, -1 );
+                              admin_source, NULL, impersonation_level, elevation );
         /* we really need a primary group */
         assert( token->primary_group );
     }
@@ -1039,7 +1031,7 @@ static unsigned int token_access_check( struct token *token,
                                  unsigned int desired_access,
                                  LUID_AND_ATTRIBUTES *privs,
                                  unsigned int *priv_count,
-                                 const GENERIC_MAPPING *mapping,
+                                 const generic_map_t *mapping,
                                  unsigned int *granted_access,
                                  unsigned int *status )
 {
@@ -1074,7 +1066,7 @@ static unsigned int token_access_check( struct token *token,
     {
         if (priv_count) *priv_count = 0;
         if (desired_access & MAXIMUM_ALLOWED)
-            *granted_access = mapping->GenericAll;
+            *granted_access = mapping->all;
         else
             *granted_access = desired_access;
         return *status = STATUS_SUCCESS;
@@ -1150,8 +1142,7 @@ static unsigned int token_access_check( struct token *token,
             sid = (const SID *)&ad_ace->SidStart;
             if (token_sid_present( token, sid, TRUE ))
             {
-                unsigned int access = ad_ace->Mask;
-                map_generic_mask(&access, mapping);
+                unsigned int access = map_access( ad_ace->Mask, mapping );
                 if (desired_access & MAXIMUM_ALLOWED)
                     denied_access |= access;
                 else
@@ -1166,8 +1157,7 @@ static unsigned int token_access_check( struct token *token,
             sid = (const SID *)&aa_ace->SidStart;
             if (token_sid_present( token, sid, FALSE ))
             {
-                unsigned int access = aa_ace->Mask;
-                map_generic_mask(&access, mapping);
+                unsigned int access = map_access( aa_ace->Mask, mapping );
                 if (desired_access & MAXIMUM_ALLOWED)
                     current_access |= access;
                 else
@@ -1210,27 +1200,31 @@ const SID *token_get_primary_group( struct token *token )
     return token->primary_group;
 }
 
+unsigned int token_get_session_id( struct token *token )
+{
+    return token->session_id;
+}
+
 int check_object_access(struct token *token, struct object *obj, unsigned int *access)
 {
-    GENERIC_MAPPING mapping;
+    generic_map_t mapping;
     unsigned int status;
     int res;
 
     if (!token)
         token = current->token ? current->token : current->process->token;
 
-    mapping.GenericAll = obj->ops->map_access( obj, GENERIC_ALL );
+    mapping.all = obj->ops->map_access( obj, GENERIC_ALL );
 
     if (!obj->sd)
     {
-        if (*access & MAXIMUM_ALLOWED)
-            *access = mapping.GenericAll;
+        if (*access & MAXIMUM_ALLOWED) *access = mapping.all;
         return TRUE;
     }
 
-    mapping.GenericRead  = obj->ops->map_access( obj, GENERIC_READ );
-    mapping.GenericWrite = obj->ops->map_access( obj, GENERIC_WRITE );
-    mapping.GenericExecute = obj->ops->map_access( obj, GENERIC_EXECUTE );
+    mapping.read  = obj->ops->map_access( obj, GENERIC_READ );
+    mapping.write = obj->ops->map_access( obj, GENERIC_WRITE );
+    mapping.exec = obj->ops->map_access( obj, GENERIC_EXECUTE );
 
     res = token_access_check( token, obj->sd, *access, NULL, NULL,
                               &mapping, access, &status ) == STATUS_SUCCESS &&
@@ -1374,7 +1368,8 @@ DECL_HANDLER(duplicate_token)
         struct token *token = token_duplicate( src_token, req->primary, req->impersonation_level, sd, NULL, 0, NULL, 0 );
         if (token)
         {
-            reply->new_handle = alloc_handle_no_access_check( current->process, token, req->access, objattr->attributes );
+            unsigned int access = req->access ? req->access : get_handle_access( current->process, req->handle );
+            reply->new_handle = alloc_handle_no_access_check( current->process, token, access, objattr->attributes );
             release_object( token );
         }
         release_object( src_token );
@@ -1451,7 +1446,6 @@ DECL_HANDLER(access_check)
                                                  TOKEN_QUERY,
                                                  &token_ops )))
     {
-        GENERIC_MAPPING mapping;
         unsigned int status;
         LUID_AND_ATTRIBUTES priv;
         unsigned int priv_count = 1;
@@ -1473,14 +1467,8 @@ DECL_HANDLER(access_check)
             return;
         }
 
-        mapping.GenericRead = req->mapping_read;
-        mapping.GenericWrite = req->mapping_write;
-        mapping.GenericExecute = req->mapping_execute;
-        mapping.GenericAll = req->mapping_all;
-
-        status = token_access_check(
-            token, sd, req->desired_access, &priv, &priv_count, &mapping,
-            &reply->access_granted, &reply->access_status );
+        status = token_access_check( token, sd, req->desired_access, &priv, &priv_count, &req->mapping,
+                                     &reply->access_granted, &reply->access_status );
 
         reply->privileges_len = priv_count*sizeof(LUID_AND_ATTRIBUTES);
 
@@ -1606,38 +1594,20 @@ DECL_HANDLER(get_token_groups)
     }
 }
 
-DECL_HANDLER(get_token_impersonation_level)
+DECL_HANDLER(get_token_info)
 {
     struct token *token;
 
-    if ((token = (struct token *)get_handle_obj( current->process, req->handle,
-                                                 TOKEN_QUERY,
-                                                 &token_ops )))
-    {
-        if (token->primary)
-            set_error( STATUS_INVALID_PARAMETER );
-        else
-            reply->impersonation_level = token->impersonation_level;
-
-        release_object( token );
-    }
-}
-
-DECL_HANDLER(get_token_statistics)
-{
-    struct token *token;
-
-    if ((token = (struct token *)get_handle_obj( current->process, req->handle,
-                                                 TOKEN_QUERY,
-                                                 &token_ops )))
+    if ((token = (struct token *)get_handle_obj( current->process, req->handle, TOKEN_QUERY, &token_ops )))
     {
         reply->token_id = token->token_id;
         reply->modified_id = token->modified_id;
+        reply->session_id = token->session_id;
         reply->primary = token->primary;
         reply->impersonation_level = token->impersonation_level;
+        reply->elevation = token->elevation;
         reply->group_count = list_count( &token->groups );
         reply->privilege_count = list_count( &token->privileges );
-
         release_object( token );
     }
 }
@@ -1687,3 +1657,32 @@ DECL_HANDLER(set_token_default_dacl)
         release_object( token );
     }
 }
+
+DECL_HANDLER(create_linked_token)
+{
+    struct token *token, *linked;
+    int elevation;
+
+    if ((token = (struct token *)get_handle_obj( current->process, req->handle,
+                                                 TOKEN_QUERY, &token_ops )))
+    {
+        switch (token->elevation)
+        {
+        case TokenElevationTypeFull:
+            elevation = TokenElevationTypeLimited;
+            break;
+        case TokenElevationTypeLimited:
+            elevation = TokenElevationTypeFull;
+            break;
+        default:
+            release_object( token );
+            return;
+        }
+        if ((linked = token_create_admin( FALSE, SecurityIdentification, elevation, token->session_id )))
+        {
+            reply->linked = alloc_handle( current->process, linked, TOKEN_ALL_ACCESS, 0 );
+            release_object( linked );
+        }
+        release_object( token );
+    }
+}
