@@ -19,7 +19,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -27,48 +26,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-# include <sys/stat.h>
-#endif
 
 #include "build.h"
 
-#if defined(_WIN32) && !defined(__CYGWIN__)
-# define PATH_SEPARATOR ';'
-#else
-# define PATH_SEPARATOR ':'
-#endif
-
 static struct strarray tmp_files;
-static struct strarray empty_strarray;
 static const char *output_file_source_name;
 
-static const struct
-{
-    const char *name;
-    enum target_cpu cpu;
-} cpu_names[] =
-{
-    { "i386",    CPU_x86 },
-    { "i486",    CPU_x86 },
-    { "i586",    CPU_x86 },
-    { "i686",    CPU_x86 },
-    { "i786",    CPU_x86 },
-    { "amd64",   CPU_x86_64 },
-    { "x86_64",  CPU_x86_64 },
-    { "powerpc", CPU_POWERPC },
-    { "arm",     CPU_ARM },
-    { "armv5",   CPU_ARM },
-    { "armv6",   CPU_ARM },
-    { "armv7",   CPU_ARM },
-    { "armv7a",  CPU_ARM },
-    { "arm64",   CPU_ARM64 },
-    { "aarch64", CPU_ARM64 },
-};
-
 /* atexit handler to clean tmp files */
 void cleanup_tmp_files(void)
 {
@@ -77,41 +40,6 @@ void cleanup_tmp_files(void)
 }
 
 
-void *xmalloc (size_t size)
-{
-    void *res;
-
-    res = malloc (size ? size : 1);
-    if (res == NULL)
-    {
-        fprintf (stderr, "Virtual memory exhausted.\n");
-        exit (1);
-    }
-    return res;
-}
-
-void *xrealloc (void *ptr, size_t size)
-{
-    void *res = realloc (ptr, size);
-    if (size && res == NULL)
-    {
-        fprintf (stderr, "Virtual memory exhausted.\n");
-        exit (1);
-    }
-    return res;
-}
-
-char *xstrdup( const char *str )
-{
-    char *res = strdup( str );
-    if (!res)
-    {
-        fprintf (stderr, "Virtual memory exhausted.\n");
-        exit (1);
-    }
-    return res;
-}
-
 char *strupper(char *s)
 {
     char *p;
@@ -119,88 +47,6 @@ char *strupper(char *s)
     return s;
 }
 
-int strendswith(const char* str, const char* end)
-{
-    int l = strlen(str);
-    int m = strlen(end);
-    return l >= m && strcmp(str + l - m, end) == 0;
-}
-
-char *strmake( const char* fmt, ... )
-{
-    int n;
-    size_t size = 100;
-    va_list ap;
-
-    for (;;)
-    {
-        char *p = xmalloc( size );
-        va_start( ap, fmt );
-	n = vsnprintf( p, size, fmt, ap );
-	va_end( ap );
-        if (n == -1) size *= 2;
-        else if ((size_t)n >= size) size = n + 1;
-        else return p;
-        free( p );
-    }
-}
-
-static struct strarray strarray_copy( struct strarray src )
-{
-    struct strarray array;
-    array.count = src.count;
-    array.max = src.max;
-    array.str = xmalloc( array.max * sizeof(*array.str) );
-    memcpy( array.str, src.str, array.count * sizeof(*array.str) );
-    return array;
-}
-
-static void strarray_add_one( struct strarray *array, const char *str )
-{
-    if (array->count == array->max)
-    {
-        array->max *= 2;
-        if (array->max < 16) array->max = 16;
-        array->str = xrealloc( array->str, array->max * sizeof(*array->str) );
-    }
-    array->str[array->count++] = str;
-}
-
-void strarray_add( struct strarray *array, ... )
-{
-    va_list valist;
-    const char *str;
-
-    va_start( valist, array );
-    while ((str = va_arg( valist, const char *))) strarray_add_one( array, str );
-    va_end( valist );
-}
-
-void strarray_addv( struct strarray *array, char * const *argv )
-{
-    while (*argv) strarray_add_one( array, *argv++ );
-}
-
-void strarray_addall( struct strarray *array, struct strarray args )
-{
-    unsigned int i;
-
-    for (i = 0; i < args.count; i++) strarray_add_one( array, args.str[i] );
-}
-
-struct strarray strarray_fromstring( const char *str, const char *delim )
-{
-    const char *tok;
-    struct strarray array = empty_strarray;
-    char *buf = xstrdup( str );
-
-    for (tok = strtok( buf, delim ); tok; tok = strtok( NULL, delim ))
-	strarray_add_one( &array, strdup( tok ));
-
-    free( buf );
-    return array;
-}
-
 void fatal_error( const char *msg, ... )
 {
     va_list valist;
@@ -288,20 +134,8 @@ static struct strarray get_tools_path(void)
 
     if (!done)
     {
-        dirs = strarray_copy( tools_path );
-
-        /* then append the PATH directories */
-        if (getenv( "PATH" ))
-        {
-            char *p = xstrdup( getenv( "PATH" ));
-            while (*p)
-            {
-                strarray_add_one( &dirs, p );
-                while (*p && *p != PATH_SEPARATOR) p++;
-                if (!*p) break;
-                *p++ = 0;
-            }
-        }
+        strarray_addall( &dirs, tools_path );
+        strarray_addall( &dirs, strarray_frompath( getenv( "PATH" )));
         done = 1;
     }
     return dirs;
@@ -342,17 +176,13 @@ static const char *find_binary( const char *prefix, const char *name )
 
 void spawn( struct strarray args )
 {
-    unsigned int i;
     int status;
     const char *argv0 = find_binary( NULL, args.str[0] );
 
     if (argv0) args.str[0] = argv0;
-    strarray_add_one( &args, NULL );
-    if (verbose)
-        for (i = 0; args.str[i]; i++)
-            fprintf( stderr, "%s%c", args.str[i], args.str[i+1] ? ' ' : '\n' );
+    if (verbose) strarray_trace( args );
 
-    if ((status = _spawnvp( _P_WAIT, args.str[0], args.str )))
+    if ((status = strarray_spawn( args )))
     {
 	if (status > 0) fatal_error( "%s failed with status %u\n", args.str[0], status );
 	else fatal_perror( "winebuild" );
@@ -360,9 +190,50 @@ void spawn( struct strarray args )
     }
 }
 
+static const char *find_clang_tool( struct strarray clang, const char *tool )
+{
+    const char *out = get_temp_file_name( "print_tool", ".out" );
+    struct strarray args = empty_strarray;
+    int sout = -1;
+    char *path, *p;
+    struct stat st;
+    size_t cnt;
+
+    strarray_addall( &args, clang );
+    strarray_add( &args, strmake( "-print-prog-name=%s", tool ));
+    if (verbose) strarray_add( &args, "-v" );
+
+    sout = dup( fileno(stdout) );
+    freopen( out, "w", stdout );
+    spawn( args );
+    if (sout >= 0)
+    {
+        dup2( sout, fileno(stdout) );
+        close( sout );
+    }
+
+    if (stat(out, &st) || !st.st_size) return NULL;
+
+    path = xmalloc(st.st_size + 1);
+    sout = open(out, O_RDONLY);
+    if (sout == -1) return NULL;
+    cnt = read(sout, path, st.st_size);
+    close(sout);
+    path[cnt] = 0;
+    if ((p = strchr(path, '\n'))) *p = 0;
+    /* clang returns passed command instead of full path if the tool could not be found */
+    if (!strcmp(path, tool))
+    {
+        free( path );
+        return NULL;
+    }
+    return path;
+}
+
 /* find a build tool in the path, trying the various names */
 struct strarray find_tool( const char *name, const char * const *names )
 {
+    struct strarray ret = empty_strarray;
     const char *file;
     const char *alt_names[2];
 
@@ -375,74 +246,113 @@ struct strarray find_tool( const char *name, const char * const *names )
 
     while (*names)
     {
-        if ((file = find_binary( target_alias, *names ))
-            || (names == alt_names && (file = find_binary( "llvm", *names ))))
-        {
-            struct strarray ret = empty_strarray;
-            strarray_add_one( &ret, file );
-            return ret;
-        }
+        if ((file = find_binary( target_alias, *names ))) break;
         names++;
     }
-    fatal_error( "cannot find the '%s' tool\n", name );
+
+    if (!file)
+    {
+        if (cc_command.count) file = find_clang_tool( cc_command, name );
+        if (!file && !(file = find_binary( "llvm", name )))
+        {
+            struct strarray clang = empty_strarray;
+            strarray_add( &clang, "clang" );
+            file = find_clang_tool( clang, strmake( "llvm-%s", name ));
+        }
+    }
+    if (!file) fatal_error( "cannot find the '%s' tool\n", name );
+
+    strarray_add( &ret, file );
+    return ret;
+}
+
+/* find a link tool in the path */
+struct strarray find_link_tool(void)
+{
+    struct strarray ret = empty_strarray;
+    const char *file = NULL;
+
+    if (cc_command.count) file = find_clang_tool( cc_command, "lld-link" );
+    if (!file) file = find_binary( NULL, "lld-link" );
+    if (!file)
+    {
+        struct strarray clang = empty_strarray;
+        strarray_add( &clang, "clang" );
+        file = find_clang_tool( clang, "lld-link" );
+    }
+
+    if (!file) fatal_error( "cannot find the 'lld-link' tool\n" );
+    strarray_add( &ret, file );
+    return ret;
 }
 
 struct strarray get_as_command(void)
 {
-    struct strarray args;
+    struct strarray args = empty_strarray;
+    const char *file;
     unsigned int i;
+    int using_cc = 0;
 
     if (cc_command.count)
     {
-        args = strarray_copy( cc_command );
-        strarray_add( &args, "-xassembler", "-c", NULL );
+        strarray_addall( &args, cc_command );
+        using_cc = 1;
+    }
+    else if (as_command.count)
+    {
+        strarray_addall( &args, as_command );
+    }
+    else if ((file = find_binary( target_alias, "as" )) || (file = find_binary( target_alias, "gas ")))
+    {
+        strarray_add( &args, file );
+    }
+    else if ((file = find_binary( NULL, "clang" )))
+    {
+        strarray_add( &args, file );
+        if (target_alias)
+        {
+            strarray_add( &args, "-target" );
+            strarray_add( &args, target_alias );
+        }
+        using_cc = 1;
+    }
+
+    if (using_cc)
+    {
+        strarray_add( &args, "-xassembler" );
+        strarray_add( &args, "-c" );
         if (force_pointer_size)
-            strarray_add_one( &args, (force_pointer_size == 8) ? "-m64" : "-m32" );
-        if (cpu_option) strarray_add_one( &args, strmake("-mcpu=%s", cpu_option) );
-        if (fpu_option) strarray_add_one( &args, strmake("-mfpu=%s", fpu_option) );
-        if (arch_option) strarray_add_one( &args, strmake("-march=%s", arch_option) );
+            strarray_add( &args, (force_pointer_size == 8) ? "-m64" : "-m32" );
+        if (cpu_option) strarray_add( &args, strmake("-mcpu=%s", cpu_option) );
+        if (fpu_option) strarray_add( &args, strmake("-mfpu=%s", fpu_option) );
+        if (arch_option) strarray_add( &args, strmake("-march=%s", arch_option) );
         for (i = 0; i < tools_path.count; i++)
-            strarray_add_one( &args, strmake("-B%s", tools_path.str[i] ));
+            strarray_add( &args, strmake("-B%s", tools_path.str[i] ));
         return args;
     }
 
-    if (!as_command.count)
-    {
-        static const char * const commands[] = { "gas", "as", NULL };
-        as_command = find_tool( "as", commands );
-    }
-
-    args = strarray_copy( as_command );
-
     if (force_pointer_size)
     {
-        switch (target_platform)
+        switch (target.platform)
         {
         case PLATFORM_APPLE:
-            strarray_add( &args, "-arch", (force_pointer_size == 8) ? "x86_64" : "i386", NULL );
+            strarray_add( &args, "-arch" );
+            strarray_add( &args, (force_pointer_size == 8) ? "x86_64" : "i386" );
             break;
         default:
-            switch(target_cpu)
-            {
-            case CPU_POWERPC:
-                strarray_add_one( &args, (force_pointer_size == 8) ? "-a64" : "-a32" );
-                break;
-            default:
-                strarray_add_one( &args, (force_pointer_size == 8) ? "--64" : "--32" );
-                break;
-            }
+            strarray_add( &args, (force_pointer_size == 8) ? "--64" : "--32" );
             break;
         }
     }
 
-    if (cpu_option) strarray_add_one( &args, strmake("-mcpu=%s", cpu_option) );
-    if (fpu_option) strarray_add_one( &args, strmake("-mfpu=%s", fpu_option) );
+    if (cpu_option) strarray_add( &args, strmake("-mcpu=%s", cpu_option) );
+    if (fpu_option) strarray_add( &args, strmake("-mfpu=%s", fpu_option) );
     return args;
 }
 
 struct strarray get_ld_command(void)
 {
-    struct strarray args;
+    struct strarray args = empty_strarray;
 
     if (!ld_command.count)
     {
@@ -450,37 +360,34 @@ struct strarray get_ld_command(void)
         ld_command = find_tool( "ld", commands );
     }
 
-    args = strarray_copy( ld_command );
+    strarray_addall( &args, ld_command );
 
     if (force_pointer_size)
     {
-        switch (target_platform)
+        switch (target.platform)
         {
         case PLATFORM_APPLE:
-            strarray_add( &args, "-arch", (force_pointer_size == 8) ? "x86_64" : "i386", NULL );
+            strarray_add( &args, "-arch" );
+            strarray_add( &args, (force_pointer_size == 8) ? "x86_64" : "i386" );
             break;
         case PLATFORM_FREEBSD:
-            strarray_add( &args, "-m", (force_pointer_size == 8) ? "elf_x86_64_fbsd" : "elf_i386_fbsd", NULL );
+            strarray_add( &args, "-m" );
+            strarray_add( &args, (force_pointer_size == 8) ? "elf_x86_64_fbsd" : "elf_i386_fbsd" );
             break;
+        case PLATFORM_MINGW:
         case PLATFORM_WINDOWS:
-            strarray_add( &args, "-m", (force_pointer_size == 8) ? "i386pep" : "i386pe", NULL );
+            strarray_add( &args, "-m" );
+            strarray_add( &args, (force_pointer_size == 8) ? "i386pep" : "i386pe" );
             break;
         default:
-            switch(target_cpu)
-            {
-            case CPU_POWERPC:
-                strarray_add( &args, "-m", (force_pointer_size == 8) ? "elf64ppc" : "elf32ppc", NULL );
-                break;
-            default:
-                strarray_add( &args, "-m", (force_pointer_size == 8) ? "elf_x86_64" : "elf_i386", NULL );
-                break;
-            }
+            strarray_add( &args, "-m" );
+            strarray_add( &args, (force_pointer_size == 8) ? "elf_x86_64" : "elf_i386" );
             break;
         }
     }
 
-    if (target_cpu == CPU_ARM && target_platform != PLATFORM_WINDOWS)
-        strarray_add( &args, "--no-wchar-size-warning", NULL );
+    if (target.cpu == CPU_ARM && !is_pe())
+        strarray_add( &args, "--no-wchar-size-warning" );
 
     return args;
 }
@@ -501,31 +408,12 @@ const char *get_nm_command(void)
 char *get_temp_file_name( const char *prefix, const char *suffix )
 {
     char *name;
-    const char *ext, *basename;
     int fd;
 
-    if (!prefix || !prefix[0]) prefix = "winebuild";
-    if (!suffix) suffix = "";
-    if ((basename = strrchr( prefix, '/' ))) basename++;
-    else basename = prefix;
-    if (!(ext = strchr( basename, '.' ))) ext = prefix + strlen(prefix);
-    name = xmalloc( sizeof("/tmp/") + (ext - prefix) + sizeof(".XXXXXX") + strlen(suffix) );
-    memcpy( name, prefix, ext - prefix );
-    strcpy( name + (ext - prefix), ".XXXXXX" );
-    strcat( name, suffix );
-
-    if ((fd = mkstemps( name, strlen(suffix) )) == -1)
-    {
-        strcpy( name, "/tmp/" );
-        memcpy( name + 5, basename, ext - basename );
-        strcpy( name + 5 + (ext - basename), ".XXXXXX" );
-        strcat( name, suffix );
-        if ((fd = mkstemps( name, strlen(suffix) )) == -1)
-            fatal_error( "could not generate a temp file\n" );
-    }
-
+    if (prefix) prefix = get_basename_noext( prefix );
+    fd = make_temp_file( prefix, suffix, &name );
     close( fd );
-    strarray_add_one( &tmp_files, name );
+    strarray_add( &tmp_files, name );
     return name;
 }
 
@@ -544,15 +432,6 @@ unsigned char *output_buffer;
 size_t output_buffer_pos;
 size_t output_buffer_size;
 
-static void check_output_buffer_space( size_t size )
-{
-    if (output_buffer_pos + size >= output_buffer_size)
-    {
-        output_buffer_size = max( output_buffer_size * 2, output_buffer_pos + size );
-        output_buffer = xrealloc( output_buffer, output_buffer_size );
-    }
-}
-
 void init_input_buffer( const char *file )
 {
     int fd;
@@ -571,22 +450,6 @@ void init_input_buffer( const char *file )
     byte_swapped = 0;
 }
 
-void init_output_buffer(void)
-{
-    output_buffer_size = 1024;
-    output_buffer_pos = 0;
-    output_buffer = xmalloc( output_buffer_size );
-}
-
-void flush_output_buffer(void)
-{
-    open_output_file();
-    if (fwrite( output_buffer, 1, output_buffer_pos, output_file ) != output_buffer_pos)
-        fatal_error( "Error writing to %s\n", output_file_name );
-    close_output_file();
-    free( output_buffer );
-}
-
 unsigned char get_byte(void)
 {
     if (input_buffer_pos >= input_buffer_size)
@@ -619,46 +482,6 @@ unsigned int get_dword(void)
     return ret;
 }
 
-void put_data( const void *data, size_t size )
-{
-    check_output_buffer_space( size );
-    memcpy( output_buffer + output_buffer_pos, data, size );
-    output_buffer_pos += size;
-}
-
-void put_byte( unsigned char val )
-{
-    check_output_buffer_space( 1 );
-    output_buffer[output_buffer_pos++] = val;
-}
-
-void put_word( unsigned short val )
-{
-    if (byte_swapped) val = (val << 8) | (val >> 8);
-    put_data( &val, sizeof(val) );
-}
-
-void put_dword( unsigned int val )
-{
-    if (byte_swapped)
-        val = ((val << 24) | ((val << 8) & 0x00ff0000) | ((val >> 8) & 0x0000ff00) | (val >> 24));
-    put_data( &val, sizeof(val) );
-}
-
-void put_qword( unsigned int val )
-{
-    if (byte_swapped)
-    {
-        put_dword( 0 );
-        put_dword( val );
-    }
-    else
-    {
-        put_dword( val );
-        put_dword( 0 );
-    }
-}
-
 /* pointer-sized word */
 void put_pword( unsigned int val )
 {
@@ -666,16 +489,6 @@ void put_pword( unsigned int val )
     else put_dword( val );
 }
 
-void align_output( unsigned int align )
-{
-    size_t size = align - (output_buffer_pos % align);
-
-    if (size == align) return;
-    check_output_buffer_space( size );
-    memset( output_buffer + output_buffer_pos, 0, size );
-    output_buffer_pos += size;
-}
-
 /* output a standard header for generated files */
 void output_standard_file_header(void)
 {
@@ -690,6 +503,11 @@ void output_standard_file_header(void)
         output( "\t.globl  @feat.00\n" );
         output( ".set @feat.00, 1\n" );
     }
+    if (thumb_mode)
+    {
+        output( "\t.syntax unified\n" );
+        output( "\t.thumb\n" );
+    }
 }
 
 /* dump a byte stream into the assembly code */
@@ -798,7 +616,7 @@ int remove_stdcall_decoration( char *name )
 {
     char *p, *end = strrchr( name, '@' );
     if (!end || !end[1] || end == name) return -1;
-    if (target_cpu != CPU_x86) return -1;
+    if (target.cpu != CPU_i386) return -1;
     /* make sure all the rest is digits */
     for (p = end + 1; *p; p++) if (!isdigit(*p)) return -1;
     *end = 0;
@@ -814,7 +632,9 @@ int remove_stdcall_decoration( char *name )
 void assemble_file( const char *src_file, const char *obj_file )
 {
     struct strarray args = get_as_command();
-    strarray_add( &args, "-o", obj_file, src_file, NULL );
+    strarray_add( &args, "-o" );
+    strarray_add( &args, obj_file );
+    strarray_add( &args, src_file );
     spawn( args );
 }
 
@@ -836,6 +656,7 @@ DLLSPEC *alloc_dll_spec(void)
     spec->subsystem          = 0;
     spec->subsystem_major    = 4;
     spec->subsystem_minor    = 0;
+    spec->syscall_table      = 0;
     if (get_ptr_size() > 4)
         spec->characteristics |= IMAGE_FILE_LARGE_ADDRESS_AWARE;
     else
@@ -921,12 +742,12 @@ const char *get_link_name( const ORDDEF *odp )
     static char *buffer;
     char *ret;
 
-    if (target_cpu != CPU_x86) return odp->link_name;
+    if (target.cpu != CPU_i386) return odp->link_name;
 
     switch (odp->type)
     {
     case TYPE_STDCALL:
-        if (target_platform == PLATFORM_WINDOWS)
+        if (is_pe())
         {
             if (odp->flags & FLAG_THISCALL) return odp->link_name;
             if (odp->flags & FLAG_FASTCALL) ret = strmake( "@%s@%u", odp->link_name, get_args_size( odp ));
@@ -942,7 +763,7 @@ const char *get_link_name( const ORDDEF *odp )
         break;
 
     case TYPE_PASCAL:
-        if (target_platform == PLATFORM_WINDOWS && !kill_at)
+        if (is_pe() && !kill_at)
         {
             int args = get_args_size( odp );
             if (odp->flags & FLAG_REGISTER) args += get_ptr_size();  /* context argument */
@@ -976,16 +797,6 @@ int sort_func_list( ORDDEF **list, int count, int (*compare)(const void *, const
 }
 
 
-/* parse a cpu name and return the corresponding value */
-int get_cpu_from_name( const char *name )
-{
-    unsigned int i;
-
-    for (i = 0; i < ARRAY_SIZE(cpu_names); i++)
-        if (!strcmp( cpu_names[i].name, name )) return cpu_names[i].cpu;
-    return -1;
-}
-
 /*****************************************************************
  *  Function:    get_alignment
  *
@@ -1016,13 +827,12 @@ unsigned int get_alignment(unsigned int align)
 
     assert( !(align & (align - 1)) );
 
-    switch(target_cpu)
+    switch (target.cpu)
     {
-    case CPU_x86:
+    case CPU_i386:
     case CPU_x86_64:
-        if (target_platform != PLATFORM_APPLE) return align;
+        if (target.platform != PLATFORM_APPLE) return align;
         /* fall through */
-    case CPU_POWERPC:
     case CPU_ARM:
     case CPU_ARM64:
         n = 0;
@@ -1040,24 +850,6 @@ unsigned int get_page_size(void)
     return 0x1000;  /* same on all platforms */
 }
 
-/* return the size of a pointer on the target CPU */
-unsigned int get_ptr_size(void)
-{
-    switch(target_cpu)
-    {
-    case CPU_x86:
-    case CPU_POWERPC:
-    case CPU_ARM:
-        return 4;
-    case CPU_x86_64:
-    case CPU_ARM64:
-        return 8;
-    }
-    /* unreached */
-    assert(0);
-    return 0;
-}
-
 /* return the total size in bytes of the arguments on the stack */
 unsigned int get_args_size( const ORDDEF *odp )
 {
@@ -1069,11 +861,12 @@ unsigned int get_args_size( const ORDDEF *odp )
         {
         case ARG_INT64:
         case ARG_DOUBLE:
+            if (target.cpu == CPU_ARM) size = (size + 7) & ~7;
             size += 8;
             break;
         case ARG_INT128:
             /* int128 is passed as pointer on x86_64 */
-            if (target_cpu != CPU_x86_64)
+            if (target.cpu != CPU_x86_64)
             {
                 size += 16;
                 break;
@@ -1092,10 +885,11 @@ const char *asm_name( const char *sym )
 {
     static char *buffer;
 
-    switch (target_platform)
+    switch (target.platform)
     {
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
-        if (target_cpu != CPU_x86) return sym;
+        if (target.cpu != CPU_i386) return sym;
         if (sym[0] == '@') return sym;  /* fastcall */
         /* fall through */
     case PLATFORM_APPLE:
@@ -1113,19 +907,24 @@ const char *func_declaration( const char *func )
 {
     static char *buffer;
 
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
         return "";
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
         free( buffer );
-        buffer = strmake( ".def %s\n\t.scl 2\n\t.type 32\n\t.endef", asm_name(func) );
+        buffer = strmake( ".def %s\n\t.scl 2\n\t.type 32\n\t.endef%s", asm_name(func),
+                          thumb_mode ? "\n\t.thumb_func" : "" );
         break;
     default:
         free( buffer );
-        switch(target_cpu)
+        switch (target.cpu)
         {
         case CPU_ARM:
+            buffer = strmake( ".type %s,%%function%s", func,
+                              thumb_mode ? "\n\t.thumb_func" : "" );
+            break;
         case CPU_ARM64:
             buffer = strmake( ".type %s,%%function", func );
             break;
@@ -1141,9 +940,10 @@ const char *func_declaration( const char *func )
 /* output a size declaration for an assembly function */
 void output_function_size( const char *name )
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
         break;
     default:
@@ -1171,8 +971,9 @@ void output_rva( const char *format, ... )
     va_list valist;
 
     va_start( valist, format );
-    switch (target_platform)
+    switch (target.platform)
     {
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
         output( "\t.rva " );
         vfprintf( output_file, format, valist );
@@ -1190,13 +991,14 @@ void output_rva( const char *format, ... )
 /* output the GNU note for non-exec stack */
 void output_gnu_stack_note(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
     case PLATFORM_APPLE:
         break;
     default:
-        switch(target_cpu)
+        switch (target.cpu)
         {
         case CPU_ARM:
         case CPU_ARM64:
@@ -1216,14 +1018,15 @@ const char *asm_globl( const char *func )
     static char *buffer;
 
     free( buffer );
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
         buffer = strmake( "\t.globl _%s\n\t.private_extern _%s\n_%s:", func, func, func );
         break;
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
-        buffer = strmake( "\t.globl %s%s\n%s%s:", target_cpu == CPU_x86 ? "_" : "", func,
-                          target_cpu == CPU_x86 ? "_" : "", func );
+        buffer = strmake( "\t.globl %s%s\n%s%s:", target.cpu == CPU_i386 ? "_" : "", func,
+                          target.cpu == CPU_i386 ? "_" : "", func );
         break;
     default:
         buffer = strmake( "\t.globl %s\n\t.hidden %s\n%s:", func, func, func );
@@ -1245,7 +1048,7 @@ const char *get_asm_ptr_keyword(void)
 
 const char *get_asm_string_keyword(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
         return ".asciz";
@@ -1256,9 +1059,10 @@ const char *get_asm_string_keyword(void)
 
 const char *get_asm_export_section(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:   return ".data";
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS: return ".section .edata";
     default:               return ".section .data";
     }
@@ -1266,7 +1070,7 @@ const char *get_asm_export_section(void)
 
 const char *get_asm_rodata_section(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE: return ".const";
     default:             return ".section .rodata";
@@ -1275,9 +1079,10 @@ const char *get_asm_rodata_section(void)
 
 const char *get_asm_rsrc_section(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:   return ".data";
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS: return ".section .rsrc";
     default:               return ".section .data";
     }
@@ -1285,7 +1090,7 @@ const char *get_asm_rsrc_section(void)
 
 const char *get_asm_string_section(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE: return ".cstring";
     default:             return ".section .rodata";
@@ -1296,7 +1101,7 @@ const char *arm64_page( const char *sym )
 {
     static char *buffer;
 
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
         free( buffer );
@@ -1312,7 +1117,7 @@ const char *arm64_pageoff( const char *sym )
     static char *buffer;
 
     free( buffer );
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
         buffer = strmake( "%s@PAGEOFF", sym );
